@@ -12,23 +12,18 @@
 #undef HAVE_LDAP
 #undef HAVE_TIMEZONE
 #undef HAVE_TM_GMTOFF
-#undef USING_OAF
 #undef HAVE_KRB4
+#undef NEED_KRB_SENDAUTH_PROTO
 #undef HAVE_KRB5
+#undef HAVE_NSS
 #undef MOVEMAIL_PATH
-#undef USING_GNOME_PRINT_0_20
 #undef USE_DOT
 #undef USE_FCNTL
 #undef USE_FLOCK
-#undef NEED_KRB_SENDAUTH_PROTO
-#undef HAVE_ISBLANK
 #undef ENABLE_NNTP
 
 /* Sub-version identification string.  */
 #undef SUB_VERSION
 
-/* Define this if you want to build against the development gtk */
-#undef HAVE_DEVGTK
-
 /* Define if ctime_r takes three arguments */
 #undef CTIME_R_THREE_ARGS
