@@ -11,6 +11,7 @@
 #include <config.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkmarshal.h>
+#include <libgnome/gnome-defs.h>
 #include <liboaf/liboaf.h>
 
 #include "addressbook.h"
@@ -42,10 +43,13 @@ struct _EBookPrivate {
 	 * the op structures are popped off the front of the queue.
 	 */
 	GList                 *pending_ops;
+
+	guint op_tag;
 };
 
 enum {
 	OPEN_PROGRESS,
+	WRITABLE_STATUS,
 	LINK_STATUS,
 	LAST_SIGNAL
 };
@@ -53,6 +57,8 @@ enum {
 static guint e_book_signals [LAST_SIGNAL];
 
 typedef struct {
+	guint     tag;
+	gboolean  active;
 	gpointer  cb;
 	gpointer  closure;
 	EBookViewListener *listener;
@@ -61,7 +67,7 @@ typedef struct {
 /*
  * Local response queue management.
  */
-static void
+static guint
 e_book_queue_op (EBook    *book,
 		 gpointer  cb,
 		 gpointer  closure,
@@ -70,12 +76,16 @@ e_book_queue_op (EBook    *book,
 	EBookOp *op;
 
 	op           = g_new0 (EBookOp, 1);
+	op->tag      = book->priv->op_tag++;
+	op->active   = TRUE;
 	op->cb       = cb;
 	op->closure  = closure;
 	op->listener = listener;
 
 	book->priv->pending_ops =
 		g_list_append (book->priv->pending_ops, op);
+
+	return op->tag;
 }
 
 static EBookOp *
@@ -99,6 +109,23 @@ e_book_pop_op (EBook *book)
 	return op;
 }
 
+static gboolean
+e_book_cancel_op (EBook *book, guint tag)
+{
+	GList *iter;
+	gboolean cancelled = FALSE;
+
+	for (iter = book->priv->pending_ops; iter != NULL && !cancelled; iter = g_list_next (iter)) {
+		EBookOp *op = iter->data;
+		if (op->tag == tag) {
+			op->active = FALSE;
+			cancelled = TRUE;
+		}
+	}
+	
+	return cancelled;
+}
+
 static void
 e_book_do_response_create_card (EBook                 *book,
 				EBookListenerResponse *resp)
@@ -156,24 +183,19 @@ e_book_do_response_get_cursor (EBook                 *book,
 
 	cursor = e_card_cursor_new(resp->cursor);
 
-	if (op->cb)
-		((EBookCursorCallback) op->cb) (book, resp->status, cursor, op->closure);
+	if (op->cb) {
+		if (op->active)
+			((EBookCursorCallback) op->cb) (book, resp->status, cursor, op->closure);
+		else
+			((EBookCursorCallback) op->cb) (book, E_BOOK_STATUS_CANCELLED, NULL, op->closure);
+	}
 
 	/*
 	 * Release the remote GNOME_Evolution_Addressbook_Book in the PAS.
 	 */
 	CORBA_exception_init (&ev);
 
-	Bonobo_Unknown_unref  (resp->cursor, &ev);
-
-	if (ev._major != CORBA_NO_EXCEPTION) {
-		g_warning ("e_book_do_response_get_cursor: Exception unref'ing "
-			   "remote GNOME_Evolution_Addressbook_CardCursor interface!\n");
-		CORBA_exception_free (&ev);
-		CORBA_exception_init (&ev);
-	}
-	
-	CORBA_Object_release (resp->cursor, &ev);
+	bonobo_object_release_unref (resp->cursor, &ev);
 
 	if (ev._major != CORBA_NO_EXCEPTION) {
 		g_warning ("e_book_do_response_get_cursor: Exception releasing "
@@ -204,27 +226,25 @@ e_book_do_response_get_view (EBook                 *book,
 			   "in local op queue!\n");
 		return;
 	}
-
-	book_view = e_book_view_new(resp->book_view, op->listener);
 	
-	if (op->cb)
-		((EBookBookViewCallback) op->cb) (book, resp->status, book_view, op->closure);
+	book_view = e_book_view_new(resp->book_view, op->listener);
 
+	/* Only execute the callback if the operation is still flagged as active (i.e. hasn't
+	   been cancelled.  This is mildly wasteful since we unnecessaryily create the
+	   book_view, etc... but I'm leery of tinkering with the CORBA magic. */
+	if (op->cb) {
+		if (op->active)
+			((EBookBookViewCallback) op->cb) (book, resp->status, book_view, op->closure);
+		else
+			((EBookBookViewCallback) op->cb) (book, E_BOOK_STATUS_CANCELLED, NULL, op->closure);
+	}
+	
 	/*
 	 * Release the remote GNOME_Evolution_Addressbook_Book in the PAS.
 	 */
 	CORBA_exception_init (&ev);
 
-	Bonobo_Unknown_unref  (resp->book_view, &ev);
-
-	if (ev._major != CORBA_NO_EXCEPTION) {
-		g_warning ("e_book_do_response_get_view: Exception unref'ing "
-			   "remote GNOME_Evolution_Addressbook_BookView interface!\n");
-		CORBA_exception_free (&ev);
-		CORBA_exception_init (&ev);
-	}
-	
-	CORBA_Object_release (resp->book_view, &ev);
+	bonobo_object_release_unref  (resp->book_view, &ev);
 
 	if (ev._major != CORBA_NO_EXCEPTION) {
 		g_warning ("e_book_do_response_get_view: Exception releasing "
@@ -257,24 +277,19 @@ e_book_do_response_get_changes (EBook                 *book,
 
 	book_view = e_book_view_new (resp->book_view, op->listener);
 	
-	if (op->cb)
-		((EBookBookViewCallback) op->cb) (book, resp->status, book_view, op->closure);
+	if (op->cb) {
+		if (op->active)
+			((EBookBookViewCallback) op->cb) (book, resp->status, book_view, op->closure);
+		else
+			((EBookBookViewCallback) op->cb) (book, E_BOOK_STATUS_CANCELLED, NULL, op->closure);
+	}
 
 	/*
 	 * Release the remote GNOME_Evolution_Addressbook_BookView in the PAS.
 	 */
 	CORBA_exception_init (&ev);
 
-	Bonobo_Unknown_unref  (resp->book_view, &ev);
-
-	if (ev._major != CORBA_NO_EXCEPTION) {
-		g_warning ("e_book_do_response_get_changes: Exception unref'ing "
-			   "remote GNOME_Evolution_Addressbook_BookView interface!\n");
-		CORBA_exception_free (&ev);
-		CORBA_exception_init (&ev);
-	}
-	
-	CORBA_Object_release (resp->book_view, &ev);
+	bonobo_object_release_unref  (resp->book_view, &ev);
 
 	if (ev._major != CORBA_NO_EXCEPTION) {
 		g_warning ("e_book_do_response_get_changes: Exception releasing "
@@ -331,6 +346,37 @@ e_book_do_link_event (EBook                 *book,
 			 resp->connected);
 }
 
+static void
+e_book_do_writable_event (EBook                 *book,
+			  EBookListenerResponse *resp)
+{
+	gtk_signal_emit (GTK_OBJECT (book), e_book_signals [WRITABLE_STATUS],
+			 resp->writable);
+}
+
+static void
+e_book_do_response_get_supported_fields (EBook                 *book,
+					 EBookListenerResponse *resp)
+{
+	EBookOp *op;
+
+	op = e_book_pop_op (book);
+
+	if (op == NULL) {
+		g_warning ("e_book_do_response_get_supported_fields: Cannot find operation "
+			   "in local op queue!\n");
+		return;
+	}
+
+	if (op->cb) {
+		if (op->active)
+			((EBookFieldsCallback) op->cb) (book, resp->status, resp->fields, op->closure);
+		else
+			((EBookFieldsCallback) op->cb) (book, E_BOOK_STATUS_CANCELLED, NULL, op->closure);
+	}
+
+	g_free (op);
+}
 
 /*
  * Reading notices out of the EBookListener's queue.
@@ -366,6 +412,9 @@ e_book_check_listener_queue (EBookListener *listener, EBook *book)
 	case OpenBookResponse:
 		e_book_do_response_open (book, resp);
 		break;
+	case GetSupportedFieldsResponse:
+		e_book_do_response_get_supported_fields (book, resp);
+		break;
 
 	case OpenProgressEvent:
 		e_book_do_progress_event (book, resp);
@@ -373,6 +422,9 @@ e_book_check_listener_queue (EBookListener *listener, EBook *book)
 	case LinkStatusEvent:
 		e_book_do_link_event (book, resp);
 		break;
+	case WritableStatusEvent:
+		e_book_do_writable_event (book, resp);
+		break;
 	default:
 		g_error ("EBook: Unknown operation %d in listener queue!\n",
 			 resp->op);
@@ -467,16 +519,7 @@ e_book_unload_uri (EBook *book)
 	 */
 	CORBA_exception_init (&ev);
 
-	Bonobo_Unknown_unref  (book->priv->corba_book, &ev);
-
-	if (ev._major != CORBA_NO_EXCEPTION) {
-		g_warning ("e_book_unload_uri: Exception unref'ing "
-			   "remote GNOME_Evolution_Addressbook_Book interface!\n");
-		CORBA_exception_free (&ev);
-		CORBA_exception_init (&ev);
-	}
-	
-	CORBA_Object_release (book->priv->corba_book, &ev);
+	bonobo_object_release_unref  (book->priv->corba_book, &ev);
 
 	if (ev._major != CORBA_NO_EXCEPTION) {
 		g_warning ("e_book_unload_uri: Exception releasing "
@@ -522,6 +565,34 @@ e_book_get_static_capabilities (EBook *book)
 	return ret_val;
 }
 
+guint
+e_book_get_supported_fields (EBook              *book,
+			     EBookFieldsCallback cb,
+			     gpointer            closure)
+{
+	CORBA_Environment ev;
+
+	CORBA_exception_init (&ev);
+
+	if (book->priv->load_state != URILoaded) {
+		g_warning ("e_book_unload_uri: No URI is loaded!\n");
+		return 0;
+	}
+
+	GNOME_Evolution_Addressbook_Book_getSupportedFields(book->priv->corba_book, &ev);
+
+	if (ev._major != CORBA_NO_EXCEPTION) {
+		g_warning ("e_book_get_supported_fields: Exception "
+			   "during get_supported_fields!\n");
+		CORBA_exception_free (&ev);
+		return 0;
+	}
+
+	CORBA_exception_free (&ev);
+
+	return e_book_queue_op (book, cb, closure, NULL);
+}
+
 static gboolean
 e_book_construct (EBook *book)
 {
@@ -933,19 +1004,20 @@ e_book_check_connection (EBook *book)
 	return TRUE;
 }
 
-gboolean e_book_get_cursor       (EBook               *book,
-				  gchar               *query,
-				  EBookCursorCallback  cb,
-				  gpointer             closure)
+guint
+e_book_get_cursor       (EBook               *book,
+			 gchar               *query,
+			 EBookCursorCallback  cb,
+			 gpointer             closure)
 {
 	CORBA_Environment ev;
   
-	g_return_val_if_fail (book != NULL,     FALSE);
-	g_return_val_if_fail (E_IS_BOOK (book), FALSE);
+	g_return_val_if_fail (book != NULL,     0);
+	g_return_val_if_fail (E_IS_BOOK (book), 0);
 
 	if (book->priv->load_state != URILoaded) {
 		g_warning ("e_book_check_connection: No URI loaded!\n");
-		return FALSE;
+		return 0;
 	}
 	
 	CORBA_exception_init (&ev);
@@ -956,30 +1028,31 @@ gboolean e_book_get_cursor       (EBook               *book,
 		g_warning ("e_book_get_all_cards: Exception "
 			   "querying list of cards!\n");
 		CORBA_exception_free (&ev);
-		return FALSE;
+		return 0;
 	}
 	
 	CORBA_exception_free (&ev);
 
-	e_book_queue_op (book, cb, closure, NULL);
+	return e_book_queue_op (book, cb, closure, NULL);
 
 	return TRUE;
 }
 
-gboolean e_book_get_book_view       (EBook                 *book,
-				     gchar                 *query,
-				     EBookBookViewCallback  cb,
-				     gpointer               closure)
+guint
+e_book_get_book_view       (EBook                 *book,
+			    gchar                 *query,
+			    EBookBookViewCallback  cb,
+			    gpointer               closure)
 {
 	CORBA_Environment ev;
 	EBookViewListener *listener;
   
-	g_return_val_if_fail (book != NULL,     FALSE);
-	g_return_val_if_fail (E_IS_BOOK (book), FALSE);
+	g_return_val_if_fail (book != NULL,     0);
+	g_return_val_if_fail (E_IS_BOOK (book), 0);
 
 	if (book->priv->load_state != URILoaded) {
 		g_warning ("e_book_get_book_view: No URI loaded!\n");
-		return FALSE;
+		return 0;
 	}
 
 	listener = e_book_view_listener_new();
@@ -992,26 +1065,25 @@ gboolean e_book_get_book_view       (EBook                 *book,
 		g_warning ("e_book_get_book_view: Exception "
 			   "getting book_view!\n");
 		CORBA_exception_free (&ev);
-		return FALSE;
+		return 0;
 	}
 	
 	CORBA_exception_free (&ev);
 
-	e_book_queue_op (book, cb, closure, listener);
-
-	return TRUE;
+	return e_book_queue_op (book, cb, closure, listener);
 }
 
-gboolean e_book_get_changes         (EBook                 *book,
-				     gchar                 *changeid,
-				     EBookBookViewCallback  cb,
-				     gpointer               closure)
+guint
+e_book_get_changes         (EBook                 *book,
+			    gchar                 *changeid,
+			    EBookBookViewCallback  cb,
+			    gpointer               closure)
 {
 	CORBA_Environment ev;
 	EBookViewListener *listener;
   
-	g_return_val_if_fail (book != NULL,     FALSE);
-	g_return_val_if_fail (E_IS_BOOK (book), FALSE);
+	g_return_val_if_fail (book != NULL,     0);
+	g_return_val_if_fail (E_IS_BOOK (book), 0);
 
 	if (book->priv->load_state != URILoaded) {
 		g_warning ("e_book_get_changes: No URI loaded!\n");
@@ -1028,14 +1100,31 @@ gboolean e_book_get_changes         (EBook                 *book,
 		g_warning ("e_book_changes: Exception "
 			   "getting changes!\n");
 		CORBA_exception_free (&ev);
-		return FALSE;
+		return 0;
 	}
 	
 	CORBA_exception_free (&ev);
 
-	e_book_queue_op (book, cb, closure, listener);
+	return e_book_queue_op (book, cb, closure, listener);
+}
 
-	return TRUE;
+/**
+ * e_book_cancel
+ */
+
+void
+e_book_cancel (EBook *book, guint tag)
+{
+	g_return_if_fail (book != NULL);
+	g_return_if_fail (E_IS_BOOK (book));
+	g_return_if_fail (tag != 0);
+
+	/* In an attempt to be useful, we take a bit of extra care in reporting
+	   errors.  This might come in handy someday. */
+	if (tag >= book->priv->op_tag)
+		g_warning ("Attempt to cancel unassigned operation (%u)", tag);
+	else if (! e_book_cancel_op (book, tag))
+		g_warning ("Attempt to cancel unknown operation (%u)", tag); 
 }
 
 /**
@@ -1084,6 +1173,7 @@ e_book_init (EBook *book)
 {
 	book->priv             = g_new0 (EBookPrivate, 1);
 	book->priv->load_state = URINotLoaded;
+	book->priv->op_tag = 1;
 }
 
 static void
@@ -1126,6 +1216,15 @@ e_book_class_init (EBookClass *klass)
 				GTK_TYPE_NONE, 1,
 				GTK_TYPE_BOOL);
 
+	e_book_signals [WRITABLE_STATUS] =
+		gtk_signal_new ("writable_status",
+				GTK_RUN_LAST,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (EBookClass, writable_status),
+				gtk_marshal_NONE__BOOL,
+				GTK_TYPE_NONE, 1,
+				GTK_TYPE_BOOL);
+
 	gtk_object_class_add_signals (object_class, e_book_signals,
 				      LAST_SIGNAL);
 
