@@ -9,6 +9,7 @@
  */
 
 #include <config.h>
+#include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -203,6 +204,12 @@ e_card_new (char *vcard)
 		cleanVObject(vobj);
 		vobj = next;
 	}
+	if (card->name == NULL)
+		card->name = e_card_name_new();
+	if (card->file_as == NULL)
+		card->file_as = g_strdup("");
+	if (card->fname == NULL)
+		card->fname = g_strdup("");
 	return card;
 }
 
@@ -264,6 +271,8 @@ char
 
 	if ( card->fname )
 		addPropValue(vobj, VCFullNameProp, card->fname);
+	else if (card->fname)
+		addProp(vobj, VCFullNameProp);
 
 	if ( card->name && (card->name->prefix || card->name->given || card->name->additional || card->name->family || card->name->suffix) ) {
 		VObject *nameprop;
@@ -279,6 +288,8 @@ char
 		if ( card->name->suffix )
 			addPropValue(nameprop, VCNameSuffixesProp, card->name->suffix);
 	}
+	else if (card->name)
+		addProp(vobj, VCNameProp);
 
 
 	if ( card->address ) {
@@ -1348,6 +1359,154 @@ e_card_name_from_string(const char *full_name)
 	return name;
 }
 
+
+/* This *so* doesn't belong here... at least not implemented in a
+   sucky way like this.  But by getting it in here now, I can fix it
+   up w/o adding a new feature when we are in feature freeze. :-) */
+
+/* This is very Anglocentric.  Maybe it should be by locale? */
+static gchar *name_synonyms[][2] = {
+	{ "Jon", "John" },   /* Ah, the hacker's perogative */
+	{ "Jon", "Jonathan" },
+	{ "Daniel", "Dan" },
+	{ "Joseph", "Joe" },
+	{ "Robert", "Rob" },
+	{ "Robert", "Bob" },
+	{ "Richard", "Rich" },
+	{ "Richard", "Dick" },
+	{ "William", "Will" },
+	{ "William", "Bill" },
+	{ "Anthony", "Tony" },
+	{ "Steven", "Steve" },
+	{ "Michael", "Mike" },
+	{ "Douglas", "Doug" },
+	{ "Sidney", "Sid" },
+	{ "Eric", "Erik" },
+	{ "Chris", "Christopher" },
+	{ "Chris", "Christine" },
+	{ "Chris", "Christy" },
+	{ "Elizabeth", "Liz" },
+	{ "Jeff", "Geoff" },
+	{ "Jeff", "Jeffrey" },
+	{ "Jeff", "Geoffrey" },
+	{ "Jim", "James" },
+	{ "Abigal", "Abby" },
+	{ "Amanda", "Amy" },
+	{ "Amanda", "Manda" },
+	{ "Di", "Diana" },
+	{ "Di", "Diane" },
+	{ "Maxine", "Max" },
+	{ "Rebecca", "Becca" },
+	{ "Rebecca", "Becky" },
+	{ "Jennifer", "Jen" },
+	{ "Jennifer", "Jenny" },
+	/* We could go on and on... */
+	{ NULL, NULL }
+};
+	
+static gboolean
+name_fragment_match (const gchar *a, const gchar *b)
+{
+	gint i;
+	gboolean nickname_match = FALSE;
+
+	if (!g_strcasecmp (a, b))
+		return TRUE;
+
+	/* Check for nicknames.  Yes, the linear search blows. */
+	for (i=0; name_synonyms[i][0]; ++i) {
+		if (!g_strcasecmp (name_synonyms[i][1], a)) {
+			a = name_synonyms[i][0];
+			nickname_match = TRUE;
+			break;
+		}
+	}
+
+	for (i=0; name_synonyms[i][0]; ++i) {
+		if (!g_strcasecmp (name_synonyms[i][1], b)) {
+			b = name_synonyms[i][0];
+			nickname_match = TRUE;
+			break;
+		}
+	}
+
+	return nickname_match && !g_strcasecmp (a, b);
+}
+
+gboolean
+e_card_name_match_string (const ECardName *name, const gchar *str)
+{
+	gchar *cpy, *name_str;
+	gchar **strv, **namev;
+	gint i, j, match_count;
+	gboolean matched = FALSE;
+
+	g_return_val_if_fail (name != NULL, FALSE);
+	g_return_val_if_fail (str != NULL, FALSE);
+
+	cpy = g_strdup (str);
+	strv = g_strsplit (cpy, " ", 0);
+	for (i=0; strv[i]; ++i)
+		g_strstrip (strv[i]);
+
+	name_str = e_card_name_to_string (name);
+	namev = g_strsplit (name_str, " ", 0);
+	for (i=0; namev[i]; ++i)
+		g_strstrip (namev[i]);
+
+	match_count = 0;
+	i = j = 0;
+	while (strv[i] && namev[j]) {
+		gint k1, k2;
+
+		for (k1=0; strv[i+k1]; ++k1) {
+			if (name_fragment_match (strv[i+k1], namev[j]))
+				break;
+		}
+
+		for (k2=0; namev[j+k2]; ++k2) {
+			if (name_fragment_match (strv[i], namev[j+k2]))
+				break;
+		}
+
+		if (strv[i+k1] == NULL && namev[j+k2] == NULL) {
+			matched = FALSE;
+			goto cleanup_and_return;
+		}
+
+		++match_count;
+		
+		if (k1 < k2) {
+			i += k1+1;
+			++j;
+		} else if (k2 < k1) {
+			++i;
+			j += k2+1;
+		} else if (k1 == k2) {
+			i += k1+1;
+			j += k2+1;
+		}
+	}
+
+	/* This rule could be made more precise.
+	   As it is, it will say that "Joe Smith" will match the name
+	   "Joe Allen Smith" (which is good), but "de Icaza" will match
+	   either "Miguel de Icaza" as well as Miguel's shiftless
+	   brother "Roger de Icaza".  In this sort of a case, the match
+	   threshold should go up to 3. */
+	if (match_count >= 2)
+		matched = TRUE;
+
+	
+ cleanup_and_return:
+	g_free (strv);
+	g_free (cpy);
+	g_free (namev);
+	g_free (name_str);
+
+	return matched;
+}
+
 ECardArbitrary *
 e_card_arbitrary_new(void)
 {
@@ -1382,6 +1541,59 @@ e_card_arbitrary_free(ECardArbitrary *arbitrary)
 	g_free(arbitrary);
 }
 
+/* EMail matching */
+static gboolean
+e_card_email_match_single_string (const gchar *a, const gchar *b)
+{
+	const gchar *xa = NULL, *xb = NULL;
+	gboolean match = TRUE;
+
+	for (xa=a; *xa && *xa != '@'; ++xa);
+	for (xb=b; *xb && *xb != '@'; ++xb);
+
+	if (xa-a != xb-b || *xa != *xb || g_strncasecmp (a, b, xa-a))
+		return FALSE;
+
+	if (*xa == '\0')
+		return TRUE;
+	
+	/* Find the end of the string, then walk through backwards comparing.
+	   This is so that we'll match joe@foobar.com and joe@mail.foobar.com.
+	*/
+	while (*xa)
+		++xa;
+	while (*xb)
+		++xb;
+
+	while (match && *xa != '@' && *xb != '@') {
+		match = (*xa == *xb);
+		--xa;
+		--xb;
+	}
+
+	match = match && ((*xa == *xb) || (*xa == '.') || (*xb == '.'));
+
+	return match;
+}
+
+gboolean
+e_card_email_match_string (const ECard *card, const gchar *str)
+{
+	EIterator *iter;
+	
+	g_return_val_if_fail (card && E_IS_CARD (card), FALSE);
+	g_return_val_if_fail (str != NULL, FALSE);
+
+	iter = e_list_get_iterator (card->email);
+	for (e_iterator_reset (iter); e_iterator_is_valid (iter); e_iterator_next (iter)) {
+		if (e_card_email_match_single_string (e_iterator_get (iter), str))
+			return TRUE;
+	}
+	gtk_object_unref (GTK_OBJECT (iter));
+
+	return FALSE;
+}
+
 /*
  * ECard lifecycle management and vCard loading/saving.
  */
@@ -1436,12 +1648,16 @@ e_card_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 	case ARG_FILE_AS:
 		g_free(card->file_as);
 		card->file_as = g_strdup(GTK_VALUE_STRING(*arg));
+		if (card->file_as == NULL)
+			card->file_as = g_strdup("");
 		break;
 	case ARG_FULL_NAME:
 		g_free(card->fname);
 		card->fname = g_strdup(GTK_VALUE_STRING(*arg));
 		if (card->name)
 			e_card_name_free (card->name);
+		if (card->fname == NULL)
+			card->fname = g_strdup("");
 		card->name = e_card_name_from_string (card->fname);
 		break;
 	case ARG_NAME:
@@ -3052,12 +3268,12 @@ card_to_string (Card *crd)
 	
 	add_CardStrProperty_to_string (string, _("Card: "), &crd->fname);
 	if (crd->name.prop.used) {
-		addProp_to_string (string, _("\nName: "));
-		add_strProp_to_string (string, _("\n  Prefix:     "), crd->name.prefix);
-		add_strProp_to_string (string, _("\n  Given:      "), crd->name.given);
-		add_strProp_to_string (string, _("\n  Additional: "), crd->name.additional);
-		add_strProp_to_string (string, _("\n  Family:     "), crd->name.family);
-		add_strProp_to_string (string, _("\n  Suffix:     "), crd->name.suffix);
+		addProp_to_string (string, N_("\nName: "));
+		add_strProp_to_string (string, N_("\n  Prefix:     "), crd->name.prefix);
+		add_strProp_to_string (string, N_("\n  Given:      "), crd->name.given);
+		add_strProp_to_string (string, N_("\n  Additional: "), crd->name.additional);
+		add_strProp_to_string (string, N_("\n  Family:     "), crd->name.family);
+		add_strProp_to_string (string, N_("\n  Suffix:     "), crd->name.suffix);
 		g_string_append_c (string, '\n');
 	}
 	
@@ -3071,7 +3287,7 @@ card_to_string (Card *crd)
 		char *date_str;
 		
 		date_str = card_bday_str (crd->bday);
-		add_strProp_to_string (string, _("\nBirth Date: "), date_str);
+		add_strProp_to_string (string, N_("\nBirth Date: "), date_str);
 		free (date_str);
 	}
 	
@@ -3082,15 +3298,15 @@ card_to_string (Card *crd)
 			CardDelAddr *deladdr = (CardDelAddr *) node->data;
 			
 			if (deladdr->prop.used) {
-				addProp_to_string (string, _("\nAddress:"));
+				addProp_to_string (string, N_("\nAddress:"));
 				add_strAddrType (string, deladdr->type);
-				add_strProp_to_string (string, _("\n  Postal Box:  "), deladdr->po);
-				add_strProp_to_string (string, _("\n  Ext:         "),deladdr->ext);
-				add_strProp_to_string (string, _("\n  Street:      "),deladdr->street);
-				add_strProp_to_string (string, _("\n  City:        "), deladdr->city);
-				add_strProp_to_string (string, _("\n  Region:      "), deladdr->region);
-				add_strProp_to_string (string, _("\n  Postal Code: "), deladdr->code);
-				add_strProp_to_string (string, _("\n  Country:     "), deladdr->country);
+				add_strProp_to_string (string, N_("\n  Postal Box:  "), deladdr->po);
+				add_strProp_to_string (string, N_("\n  Ext:         "), deladdr->ext);
+				add_strProp_to_string (string, N_("\n  Street:      "), deladdr->street);
+				add_strProp_to_string (string, N_("\n  City:        "), deladdr->city);
+				add_strProp_to_string (string, N_("\n  Region:      "), deladdr->region);
+				add_strProp_to_string (string, N_("\n  Postal Code: "), deladdr->code);
+				add_strProp_to_string (string, N_("\n  Country:     "), deladdr->country);
 			}
 		}
 		
@@ -3103,7 +3319,7 @@ card_to_string (Card *crd)
 		for (node = crd->dellabel.l; node; node = node->next) {
 			CardDelLabel *dellabel = (CardDelLabel *) node->data;
 			
-			add_strProp_to_string (string, _("\nDelivery Label: "),
+			add_strProp_to_string (string, N_("\nDelivery Label: "),
 					    dellabel->data);
 			add_strAddrType (string, dellabel->type);
 		}
@@ -3115,10 +3331,10 @@ card_to_string (Card *crd)
 		
 		if (crd->phone.l->next) {
 			sep = "  ";
-			g_string_append (string, _("\nTelephones:\n"));
+			g_string_append (string, N_("\nTelephones:\n"));
 		} else {
 			sep = " ";
-			g_string_append (string, _("\nTelephone:"));
+			g_string_append (string, N_("\nTelephone:"));
 		}
 		
 		for (node = crd->phone.l; node; node = node->next) {
@@ -3142,10 +3358,10 @@ card_to_string (Card *crd)
 		
 		if (crd->email.l->next) {
 			sep = "  ";
-			g_string_append (string, _("\nE-mail:\n"));
+			g_string_append (string, N_("\nE-mail:\n"));
 		} else {
 			sep = " ";
-			g_string_append (string, _("\nE-mail:"));
+			g_string_append (string, N_("\nE-mail:"));
 		}
 		
 		
@@ -3164,13 +3380,13 @@ card_to_string (Card *crd)
 		  g_string_append_c (string, '\n');
 	}
 
-	add_CardStrProperty_to_string (string, _("\nMailer: "), &crd->mailer);
+	add_CardStrProperty_to_string (string, N_("\nMailer: "), &crd->mailer);
 	
 	if (crd->timezn.prop.used) {
 		char *str;
 		
 		str = card_timezn_str (crd->timezn);
-		add_strProp_to_string (string, _("\nTime Zone: "), str);
+		add_strProp_to_string (string, N_("\nTime Zone: "), str);
 		free (str);
 	}
 	
@@ -3178,11 +3394,11 @@ card_to_string (Card *crd)
 		char *str;
 		
 		str = card_geopos_str (crd->geopos);
-		add_strProp_to_string (string, _("\nGeo Location: "), str);
+		add_strProp_to_string (string, N_("\nGeo Location: "), str);
 		free (str);
 	}
 	
-        add_CardStrProperty_to_string (string, _("\nBusiness Role: "), &crd->role);
+        add_CardStrProperty_to_string (string, N_("\nBusiness Role: "), &crd->role);
 	
 /*	if (crd->logo.prop.used) {
 		addPropSizedValue (string, _ ("\nLogo: "), 
@@ -3194,17 +3410,17 @@ card_to_string (Card *crd)
 	  addstringectProp (string, card_convert_to_stringect (crd->agent));*/
 	
 	if (crd->org.prop.used) {
-		addProp_to_string (string, _("\nOrg: "));
-		add_strProp_to_string (string, _("\n  Name:  "), crd->org.name);
-		add_strProp_to_string (string, _("\n  Unit:  "), crd->org.unit1);
-		add_strProp_to_string (string, _("\n  Unit2: "), crd->org.unit2);
-		add_strProp_to_string (string, _("\n  Unit3: "), crd->org.unit3);
-		add_strProp_to_string (string, _("\n  Unit4: "), crd->org.unit4);
+		addProp_to_string (string, N_("\nOrg: "));
+		add_strProp_to_string (string, N_("\n  Name:  "), crd->org.name);
+		add_strProp_to_string (string, N_("\n  Unit:  "), crd->org.unit1);
+		add_strProp_to_string (string, N_("\n  Unit2: "), crd->org.unit2);
+		add_strProp_to_string (string, N_("\n  Unit3: "), crd->org.unit3);
+		add_strProp_to_string (string, N_("\n  Unit4: "), crd->org.unit4);
 		g_string_append_c (string, '\n');
 	}
 	
-        add_CardStrProperty_to_string (string, _("\nCategories: "), &crd->categories);
-        add_CardStrProperty_to_string (string, _("\nComment: "), &crd->comment);
+        add_CardStrProperty_to_string (string, N_("\nCategories: "), &crd->categories);
+        add_CardStrProperty_to_string (string, N_("\nComment: "), &crd->comment);
 	
 /*	if (crd->sound.prop.used) {
 		if (crd->sound.type != SOUND_PHONETIC)
@@ -3217,10 +3433,10 @@ card_to_string (Card *crd)
 		add_SoundType (string, crd->sound.type);
 	}*/
 	
-        add_CardStrProperty_to_string (string, _("\nUnique String: "), &crd->uid);
+        add_CardStrProperty_to_string (string, N_("\nUnique String: "), &crd->uid);
 	
 	if (crd->key.prop.used) {
-		add_strProp_to_string (string, _("\nPublic Key: "), crd->key.data);
+		add_strProp_to_string (string, N_("\nPublic Key: "), crd->key.data);
 		add_strKeyType (string, crd->key.type);
 	}
 	
