@@ -88,10 +88,39 @@ add_card_cb (EBook *book, EBookStatus status, const gchar *id, gpointer closure)
 	printf ("Done getting all cards.\n");	
 }
 
+static void
+get_fields_cb (EBook *book, EBookStatus status, EList *fields, gpointer closure)
+{
+	if (fields) {
+		EIterator *iter = e_list_get_iterator (fields);
+
+		printf ("Supported fields:\n");
+
+		for (; e_iterator_is_valid (iter); e_iterator_next (iter)) {
+			printf (" %s\n", (char*)e_iterator_get (iter));
+		}
+
+		gtk_object_unref(GTK_OBJECT(fields));
+	}
+	else {
+		printf ("No supported fields?\n");
+	}
+
+	e_book_add_vcard(book, cardstr, add_card_cb, NULL);
+}
+
+
+static void
+auth_user_cb (EBook *book, EBookStatus status, gpointer closure)
+{
+	printf ("user authenticated\n");
+	e_book_get_supported_fields (book, get_fields_cb, closure);
+}
+
 static void
 book_open_cb (EBook *book, EBookStatus status, gpointer closure)
 {
-	e_book_add_vcard(book, cardstr, add_card_cb, NULL);
+	e_book_authenticate_user (book, "username", "password", auth_user_cb, NULL);
 }
 
 static guint
