@@ -7,6 +7,7 @@
 
 #include <config.h>
 #include <gtk/gtksignal.h>
+#include "e-util/e-list.h"
 #include "pas-book.h"
 
 static BonoboObjectClass *pas_book_parent_class;
@@ -119,6 +120,18 @@ pas_book_queue_authenticate_user (PASBook *book,
 	pas_book_queue_request (book, req);
 }
 
+static void
+pas_book_queue_get_supported_fields (PASBook *book)
+{
+	PASRequest *req;
+
+	req     = g_new0 (PASRequest, 1);
+	req->op = GetSupportedFields;
+
+	pas_book_queue_request (book, req);
+}
+
+
 static void
 pas_book_queue_get_book_view (PASBook *book, const GNOME_Evolution_Addressbook_BookViewListener listener, const char *search)
 {
@@ -131,7 +144,7 @@ pas_book_queue_get_book_view (PASBook *book, const GNOME_Evolution_Addressbook_B
 	
 	CORBA_exception_init (&ev);
 
-	req->listener = CORBA_Object_duplicate(listener, &ev);
+	req->listener = bonobo_object_dup_ref(listener, &ev);
 
 	if (ev._major != CORBA_NO_EXCEPTION) {
 		g_warning ("pas_book_queue_get_book_view: Exception "
@@ -155,7 +168,7 @@ pas_book_queue_get_changes (PASBook *book, const GNOME_Evolution_Addressbook_Boo
 	
 	CORBA_exception_init (&ev);
 
-	req->listener = CORBA_Object_duplicate(listener, &ev);
+	req->listener = bonobo_object_dup_ref(listener, &ev);
 
 	if (ev._major != CORBA_NO_EXCEPTION) {
 		g_warning ("pas_book_queue_get_changes: Exception "
@@ -315,6 +328,15 @@ impl_GNOME_Evolution_Addressbook_Book_getStaticCapabilities (PortableServer_Serv
 	return ret_val;
 }
 
+static void
+impl_GNOME_Evolution_Addressbook_Book_getSupportedFields (PortableServer_Servant servant,
+							  CORBA_Environment *ev)
+{
+	PASBook *book = PAS_BOOK (bonobo_object_from_servant (servant));
+
+	pas_book_queue_get_supported_fields (book);
+}
+
 /**
  * pas_book_get_backend:
  */
@@ -497,6 +519,43 @@ pas_book_respond_authenticate_user (PASBook                           *book,
 	CORBA_exception_free (&ev);
 }
 
+void
+pas_book_respond_get_supported_fields (PASBook *book,
+				       GNOME_Evolution_Addressbook_BookListener_CallStatus  status,
+				       EList   *fields)
+{
+	CORBA_Environment ev;
+	GNOME_Evolution_Addressbook_stringlist stringlist;
+	int num_fields;
+	EIterator *iter;
+	int i;
+
+	CORBA_exception_init (&ev);
+
+	num_fields = e_list_length (fields);
+
+	stringlist._buffer = CORBA_sequence_CORBA_string_allocbuf (num_fields);
+	stringlist._maximum = num_fields;
+	stringlist._length = num_fields;
+
+	iter = e_list_get_iterator (fields);
+
+	for (i = 0; e_iterator_is_valid (iter); e_iterator_next (iter), i ++) {
+		stringlist._buffer[i] = CORBA_string_dup (e_iterator_get(iter));
+	}
+
+	gtk_object_unref (GTK_OBJECT (fields));
+
+	GNOME_Evolution_Addressbook_BookListener_notifySupportedFields (
+			book->priv->listener, status,
+			&stringlist,
+			&ev);
+
+	CORBA_exception_free (&ev);
+
+	CORBA_free(stringlist._buffer);
+}
+
 /**
  * pas_book_respond_get_cursor:
  */
@@ -597,6 +656,28 @@ pas_book_report_connection (PASBook  *book,
 	CORBA_exception_free (&ev);
 }
 
+/**
+ * pas_book_report_writable:
+ */
+void
+pas_book_report_writable (PASBook                           *book,
+			  gboolean                           writable)
+{
+	CORBA_Environment ev;
+
+	CORBA_exception_init (&ev);
+
+	GNOME_Evolution_Addressbook_BookListener_notifyWritable (
+		book->priv->listener, (CORBA_boolean) writable, &ev);
+
+	if (ev._major != CORBA_NO_EXCEPTION) {
+		g_warning ("pas_book_report_writable: Exception "
+			   "responding to BookListener!\n");
+	}
+
+	CORBA_exception_free (&ev);
+}
+
 static gboolean
 pas_book_construct (PASBook                *book,
 		    PASBackend             *backend,
@@ -729,6 +810,7 @@ pas_book_get_epv (void)
 	epv->modifyCard            = impl_GNOME_Evolution_Addressbook_Book_modifyCard;
 	epv->checkConnection       = impl_GNOME_Evolution_Addressbook_Book_checkConnection;
 	epv->getStaticCapabilities = impl_GNOME_Evolution_Addressbook_Book_getStaticCapabilities;
+	epv->getSupportedFields    = impl_GNOME_Evolution_Addressbook_Book_getSupportedFields;
 	epv->getCursor             = impl_GNOME_Evolution_Addressbook_Book_getCursor;
 	epv->getBookView           = impl_GNOME_Evolution_Addressbook_Book_getBookView;
 	epv->getChanges            = impl_GNOME_Evolution_Addressbook_Book_getChanges;
