@@ -23,12 +23,6 @@
  */
 
 #include <config.h>
-#include <sys/stat.h>
-#include <utime.h>
-#include <unistd.h>
-#include <pwd.h>
-#include <signal.h>
-#include <errno.h>
 
 #include <liboaf/liboaf.h>
 #include <bonobo.h>
@@ -37,12 +31,12 @@
 #include <pi-socket.h>
 #include <pi-file.h>
 #include <pi-dlp.h>
-#include <pi-version.h>
 #include <ebook/e-book.h>
 #include <ebook/e-card-types.h>
 #include <ebook/e-card-cursor.h>
 #include <ebook/e-card.h>
 #include <ebook/e-card-simple.h>
+#include <e-pilot-util.h>
 
 #define ADDR_CONFIG_LOAD 1
 #define ADDR_CONFIG_DESTROY 1
@@ -55,7 +49,7 @@
 GnomePilotConduit * conduit_get_gpilot_conduit (guint32);
 void conduit_destroy_gpilot_conduit (GnomePilotConduit*);
 
-#define CONDUIT_VERSION "0.1.0"
+#define CONDUIT_VERSION "0.1.1"
 #ifdef G_LOG_DOMAIN
 #undef G_LOG_DOMAIN
 #endif
@@ -76,7 +70,7 @@ void conduit_destroy_gpilot_conduit (GnomePilotConduit*);
 typedef struct {
 	EBookStatus status;
 	char *id;
-} add_card_cons;
+} CardObjectChangeStatus;
 
 typedef enum {
 	CARD_ADDED,
@@ -86,7 +80,7 @@ typedef enum {
 
 typedef struct 
 {
-	char *uid;
+	ECard *card;
 	CardObjectChangeType type;
 } CardObjectChange;
 
@@ -102,10 +96,13 @@ print_local (EAddrLocalRecord *local)
 	}
 
 	if (local->addr) {
-		sprintf (buff, "['%s' '%s' '%s']",
-			 local->addr->entry[entryLastname],
-			 local->addr->entry[entryFirstname],
-			 local->addr->entry[entryCompany]);
+		g_snprintf (buff, 4096, "['%s' '%s' '%s']",
+			    local->addr->entry[entryLastname] ?
+			    local->addr->entry[entryLastname] : "",
+			    local->addr->entry[entryFirstname] ?
+			    local->addr->entry[entryFirstname] : "",
+			    local->addr->entry[entryCompany] ?
+			    local->addr->entry[entryCompany] : "");
 		return buff;
 	}
 
@@ -125,10 +122,13 @@ static char *print_remote (GnomePilotRecord *remote)
 	memset (&addr, 0, sizeof (struct Address));
 	unpack_Address (&addr, remote->record, remote->length);
 
-	sprintf (buff, "['%s' '%s' '%s']",
-		 addr.entry[entryLastname],
-		 addr.entry[entryFirstname],
-		 addr.entry[entryCompany]);
+	g_snprintf (buff, 4096, "['%s' '%s' '%s']",
+		    addr.entry[entryLastname] ?
+		    addr.entry[entryLastname] : "",
+		    addr.entry[entryFirstname] ?
+		    addr.entry[entryFirstname] : "",
+		    addr.entry[entryCompany] ?
+		    addr.entry[entryCompany] : "");
 
 	return buff;
 }
@@ -160,11 +160,11 @@ e_addr_context_destroy (EAddrConduitContext **ctxt)
 static void
 add_card_cb (EBook *ebook, EBookStatus status, const char *id, gpointer closure)
 {
-	add_card_cons *cons = (add_card_cons*)closure;
+	CardObjectChangeStatus *cons = closure;
 
 	cons->status = status;
 	cons->id = g_strdup (id);
-
+	
 	gtk_main_quit();
 }
 
@@ -188,8 +188,12 @@ cursor_cb (EBook *book, EBookStatus status, ECardCursor *cursor, gpointer closur
 
 		length = e_card_cursor_get_length (cursor);
 		ctxt->cards = NULL;
-		for (i = 0; i < length; i ++)
-			ctxt->cards = g_list_append (ctxt->cards, e_card_cursor_get_nth (cursor, i));
+		for (i = 0; i < length; i ++) {
+			ECard *card = e_card_cursor_get_nth (cursor, i);
+			
+			gtk_object_ref (GTK_OBJECT (card));
+			ctxt->cards = g_list_append (ctxt->cards, card);
+		}
 
 		gtk_main_quit(); /* end the sub event loop */
 	}
@@ -251,12 +255,57 @@ map_name (EAddrConduitContext *ctxt)
 	return filename;
 }
 
+static GList *
+next_changed_item (EAddrConduitContext *ctxt, GList *changes) 
+{
+	CardObjectChange *coc;
+	GList *l;
+	
+	for (l = changes; l != NULL; l = l->next) {
+		coc = l->data;
+		
+		if (g_hash_table_lookup (ctxt->changed_hash, e_card_get_id (coc->card)))
+			return l;
+	}
+	
+	return NULL;
+}
+
+static char *
+get_entry_text (struct Address address, int field)
+{
+	if (address.entry[field])
+		return e_pilot_utf8_from_pchar (address.entry[field]);
+	
+	return g_strdup ("");
+}
+
 static void
 compute_status (EAddrConduitContext *ctxt, EAddrLocalRecord *local, const char *uid)
 {
+	CardObjectChange *coc;
+
 	local->local.archived = FALSE;
 	local->local.secret = FALSE;
-	local->local.attr = GnomePilotRecordNothing;
+
+	coc = g_hash_table_lookup (ctxt->changed_hash, uid);
+	
+	if (coc == NULL) {
+		local->local.attr = GnomePilotRecordNothing;
+		return;
+	}
+	
+	switch (coc->type) {
+	case CARD_ADDED:
+		local->local.attr = GnomePilotRecordNew;
+		break;	
+	case CARD_MODIFIED:
+		local->local.attr = GnomePilotRecordModified;
+		break;
+	case CARD_DELETED:
+		local->local.attr = GnomePilotRecordDeleted;
+		break;
+	}
 }
 
 static GnomePilotRecord
@@ -294,6 +343,7 @@ local_record_from_ecard (EAddrLocalRecord *local, ECard *ecard, EAddrConduitCont
 	g_return_if_fail (ecard != NULL);
 
 	local->ecard = ecard;
+	gtk_object_ref (GTK_OBJECT (ecard));
 	simple = e_card_simple_new (ecard);
 	
 	local->local.ID = e_pilot_map_lookup_pid (ctxt->map, ecard->id);
@@ -304,22 +354,22 @@ local_record_from_ecard (EAddrLocalRecord *local, ECard *ecard, EAddrConduitCont
 
 	if (ecard->name) {
 		if (ecard->name->given)
-			local->addr->entry[entryFirstname] = strdup (ecard->name->given);
+			local->addr->entry[entryFirstname] = e_pilot_utf8_to_pchar (ecard->name->given);
 		if (ecard->name->family)
-			local->addr->entry[entryLastname] = strdup (ecard->name->family);
+			local->addr->entry[entryLastname] = e_pilot_utf8_to_pchar (ecard->name->family);
 		if (ecard->org)
-			local->addr->entry[entryCompany] = strdup (ecard->org);
+			local->addr->entry[entryCompany] = e_pilot_utf8_to_pchar (ecard->org);
 		if (ecard->title)
-			local->addr->entry[entryTitle] = strdup (ecard->title);
+			local->addr->entry[entryTitle] = e_pilot_utf8_to_pchar (ecard->title);
 	}
 
 	delivery = e_card_simple_get_delivery_address (simple, E_CARD_SIMPLE_ADDRESS_ID_HOME);
 	if (delivery) {
-		local->addr->entry[entryAddress] = strdup (delivery->street);
-		local->addr->entry[entryCity] = strdup (delivery->city);
-		local->addr->entry[entryState] = strdup (delivery->region);
-		local->addr->entry[entryZip] = strdup (delivery->code);
-		local->addr->entry[entryCountry] = strdup (delivery->country);
+		local->addr->entry[entryAddress] = e_pilot_utf8_to_pchar (delivery->street);
+		local->addr->entry[entryCity] = e_pilot_utf8_to_pchar (delivery->city);
+		local->addr->entry[entryState] = e_pilot_utf8_to_pchar (delivery->region);
+		local->addr->entry[entryZip] = e_pilot_utf8_to_pchar (delivery->code);
+		local->addr->entry[entryCountry] = e_pilot_utf8_to_pchar (delivery->country);
 	}
 	
 	for (i = 0; i <= 7; i++) {
@@ -344,7 +394,7 @@ local_record_from_ecard (EAddrLocalRecord *local, ECard *ecard, EAddrConduitCont
 			phone_str = e_card_simple_get_const (simple, E_CARD_SIMPLE_FIELD_PHONE_MOBILE);
 		
 		if (phone_str) {
-			local->addr->entry[phone] = strdup (phone_str);
+			local->addr->entry[phone] = e_pilot_utf8_to_pchar (phone_str);
 			local->addr->phoneLabel[phone - entryPhone1] = i;
 			phone++;
 		}
@@ -359,9 +409,9 @@ local_record_from_uid (EAddrLocalRecord *local,
 		       char *uid,
 		       EAddrConduitContext *ctxt)
 {
-	ECard *ecard;
+	ECard *ecard = NULL;
 
-	g_assert(local!=NULL);
+	g_assert (local != NULL);
 
 	ecard = e_book_get_card (ctxt->ebook, uid);
 
@@ -369,6 +419,7 @@ local_record_from_uid (EAddrLocalRecord *local,
 		local_record_from_ecard (local, ecard, ctxt);
 	} else {
 		ecard = e_card_new ("");
+		e_card_set_id (ecard, uid);
 		local_record_from_ecard (local, ecard, ctxt);
 	}
 }
@@ -383,79 +434,93 @@ ecard_from_remote_record(EAddrConduitContext *ctxt,
 	ECardSimple *simple;
 	ECardDeliveryAddress delivery;
 	char *string;
-	char *stringparts[4];
+	char *stringparts[3];
 	int i;
 
 	g_return_val_if_fail(remote!=NULL,NULL);
 	memset (&address, 0, sizeof (struct Address));
 	unpack_Address (&address, remote->record, remote->length);
 
-	if (in_card == NULL) {
+	if (in_card == NULL)
 		ecard = e_card_new("");
-	} else {
+	else
 		ecard = e_card_duplicate (in_card);
-	}
 	simple = e_card_simple_new (ecard);
 
-#define get(pilotprop) \
-        (address.entry [(pilotprop)])
-#define check(pilotprop) \
-        (address.entry [(pilotprop)] && *address.entry [(pilotprop)])
-
+	/* Name and company */
 	i = 0;
-	if (check(entryFirstname))
-		stringparts[i++] = get(entryFirstname);
-	if (check(entryLastname))
-		stringparts[i++] = get(entryLastname);
+	if (address.entry[entryFirstname] && *address.entry[entryFirstname])
+		stringparts[i++] = address.entry[entryFirstname];
+	if (address.entry[entryLastname] && *address.entry[entryLastname])
+		stringparts[i++] = address.entry[entryLastname];
 	stringparts[i] = NULL;
+
 	string = g_strjoinv(" ", stringparts);
-	e_card_simple_set(simple, E_CARD_SIMPLE_FIELD_FULL_NAME, string);
+	e_card_simple_set(simple, E_CARD_SIMPLE_FIELD_FULL_NAME, e_pilot_utf8_from_pchar (string));
 	g_free(string);
 
-	if (check (entryTitle))
-		e_card_simple_set(simple, E_CARD_SIMPLE_FIELD_TITLE, get (entryTitle));
+	if (address.entry[entryTitle]) {
+		char *txt = get_entry_text (address, entryTitle);
+		e_card_simple_set(simple, E_CARD_SIMPLE_FIELD_TITLE, txt);
+		g_free (txt);
+	}
 
-	if (check (entryCompany))
-		e_card_simple_set(simple, E_CARD_SIMPLE_FIELD_ORG, get (entryCompany));
+	if (address.entry[entryCompany]) {
+		char *txt = get_entry_text (address, entryCompany);
+		e_card_simple_set(simple, E_CARD_SIMPLE_FIELD_ORG, txt);
+		g_free (txt);
+	}
 
+	/* Address */
 	memset (&delivery, 0, sizeof (ECardDeliveryAddress));
 	delivery.flags = E_CARD_ADDR_HOME;
-	if (check (entryAddress))
-		delivery.street = get (entryAddress);
-	if (check (entryCity))
-		delivery.city = get (entryCity);
-	if (check (entryState))
-		delivery.region = get (entryState);
-	if (check (entryCountry))
-		delivery.country = get (entryCountry);
-	if (check (entryZip))
-		delivery.code = get (entryZip);
+	if (address.entry[entryAddress])
+		delivery.street = get_entry_text (address, entryAddress);
+	if (address.entry[entryCity])
+		delivery.city = get_entry_text (address, entryCity);
+	if (address.entry[entryState])
+		delivery.region = get_entry_text (address, entryState);
+	if (address.entry[entryCountry])
+		delivery.country = get_entry_text (address, entryCountry);
+	if (address.entry[entryZip])
+		delivery.code = get_entry_text (address, entryZip);
+
 	string = e_card_delivery_address_to_string (&delivery);
-	e_card_simple_set (simple,  E_CARD_SIMPLE_FIELD_ADDRESS_HOME, string);
+	e_card_simple_set (simple,  E_CARD_SIMPLE_FIELD_ADDRESS_BUSINESS, string);
 	g_free (string);
-	
+
+	free (delivery.street);
+	free (delivery.city);
+	free (delivery.region);
+	free (delivery.country);
+	free (delivery.code);
+
+	/* Phone numbers */
 	for (i = entryPhone1; i <= entryPhone5; i++) {
 		char *phonelabel = ctxt->ai.phoneLabels[address.phoneLabel[i - entryPhone1]];
+		char *phonenum = get_entry_text (address, i);
 
 		if (!strcmp (phonelabel, "E-mail"))
-			e_card_simple_set(simple, E_CARD_SIMPLE_FIELD_EMAIL, address.entry[i] ? address.entry[i] : "");
+			e_card_simple_set(simple, E_CARD_SIMPLE_FIELD_EMAIL, phonenum);
 		else if (!strcmp (phonelabel, "Home"))
-			e_card_simple_set(simple, E_CARD_SIMPLE_FIELD_PHONE_HOME,address.entry[i] ? address.entry[i] : "" );
+			e_card_simple_set(simple, E_CARD_SIMPLE_FIELD_PHONE_HOME, phonenum);
 		else if (!strcmp (phonelabel, "Work"))
-			e_card_simple_set(simple, E_CARD_SIMPLE_FIELD_PHONE_BUSINESS, address.entry[i] ? address.entry[i] : "");
+			e_card_simple_set(simple, E_CARD_SIMPLE_FIELD_PHONE_BUSINESS, phonenum);
 		else if (!strcmp (phonelabel, "Fax"))
-			e_card_simple_set(simple, E_CARD_SIMPLE_FIELD_PHONE_BUSINESS_FAX, address.entry[i] ? address.entry[i] : "");
+			e_card_simple_set(simple, E_CARD_SIMPLE_FIELD_PHONE_BUSINESS_FAX, phonenum);
 		else if (!strcmp (phonelabel, "Other"))
-			e_card_simple_set(simple, E_CARD_SIMPLE_FIELD_PHONE_OTHER, address.entry[i] ? address.entry[i] : "");
+			e_card_simple_set(simple, E_CARD_SIMPLE_FIELD_PHONE_OTHER, phonenum);
 		else if (!strcmp (phonelabel, "Main"))
-			e_card_simple_set(simple, E_CARD_SIMPLE_FIELD_PHONE_PRIMARY, address.entry[i] ? address.entry[i] : "");
+			e_card_simple_set(simple, E_CARD_SIMPLE_FIELD_PHONE_PRIMARY, phonenum);
 		else if (!strcmp (phonelabel, "Pager"))
-			e_card_simple_set(simple, E_CARD_SIMPLE_FIELD_PHONE_PAGER, address.entry[i] ? address.entry[i] : "");
+			e_card_simple_set(simple, E_CARD_SIMPLE_FIELD_PHONE_PAGER, phonenum);
 		else if (!strcmp (phonelabel, "Mobile"))
-			e_card_simple_set(simple, E_CARD_SIMPLE_FIELD_PHONE_MOBILE, address.entry[i] ? address.entry[i] : "");
+			e_card_simple_set(simple, E_CARD_SIMPLE_FIELD_PHONE_MOBILE, phonenum);
+
+		g_print ("    ['%s' : '%s']\n", phonelabel, phonenum);
+		
+		g_free (phonenum);
 	}
-#undef get
-#undef check
 
 	e_card_simple_sync_card (simple);
 	gtk_object_unref(GTK_OBJECT(simple));
@@ -471,11 +536,9 @@ check_for_slow_setting (GnomePilotConduit *c, EAddrConduitContext *ctxt)
 	int count, map_count;
 
   	count = g_list_length (ctxt->cards);
-
 	map_count = g_hash_table_size (ctxt->map->pid_map);
 	
-  	/* If there are no objects or objects but no log */
-	if ((count == 0) || (count > 0 && map_count == 0)) {
+	if (map_count == 0) {
 		GnomePilotConduitStandard *conduit;
 		LOG ("    doing slow sync\n");
 		conduit = GNOME_PILOT_CONDUIT_STANDARD (c);
@@ -491,14 +554,15 @@ card_added (EBookView *book_view, const GList *cards, EAddrConduitContext *ctxt)
 	const GList *l;
 
 	for (l = cards; l != NULL; l = l->next) {
-		ECard *card = l->data;
 		CardObjectChange *coc = g_new0 (CardObjectChange, 1);
 		
-		coc->uid = g_strdup (e_card_get_id (card));
+		coc->card = E_CARD (l->data);
 		coc->type = CARD_ADDED;
 
+		gtk_object_ref (GTK_OBJECT (coc->card));
 		ctxt->changed = g_list_prepend (ctxt->changed, coc);
-		g_hash_table_insert (ctxt->changed_hash, coc->uid, coc);
+		if (!e_pilot_map_uid_is_archived (ctxt->map, e_card_get_id (coc->card)))
+			g_hash_table_insert (ctxt->changed_hash, e_card_get_id (coc->card), coc);
 	}
 }
 
@@ -508,15 +572,16 @@ card_changed (EBookView *book_view, const GList *cards, EAddrConduitContext *ctx
 	const GList *l;
 
 	for (l = cards; l != NULL; l = l->next) {
-		ECard *card = l->data;
 		CardObjectChange *coc = g_new0 (CardObjectChange, 1);
-	
-		coc->uid = g_strdup (e_card_get_id (card));
+		
+		coc->card = E_CARD (l->data);
 		coc->type = CARD_MODIFIED;
-		g_print ("UID **** %s\n", coc->uid);
+
+		gtk_object_ref (GTK_OBJECT (coc->card));
 		ctxt->changed = g_list_prepend (ctxt->changed, coc);
-		g_hash_table_insert (ctxt->changed_hash, coc->uid, coc);
-	}	
+		if (!e_pilot_map_uid_is_archived (ctxt->map, e_card_get_id (coc->card)))
+			g_hash_table_insert (ctxt->changed_hash, e_card_get_id (coc->card), coc);
+	}
 }
 
 
@@ -525,16 +590,19 @@ card_removed (EBookView *book_view, const char *id, EAddrConduitContext *ctxt)
 {
 	CardObjectChange *coc = g_new0 (CardObjectChange, 1);
 	
-	coc->uid = g_strdup (id);
+	coc->card = e_card_new ("");
+	e_card_set_id (coc->card, id);
 	coc->type = CARD_DELETED;
 
 	ctxt->changed = g_list_prepend (ctxt->changed, coc);
-	g_hash_table_insert (ctxt->changed_hash, coc->uid, coc);
+	if (!e_pilot_map_uid_is_archived (ctxt->map, id))
+		g_hash_table_insert (ctxt->changed_hash, e_card_get_id (coc->card), coc);
 }
 
 static void
 sequence_complete (EBookView *book_view, EAddrConduitContext *ctxt)
 {
+	gtk_signal_disconnect_by_data (GTK_OBJECT (book_view), ctxt);
 	gtk_object_unref (GTK_OBJECT (book_view));
   	gtk_main_quit ();
 }
@@ -632,7 +700,7 @@ post_sync (GnomePilotConduit *conduit,
 	   GnomePilotDBInfo *dbi,
 	   EAddrConduitContext *ctxt)
 {
-	gchar *filename;
+	gchar *filename, *change_id;
 	
 	LOG ("post_sync: Address Conduit v.%s", CONDUIT_VERSION);
 	LOG ("---------------------------------------------------------\n");
@@ -640,6 +708,14 @@ post_sync (GnomePilotConduit *conduit,
 	filename = map_name (ctxt);
 	e_pilot_map_write (filename, ctxt->map);
 	g_free (filename);
+
+	/* FIX ME ugly hack - our changes musn't count, this does introduce
+	 * a race condition if anyone changes a record elsewhere during sycnc
+         */	
+	change_id = g_strdup_printf ("pilot-sync-evolution-addressbook-%d", ctxt->cfg->pilot_id);
+	e_book_get_changes (ctxt->ebook, change_id, view_cb, ctxt);
+	g_free (change_id);
+	gtk_main ();
 	
 	return 0;
 }
@@ -664,6 +740,8 @@ set_status_cleared (GnomePilotConduitSyncAbs *conduit,
 {
 	LOG ("set_status_cleared: clearing status\n");
 	
+	g_hash_table_remove (ctxt->changed_hash, e_card_get_id (local->ecard));
+	
         return 0;
 }
 
@@ -720,7 +798,7 @@ for_each_modified (GnomePilotConduitSyncAbs *conduit,
 		   EAddrLocalRecord **local,
 		   EAddrConduitContext *ctxt)
 {
-	static GList *changes, *iterator;
+	static GList *iterator;
 	static int count;
 
 	g_return_val_if_fail (local != NULL, 0);
@@ -728,39 +806,36 @@ for_each_modified (GnomePilotConduitSyncAbs *conduit,
 	if (*local == NULL) {
 		LOG ("beginning for_each_modified: beginning\n");
 		
-		changes = ctxt->changed;
+		iterator = ctxt->changed;
 		
 		count = 0;
 		
-		if (changes != NULL) {
-			CardObjectChange *coc = changes->data;
+		iterator = next_changed_item (ctxt, iterator);
+		if (iterator != NULL) {
+			CardObjectChange *coc = iterator->data;
 			
-			LOG ("iterating over %d records", g_list_length (changes));
+			LOG ("iterating over %d records", g_hash_table_size (ctxt->changed_hash));
 			 
 			*local = g_new0 (EAddrLocalRecord, 1);
-			local_record_from_uid (*local, coc->uid, ctxt);
-
-			iterator = changes;
+			local_record_from_ecard (*local, coc->card, ctxt);
 		} else {
 			LOG ("no events");
-			(*local) = NULL;
-			return 0;
+
+			*local = NULL;
 		}
 	} else {
 		count++;
-		if (g_list_next (iterator)) {
-			CardObjectChange *coc;
-
-			iterator = g_list_next (iterator);
-			coc = iterator->data;
+		iterator = g_list_next (iterator);
+		if (iterator && (iterator = next_changed_item (ctxt, iterator))) {
+			CardObjectChange *coc = iterator->data;
 
 			*local = g_new0 (EAddrLocalRecord, 1);
-			local_record_from_uid (*local, coc->uid, ctxt);
+			local_record_from_ecard (*local, coc->card, ctxt);
 		} else {
 			LOG ("for_each_modified ending");
 
-    			/* Tell the pilot the iteration is over */
-			(*local) = NULL;
+    			/* Signal the iteration is over */
+			*local = NULL;
 
 			return 0;
 		}
@@ -805,7 +880,7 @@ add_record (GnomePilotConduitSyncAbs *conduit,
 	    EAddrConduitContext *ctxt)
 {
 	ECard *ecard;
-	add_card_cons cons;
+	CardObjectChangeStatus cons;
 	int retval = 0;
 	
 	g_return_val_if_fail (remote != NULL, -1);
@@ -813,7 +888,7 @@ add_record (GnomePilotConduitSyncAbs *conduit,
 	LOG ("add_record: adding %s to desktop\n", print_remote (remote));
 
 	ecard = ecard_from_remote_record (ctxt, remote, NULL);
-
+	
 	/* add the ecard to the server */
 	e_book_add_card (ctxt->ebook, ecard, add_card_cb, &cons);
 
@@ -824,10 +899,7 @@ add_record (GnomePilotConduitSyncAbs *conduit,
 		return -1;
 	}
 
-	ctxt->cards = g_list_append (ctxt->cards,
-				     e_book_get_card (ctxt->ebook, cons.id));
-	g_free (cons.id);
-
+	e_card_set_id (ecard, cons.id);
 	e_pilot_map_insert (ctxt->map, remote->ID, ecard->id, FALSE);
 
 	return retval;
@@ -841,6 +913,9 @@ replace_record (GnomePilotConduitSyncAbs *conduit,
 {
 	ECard *new_ecard;
 	EBookStatus commit_status;
+	CardObjectChange *coc;
+	CardObjectChangeStatus cons;
+	char *old_id;
 	int retval = 0;
 	
 	g_return_val_if_fail (remote != NULL, -1);
@@ -848,19 +923,41 @@ replace_record (GnomePilotConduitSyncAbs *conduit,
 	LOG ("replace_record: replace %s with %s\n",
 	     print_local (local), print_remote (remote));
 
+	old_id = g_strdup (e_card_get_id (local->ecard));
+	coc = g_hash_table_lookup (ctxt->changed_hash, old_id);
+	
 	new_ecard = ecard_from_remote_record (ctxt, remote, local->ecard);
 	gtk_object_unref (GTK_OBJECT (local->ecard));
 	local->ecard = new_ecard;
 
-	e_book_commit_card (ctxt->ebook, local->ecard, status_cb, &commit_status);
+	if (coc && coc->type == CARD_DELETED)
+		e_book_add_card (ctxt->ebook, local->ecard, add_card_cb, &cons);
+	else
+		e_book_commit_card (ctxt->ebook, local->ecard, status_cb, &commit_status);
 	
 	gtk_main (); /* enter sub mainloop */
+
+	/* Adding a record causes wombat to assign a new uid so we must tidy */
+	if (coc && coc->type == CARD_DELETED) {
+		gboolean arch = e_pilot_map_uid_is_archived (ctxt->map, e_card_get_id (local->ecard));
+		
+		e_card_set_id (local->ecard, cons.id);
+		e_pilot_map_insert (ctxt->map, remote->ID, cons.id, arch);
+
+		coc = g_hash_table_lookup (ctxt->changed_hash, old_id);
+		if (coc) {
+			g_hash_table_remove (ctxt->changed_hash, e_card_get_id (coc->card));
+			coc->card = local->ecard;
+			g_hash_table_insert (ctxt->changed_hash, e_card_get_id (coc->card), coc);
+			
+		}
+		
+		commit_status = cons.status;
+	}
 	
 	if (commit_status != E_BOOK_STATUS_SUCCESS)
 		WARN ("replace_record: failed to update card in ebook\n");
 
-	gtk_object_unref (GTK_OBJECT (new_ecard));
-
 	return retval;
 }
 
@@ -877,13 +974,14 @@ delete_record (GnomePilotConduitSyncAbs *conduit,
 
 	LOG ("delete_record: delete %s\n", print_local (local));
 
+	e_pilot_map_remove_by_uid (ctxt->map, local->ecard->id);
 	e_book_remove_card_by_id (ctxt->ebook, local->ecard->id, status_cb, &commit_status);
 	
 	gtk_main (); /* enter sub mainloop */
 	
-	if (commit_status != E_BOOK_STATUS_SUCCESS)
+	if (commit_status != E_BOOK_STATUS_SUCCESS && commit_status != E_BOOK_STATUS_CARD_NOT_FOUND)
 		WARN ("delete_record: failed to delete card in ebook\n");
-
+	
 	return retval;
 }
 
@@ -919,7 +1017,7 @@ match (GnomePilotConduitSyncAbs *conduit,
 	g_return_val_if_fail (remote != NULL, -1);
 
 	*local = NULL;
-	uid = g_hash_table_lookup (ctxt->map->pid_map, &remote->ID);
+	uid = e_pilot_map_lookup_uid (ctxt->map, remote->ID);
 	
 	if (!uid)
 		return 0;
