@@ -56,6 +56,8 @@ static GtkWidget *e_contact_editor_build_dialog(EContactEditor *editor, gchar *e
 static void _email_arrow_pressed (GtkWidget *widget, GdkEventButton *button, EContactEditor *editor);
 static void _phone_arrow_pressed (GtkWidget *widget, GdkEventButton *button, EContactEditor *editor);
 static void _address_arrow_pressed (GtkWidget *widget, GdkEventButton *button, EContactEditor *editor);
+static void enable_writable_fields(EContactEditor *editor);
+static void set_read_only(EContactEditor *editor);
 static void fill_in_info(EContactEditor *editor);
 static void extract_info(EContactEditor *editor);
 static void set_fields(EContactEditor *editor);
@@ -70,7 +72,9 @@ static guint contact_editor_signals[LAST_SIGNAL];
 enum {
 	ARG_0,
 	ARG_CARD,
-	ARG_IS_NEW_CARD
+	ARG_IS_NEW_CARD,
+	ARG_IS_READ_ONLY,
+	ARG_WRITABLE_FIELDS
 };
 
 enum {
@@ -117,6 +121,10 @@ e_contact_editor_class_init (EContactEditorClass *klass)
 			   GTK_ARG_READWRITE, ARG_CARD);
   gtk_object_add_arg_type ("EContactEditor::is_new_card", GTK_TYPE_BOOL,
 			   GTK_ARG_READWRITE, ARG_IS_NEW_CARD);
+  gtk_object_add_arg_type ("EContactEditor::writable_fields", GTK_TYPE_POINTER,
+			   GTK_ARG_READWRITE, ARG_WRITABLE_FIELDS);
+  gtk_object_add_arg_type ("EContactEditor::is_read_only", GTK_TYPE_BOOL,
+			   GTK_ARG_READWRITE, ARG_IS_READ_ONLY);
 
   contact_editor_signals[ADD_CARD] =
 	  gtk_signal_new ("add_card",
@@ -489,8 +497,13 @@ full_name_clicked(GtkWidget *button, EContactEditor *editor)
 {
 	GnomeDialog *dialog = GNOME_DIALOG(e_contact_editor_fullname_new(editor->name));
 	int result;
+
+	gtk_object_set (GTK_OBJECT (dialog),
+			"is_read_only", editor->is_read_only,
+			NULL);
 	gtk_widget_show(GTK_WIDGET(dialog));
 	result = gnome_dialog_run (dialog);
+
 	if (result == 0) {
 		ECardName *name;
 		GtkWidget *fname_widget;
@@ -527,6 +540,9 @@ full_addr_clicked(GtkWidget *button, EContactEditor *editor)
 	address = e_card_simple_get_delivery_address(editor->simple, editor->address_choice);
 
 	dialog = GNOME_DIALOG(e_contact_editor_address_new(address));
+	gtk_object_set (GTK_OBJECT (dialog),
+			"is_read_only", editor->is_read_only,
+			NULL);
 	gtk_widget_show(GTK_WIDGET(dialog));
 
 	result = gnome_dialog_run (dialog);
@@ -570,6 +586,9 @@ categories_clicked(GtkWidget *button, EContactEditor *editor)
 			       "categories", &categories,
 			       NULL);
 	dialog = GNOME_DIALOG(e_categories_new(categories));
+	gtk_object_set(GTK_OBJECT(dialog),
+		       "header", _("This contact belongs to these categories:"),
+		       NULL);
 	gtk_widget_show(GTK_WIDGET(dialog));
 	result = gnome_dialog_run (dialog);
 	g_free (categories);
@@ -969,7 +988,10 @@ e_contact_editor_init (EContactEditor *e_contact_editor)
 void
 e_contact_editor_destroy (GtkObject *object) {
 	EContactEditor *e_contact_editor = E_CONTACT_EDITOR(object);
-	
+
+	if (e_contact_editor->writable_fields) {
+		gtk_object_unref(GTK_OBJECT(e_contact_editor->writable_fields));
+	}
 	if (e_contact_editor->email_list) {
 		g_list_foreach(e_contact_editor->email_list, (GFunc) g_free, NULL);
 		g_list_free(e_contact_editor->email_list);
@@ -1015,7 +1037,10 @@ e_contact_editor_destroy (GtkObject *object) {
 }
 
 EContactEditor *
-e_contact_editor_new (ECard *card, gboolean is_new_card)
+e_contact_editor_new (ECard *card,
+		      gboolean is_new_card,
+		      EList *fields,
+		      gboolean is_read_only)
 {
 	EContactEditor *ce;
 
@@ -1024,6 +1049,8 @@ e_contact_editor_new (ECard *card, gboolean is_new_card)
 	gtk_object_set (GTK_OBJECT (ce),
 			"card", card,
 			"is_new_card", is_new_card,
+			"writable_fields", fields,
+			"is_read_only", is_read_only,
 			NULL);
 
 	gtk_widget_show (ce->app);
@@ -1051,6 +1078,27 @@ e_contact_editor_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 	case ARG_IS_NEW_CARD:
 		editor->is_new_card = GTK_VALUE_BOOL (*arg) ? TRUE : FALSE;
 		break;
+
+	case ARG_IS_READ_ONLY: {
+		gboolean new_value = GTK_VALUE_BOOL (*arg) ? TRUE : FALSE;
+		gboolean changed = (editor->is_read_only != new_value);
+
+		editor->is_read_only = new_value;
+
+		if (changed)
+			set_read_only (editor);
+		break;
+	}
+	case ARG_WRITABLE_FIELDS:
+		if (editor->writable_fields)
+			gtk_object_unref(GTK_OBJECT(editor->writable_fields));
+		editor->writable_fields = GTK_VALUE_POINTER (*arg);
+		if (editor->writable_fields)
+			gtk_object_ref (GTK_OBJECT (editor->writable_fields));
+		else
+			editor->writable_fields = e_list_new(NULL, NULL, NULL);
+		enable_writable_fields (editor);
+		break;
 	}
 }
 
@@ -1072,6 +1120,16 @@ e_contact_editor_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 		GTK_VALUE_BOOL (*arg) = e_contact_editor->is_new_card ? TRUE : FALSE;
 		break;
 
+	case ARG_IS_READ_ONLY:
+		GTK_VALUE_BOOL (*arg) = e_contact_editor->is_read_only ? TRUE : FALSE;
+		break;
+
+	case ARG_WRITABLE_FIELDS:
+		if (e_contact_editor->writable_fields)
+			GTK_VALUE_POINTER (*arg) = e_list_duplicate (e_contact_editor->writable_fields);
+		else
+			GTK_VALUE_POINTER (*arg) = NULL;
+		break;
 	default:
 		arg->type = GTK_TYPE_INVALID;
 		break;
@@ -1161,6 +1219,99 @@ e_contact_editor_build_ui_info(GList *list, GnomeUIInfo **infop)
 	*infop = info;
 }
 
+static void
+e_contact_editor_build_phone_ui (EContactEditor *editor)
+{
+	int i;
+
+	if (editor->phone_list == NULL) {
+		static char *info[] = {
+			N_("Assistant"),
+			N_("Business"),
+			N_("Business 2"),
+			N_("Business Fax"),
+			N_("Callback"),
+			N_("Car"),
+			N_("Company"),
+			N_("Home"),
+			N_("Home 2"),
+			N_("Home Fax"),
+			N_("ISDN"),
+			N_("Mobile"),
+			N_("Other"),
+			N_("Other Fax"),
+			N_("Pager"),
+			N_("Primary"),
+			N_("Radio"),
+			N_("Telex"),
+			N_("TTY/TDD")
+		};
+		
+		for (i = 0; i < sizeof(info) / sizeof(info[0]); i++) {
+			editor->phone_list = g_list_append(editor->phone_list, g_strdup(info[i]));
+		}
+	}
+	if (editor->phone_info == NULL) {
+		e_contact_editor_build_ui_info(editor->phone_list, &editor->phone_info);
+		
+		if ( editor->phone_popup )
+			gtk_widget_unref(editor->phone_popup);
+		
+		editor->phone_popup = gnome_popup_menu_new(editor->phone_info);
+	}
+}
+
+static void
+e_contact_editor_build_email_ui (EContactEditor *editor)
+{
+	int i;
+
+	if (editor->email_list == NULL) {
+		static char *info[] = {
+			N_("Primary Email"),
+			N_("Email 2"),
+			N_("Email 3")
+		};
+		
+		for (i = 0; i < sizeof(info) / sizeof(info[0]); i++) {
+			editor->email_list = g_list_append(editor->email_list, g_strdup(info[i]));
+		}
+	}
+	if (editor->email_info == NULL) {
+		e_contact_editor_build_ui_info(editor->email_list, &editor->email_info);
+
+		if ( editor->email_popup )
+			gtk_widget_unref(editor->email_popup);
+		
+		editor->email_popup = gnome_popup_menu_new(editor->email_info);
+	}
+}
+
+static void
+e_contact_editor_build_address_ui (EContactEditor *editor)
+{
+	int i;
+
+	if (editor->address_list == NULL) {
+		static char *info[] = {
+			N_("Business"),
+			N_("Home"),
+			N_("Other")
+		};
+		
+		for (i = 0; i < sizeof(info) / sizeof(info[0]); i++) {
+			editor->address_list = g_list_append(editor->address_list, g_strdup(info[i]));
+		}
+	}
+	if (editor->address_info == NULL) {
+		e_contact_editor_build_ui_info(editor->address_list, &editor->address_info);
+
+		if ( editor->address_popup )
+			gtk_widget_unref(editor->address_popup);
+		
+		editor->address_popup = gnome_popup_menu_new(editor->address_info);
+	}
+}
 #if 0
 static void
 _dialog_clicked(GtkWidget *dialog, gint button, EContactEditor *editor)
@@ -1269,41 +1420,7 @@ _phone_arrow_pressed (GtkWidget *widget, GdkEventButton *button, EContactEditor
 	label = g_strdup_printf("label-phone%d", which);
 	entry = g_strdup_printf("entry-phone%d", which);
 
-	if (editor->phone_list == NULL) {
-		static char *info[] = {
-			N_("Assistant"),
-			N_("Business"),
-			N_("Business 2"),
-			N_("Business Fax"),
-			N_("Callback"),
-			N_("Car"),
-			N_("Company"),
-			N_("Home"),
-			N_("Home 2"),
-			N_("Home Fax"),
-			N_("ISDN"),
-			N_("Mobile"),
-			N_("Other"),
-			N_("Other Fax"),
-			N_("Pager"),
-			N_("Primary"),
-			N_("Radio"),
-			N_("Telex"),
-			N_("TTY/TDD")
-		};
-		
-		for (i = 0; i < sizeof(info) / sizeof(info[0]); i++) {
-			editor->phone_list = g_list_append(editor->phone_list, g_strdup(info[i]));
-		}
-	}
-	if (editor->phone_info == NULL) {
-		e_contact_editor_build_ui_info(editor->phone_list, &editor->phone_info);
-		
-		if ( editor->phone_popup )
-			gtk_widget_unref(editor->phone_popup);
-		
-		editor->phone_popup = gnome_popup_menu_new(editor->phone_info);
-	}
+	e_contact_editor_build_phone_ui (editor);
 	
 	for(i = 0; i < E_CARD_SIMPLE_PHONE_ID_LAST; i++) {
 		const ECardPhone *phone = e_card_simple_get_phone(editor->simple, i);
@@ -1320,6 +1437,8 @@ _phone_arrow_pressed (GtkWidget *widget, GdkEventButton *button, EContactEditor
 	if (result != -1) {
 		editor->phone_choice[which - 1] = result;
 		set_fields(editor);
+		gtk_widget_set_sensitive (glade_xml_get_widget (editor->gui, label), TRUE);
+		gtk_widget_set_sensitive (glade_xml_get_widget (editor->gui, entry), !editor->is_read_only);
 	}
 
 	g_free(label);
@@ -1331,26 +1450,9 @@ _email_arrow_pressed (GtkWidget *widget, GdkEventButton *button, EContactEditor
 {
 	int i;
 	int result;
-	if (editor->email_list == NULL) {
-		static char *info[] = {
-			N_("Primary Email"),
-			N_("Email 2"),
-			N_("Email 3")
-		};
-		
-		for (i = 0; i < sizeof(info) / sizeof(info[0]); i++) {
-			editor->email_list = g_list_append(editor->email_list, g_strdup(info[i]));
-		}
-	}
-	if (editor->email_info == NULL) {
-		e_contact_editor_build_ui_info(editor->email_list, &editor->email_info);
 
-		if ( editor->email_popup )
-			gtk_widget_unref(editor->email_popup);
-		
-		editor->email_popup = gnome_popup_menu_new(editor->email_info);
-	}
-	
+	e_contact_editor_build_email_ui (editor);
+
 	for(i = 0; i < E_CARD_SIMPLE_EMAIL_ID_LAST; i++) {
 		const char *string = e_card_simple_get_email(editor->simple, i);
 		gboolean checked;
@@ -1366,6 +1468,11 @@ _email_arrow_pressed (GtkWidget *widget, GdkEventButton *button, EContactEditor
 	if (result != -1) {
 		editor->email_choice = result;
 		set_fields(editor);
+
+		/* make sure the buttons/entry is/are sensitive */
+		gtk_widget_set_sensitive (glade_xml_get_widget (editor->gui, "label-email1"), TRUE);
+		gtk_widget_set_sensitive (glade_xml_get_widget (editor->gui, "entry-email1"), !editor->is_read_only);
+		gtk_widget_set_sensitive (glade_xml_get_widget (editor->gui, "checkbutton-htmlmail"), !editor->is_read_only);
 	}
 }
 
@@ -1374,26 +1481,9 @@ _address_arrow_pressed (GtkWidget *widget, GdkEventButton *button, EContactEdito
 {
 	int i;
 	int result;
-	if (editor->address_list == NULL) {
-		static char *info[] = {
-			N_("Business"),
-			N_("Home"),
-			N_("Other")
-		};
-		
-		for (i = 0; i < sizeof(info) / sizeof(info[0]); i++) {
-			editor->address_list = g_list_append(editor->address_list, g_strdup(info[i]));
-		}
-	}
-	if (editor->address_info == NULL) {
-		e_contact_editor_build_ui_info(editor->address_list, &editor->address_info);
 
-		if ( editor->address_popup )
-			gtk_widget_unref(editor->address_popup);
-		
-		editor->address_popup = gnome_popup_menu_new(editor->address_info);
-	}
-	
+	e_contact_editor_build_address_ui (editor);
+
 	for(i = 0; i < E_CARD_SIMPLE_ADDRESS_ID_LAST; i++) {
 		const ECardAddrLabel *address = e_card_simple_get_address(editor->simple, i);
 		gboolean checked;
@@ -1408,6 +1498,11 @@ _address_arrow_pressed (GtkWidget *widget, GdkEventButton *button, EContactEdito
 
 	if (result != -1) {
 		set_address_field(editor, result);
+
+		/* make sure the buttons/entry is/are sensitive */
+		gtk_widget_set_sensitive (glade_xml_get_widget (editor->gui, "label-address"), TRUE);
+		gtk_widget_set_sensitive (glade_xml_get_widget (editor->gui, "button-fulladdr"), !editor->is_read_only);
+		gtk_widget_set_sensitive (glade_xml_get_widget (editor->gui, "text-address"), !editor->is_read_only);
 	}
 }
 
@@ -1597,6 +1692,223 @@ fill_in_single_field(EContactEditor *editor, char *name)
 	}
 }
 
+static void
+disable_widget_foreach (char *key, GtkWidget *widget, gpointer closure)
+{
+	gtk_widget_set_sensitive (widget, FALSE);
+}
+
+static struct {
+	char *widget_name;
+	ECardSimpleField field_id;
+	gboolean desensitize_for_read_only;
+} widget_field_mappings[] = {
+	{ "entry-web", E_CARD_SIMPLE_FIELD_URL, TRUE },
+	{ "accellabel-web", E_CARD_SIMPLE_FIELD_URL },
+
+	{ "entry-jobtitle", E_CARD_SIMPLE_FIELD_TITLE, TRUE },
+	{ "label-jobtitle", E_CARD_SIMPLE_FIELD_TITLE },
+
+	{ "entry-company", E_CARD_SIMPLE_FIELD_ORG, TRUE },
+	{ "label-company", E_CARD_SIMPLE_FIELD_ORG },
+
+	{ "combo-file-as", E_CARD_SIMPLE_FIELD_FILE_AS, TRUE },
+	{ "entry-file-as", E_CARD_SIMPLE_FIELD_FILE_AS, TRUE },
+	{ "accellabel-fileas", E_CARD_SIMPLE_FIELD_FILE_AS },
+
+	{ "label-department", E_CARD_SIMPLE_FIELD_ORG_UNIT },
+	{ "entry-department", E_CARD_SIMPLE_FIELD_ORG_UNIT, TRUE },
+
+	{ "label-office", E_CARD_SIMPLE_FIELD_OFFICE },
+	{ "entry-office", E_CARD_SIMPLE_FIELD_OFFICE, TRUE },
+
+	{ "label-profession", E_CARD_SIMPLE_FIELD_ROLE },
+	{ "entry-profession", E_CARD_SIMPLE_FIELD_ROLE, TRUE },
+
+	{ "label-manager", E_CARD_SIMPLE_FIELD_MANAGER },
+	{ "entry-manager", E_CARD_SIMPLE_FIELD_MANAGER, TRUE },
+
+	{ "label-assistant", E_CARD_SIMPLE_FIELD_ASSISTANT },
+	{ "entry-assistant", E_CARD_SIMPLE_FIELD_ASSISTANT, TRUE },
+
+	{ "label-nickname", E_CARD_SIMPLE_FIELD_NICKNAME },
+	{ "entry-nickname", E_CARD_SIMPLE_FIELD_NICKNAME, TRUE },
+
+	{ "label-spouse", E_CARD_SIMPLE_FIELD_SPOUSE },
+	{ "entry-spouse", E_CARD_SIMPLE_FIELD_SPOUSE, TRUE },
+
+	{ "label-birthday", E_CARD_SIMPLE_FIELD_BIRTH_DATE },
+	{ "dateedit-birthday", E_CARD_SIMPLE_FIELD_BIRTH_DATE, TRUE },
+
+	{ "label-anniversary", E_CARD_SIMPLE_FIELD_ANNIVERSARY },
+	{ "dateedit-anniversary", E_CARD_SIMPLE_FIELD_ANNIVERSARY, TRUE },
+
+	{ "label-comments", E_CARD_SIMPLE_FIELD_NOTE },
+	{ "text-comments", E_CARD_SIMPLE_FIELD_NOTE, TRUE },
+
+	{ "button-fullname", E_CARD_SIMPLE_FIELD_FULL_NAME },
+	{ "entry-fullname", E_CARD_SIMPLE_FIELD_FULL_NAME, TRUE },
+
+	{ "button-categories", E_CARD_SIMPLE_FIELD_CATEGORIES, TRUE },
+	{ "entry-categories", E_CARD_SIMPLE_FIELD_CATEGORIES, TRUE }
+};
+static int num_widget_field_mappings = sizeof(widget_field_mappings) / sizeof (widget_field_mappings[0]);
+
+static void
+enable_writable_fields(EContactEditor *editor)
+{
+	EList *fields = editor->writable_fields;
+	EIterator *iter;
+	GHashTable *dropdown_hash, *supported_hash;
+	int i;
+	ECardSimple *card;
+	char *widget_name;
+
+	if (!fields)
+		return;
+
+	card = e_card_simple_new (e_card_new (""));
+
+	dropdown_hash = g_hash_table_new (g_str_hash, g_str_equal);
+	supported_hash = g_hash_table_new (g_str_hash, g_str_equal);
+
+	/* build our hashtable of the drop down menu items */
+	e_contact_editor_build_phone_ui (editor);
+	for (i = 0; i < E_CARD_SIMPLE_PHONE_ID_LAST; i ++)
+		g_hash_table_insert (dropdown_hash,
+				     (char*)e_card_simple_get_ecard_field(card, e_card_simple_map_phone_to_field (i)),
+				     editor->phone_info[i].widget);
+	e_contact_editor_build_email_ui (editor);
+	for (i = 0; i < E_CARD_SIMPLE_EMAIL_ID_LAST; i ++)
+		g_hash_table_insert (dropdown_hash,
+				     (char*)e_card_simple_get_ecard_field(card, e_card_simple_map_email_to_field (i)),
+				     editor->email_info[i].widget);
+	e_contact_editor_build_address_ui (editor);
+	for (i = 0; i < E_CARD_SIMPLE_ADDRESS_ID_LAST; i ++)
+		g_hash_table_insert (dropdown_hash,
+				     (char*)e_card_simple_get_ecard_field(card, e_card_simple_map_address_to_field (i)),
+				     editor->address_info[i].widget);
+
+	/* then disable them all */
+	g_hash_table_foreach (dropdown_hash, (GHFunc)disable_widget_foreach, NULL);
+
+	/* disable the label widgets for the dropdowns (4 phone, 1
+           email and the toggle button, and 1 address and one for
+           the full adress button */
+	for (i = 0; i < 4; i ++) {
+		widget_name = g_strdup_printf ("label-phone%d", i+1);
+		gtk_widget_set_sensitive (glade_xml_get_widget (editor->gui, widget_name), FALSE);
+		g_free (widget_name);
+		widget_name = g_strdup_printf ("entry-phone%d", i+1);
+		gtk_widget_set_sensitive (glade_xml_get_widget (editor->gui, widget_name), FALSE);
+		g_free (widget_name);
+	}
+	gtk_widget_set_sensitive (glade_xml_get_widget (editor->gui, "label-email1"), FALSE);
+	gtk_widget_set_sensitive (glade_xml_get_widget (editor->gui, "entry-email1"), FALSE);
+	gtk_widget_set_sensitive (glade_xml_get_widget (editor->gui, "checkbutton-htmlmail"), FALSE);
+	gtk_widget_set_sensitive (glade_xml_get_widget (editor->gui, "label-address"), FALSE);
+	gtk_widget_set_sensitive (glade_xml_get_widget (editor->gui, "button-fulladdr"), FALSE);
+	gtk_widget_set_sensitive (glade_xml_get_widget (editor->gui, "text-address"), FALSE);
+
+	/* enable widgets that map directly from a field to a widget (the drop down items) */
+	iter = e_list_get_iterator (fields);
+	for (; e_iterator_is_valid (iter); e_iterator_next (iter)) {
+		char *field = (char*)e_iterator_get (iter);
+		GtkWidget *widget = g_hash_table_lookup (dropdown_hash, field);
+
+		if (widget) {
+			gtk_widget_set_sensitive (widget, TRUE);
+		}
+		else {
+			/* if it's not a field that's handled by the
+                           dropdown items, add it to the has to be
+                           used in the second step */
+			g_hash_table_insert (supported_hash, field, field);
+		}
+
+		/* ugh - this is needed to make sure we don't have a
+                   disabled label next to a drop down when the item in
+                   the menu (the one reflected in the label) is
+                   enabled. */
+		if (!strcmp (field, e_card_simple_get_ecard_field (card, e_card_simple_map_email_to_field(editor->email_choice)))) {
+			gtk_widget_set_sensitive (glade_xml_get_widget (editor->gui, "label-email1"), TRUE);
+			gtk_widget_set_sensitive (glade_xml_get_widget (editor->gui, "entry-email1"), !editor->is_read_only);
+			gtk_widget_set_sensitive (glade_xml_get_widget (editor->gui, "checkbutton-htmlmail"), TRUE);
+		}
+		else if (!strcmp (field, e_card_simple_get_ecard_field (card, e_card_simple_map_address_to_field(editor->address_choice)))) {
+			gtk_widget_set_sensitive (glade_xml_get_widget (editor->gui, "label-address"), TRUE);
+			gtk_widget_set_sensitive (glade_xml_get_widget (editor->gui, "button-fulladdr"), !editor->is_read_only);
+			gtk_widget_set_sensitive (glade_xml_get_widget (editor->gui, "text-address"), !editor->is_read_only);
+		}
+		else for (i = 0; i < 4; i ++) {
+			if (!strcmp (field, e_card_simple_get_ecard_field (card, e_card_simple_map_phone_to_field(editor->phone_choice[i])))) {
+				widget_name = g_strdup_printf ("label-phone%d", i+1);
+				gtk_widget_set_sensitive (glade_xml_get_widget (editor->gui, widget_name), TRUE);
+				g_free (widget_name);
+				widget_name = g_strdup_printf ("entry-phone%d", i+1);
+				gtk_widget_set_sensitive (glade_xml_get_widget (editor->gui, widget_name), !editor->is_read_only);
+				g_free (widget_name);
+			}
+		}
+	}
+
+	/* handle the label next to the dropdown widgets */
+
+	for (i = 0; i < num_widget_field_mappings; i ++) {
+		gboolean enabled = g_hash_table_lookup (supported_hash,
+							e_card_simple_get_ecard_field (card,
+										       widget_field_mappings[i].field_id)) != NULL;
+		gtk_widget_set_sensitive (glade_xml_get_widget(editor->gui,
+							       widget_field_mappings[i].widget_name), enabled);
+	}
+
+	g_hash_table_destroy (dropdown_hash);
+	g_hash_table_destroy (supported_hash);
+}
+
+static void
+set_read_only (EContactEditor *editor)
+{
+	int i;
+	char *label, *entry;
+	/* set the sensitivity of all the non-dropdown entry/texts/dateedits */
+	for (i = 0; i < num_widget_field_mappings; i ++) {
+		if (widget_field_mappings[i].desensitize_for_read_only)
+			gtk_widget_set_sensitive (glade_xml_get_widget(editor->gui,
+								       widget_field_mappings[i].widget_name), !editor->is_read_only);
+	}
+
+	/* handle the phone dropdown entries */
+	for (i = 0; i < 4; i ++) {
+		label = g_strdup_printf ("label-phone%d", i+1);
+		entry = g_strdup_printf ("entry-phone%d", i+1);
+
+		if (GTK_WIDGET_IS_SENSITIVE (glade_xml_get_widget (editor->gui, label)))
+			gtk_widget_set_sensitive (glade_xml_get_widget(editor->gui,
+								       entry), !editor->is_read_only);
+
+		g_free (label);
+		g_free (entry);
+	}
+
+	/* handle the email dropdown entry */
+	label = "label-email1";
+	entry = "entry-email1";
+	if (GTK_WIDGET_IS_SENSITIVE (glade_xml_get_widget (editor->gui, label))) {
+		gtk_widget_set_sensitive (glade_xml_get_widget(editor->gui,
+							       entry), !editor->is_read_only);
+		gtk_widget_set_sensitive (glade_xml_get_widget(editor->gui,
+							       "checkbutton-htmlmail"), !editor->is_read_only);
+	}
+
+	/* handle the address dropdown entry */
+	label = "label-address";
+	entry = "text-address";
+	if (GTK_WIDGET_IS_SENSITIVE (glade_xml_get_widget (editor->gui, label)))
+		gtk_widget_set_sensitive (glade_xml_get_widget(editor->gui,
+							       entry), !editor->is_read_only);
+}
+
 static void
 fill_in_info(EContactEditor *editor)
 {
@@ -1786,3 +2098,15 @@ extract_info(EContactEditor *editor)
 		}
 	}
 }
+
+/**
+ * e_contact_editor_raise:
+ * @config: The %EContactEditor object.
+ *
+ * Raises the dialog associated with this %EContactEditor object.
+ */
+void
+e_contact_editor_raise (EContactEditor *editor)
+{
+	gdk_window_raise (GTK_WIDGET (editor->app)->window);
+}
