@@ -34,6 +34,7 @@ static void e_minicard_view_destroy (GtkObject *object);
 static gboolean e_minicard_view_event (GnomeCanvasItem *item, GdkEvent *event);
 static void canvas_destroy (GtkObject *object, EMinicardView *view);
 static void disconnect_signals (EMinicardView *view);
+static void e_minicard_view_update_selection (EMinicardView *view);
 
 static EReflowSortedClass *parent_class = NULL;
 
@@ -41,7 +42,8 @@ static EReflowSortedClass *parent_class = NULL;
 enum {
 	ARG_0,
 	ARG_BOOK,
-	ARG_QUERY
+	ARG_QUERY,
+	ARG_EDITABLE
 };
 
 enum {
@@ -91,6 +93,8 @@ e_minicard_view_class_init (EMinicardViewClass *klass)
 				 GTK_ARG_READWRITE, ARG_BOOK);
 	gtk_object_add_arg_type ("EMinicardView::query", GTK_TYPE_STRING,
 				 GTK_ARG_READWRITE, ARG_QUERY);
+	gtk_object_add_arg_type ("EMinicardView::editable", GTK_TYPE_BOOL,
+				 GTK_ARG_READWRITE, ARG_EDITABLE);
 
 	e_minicard_view_signals [STATUS_MESSAGE] =
 		gtk_signal_new ("status_message",
@@ -111,11 +115,18 @@ e_minicard_view_class_init (EMinicardViewClass *klass)
 	/* GnomeCanvasItem method overrides */
 }
 
+static void
+selection_changed (ESelectionModel *selection, EMinicardView *view)
+{
+	e_minicard_view_update_selection (view);
+}
+
 static void
 e_minicard_view_init (EMinicardView *view)
 {
 	view->book = NULL;
 	view->query = g_strdup("(contains \"x-evolution-any-field\" \"\")");
+	view->editable = FALSE;
 	view->book_view = NULL;
 	view->get_view_idle = 0;
 	view->create_card_id = 0;
@@ -124,6 +135,11 @@ e_minicard_view_init (EMinicardView *view)
 	view->status_message_id = 0;
 	view->canvas_destroy_id = 0;
 	view->first_get_view = TRUE;
+
+	view->selection = e_selection_model_simple_new();
+
+	gtk_signal_connect(GTK_OBJECT(view->selection), "selection_changed",
+			   GTK_SIGNAL_FUNC(selection_changed), view);
 	
 	gtk_object_set(GTK_OBJECT(view),
 		       "empty_message", _("\n\nThere are no items to show in this view\n\n"
@@ -134,15 +150,41 @@ e_minicard_view_init (EMinicardView *view)
 	E_REFLOW_SORTED(view)->string_func  = (EReflowStringFunc) e_minicard_get_card_id;
 }
 
+static gint
+card_selected (EMinicard *card, GdkEvent *event, EMinicardView *view)
+{
+	int i = 0;
+	GList *item;
+	for (item = E_REFLOW(view)->items; item->data != card; item = item->next, i++)
+		/* Empty for loop */;
+	switch(event->type) {
+	case GDK_BUTTON_PRESS:
+		e_selection_model_do_something(E_SELECTION_MODEL(view->selection), i, 0, event->button.state);
+		return TRUE;
+		break;
+	default:
+		e_selection_model_do_something(E_SELECTION_MODEL(view->selection), i, 0, 0);
+		return FALSE;
+		break;
+	}
+}
+
 static void
 create_card(EBookView *book_view, const GList *cards, EMinicardView *view)
 {
 	for (; cards; cards = g_list_next(cards)) {
+		int position;
 		GnomeCanvasItem *item = gnome_canvas_item_new(GNOME_CANVAS_GROUP(view),
 							      e_minicard_get_type(),
 							      "card", cards->data,
+							      "editable", view->editable,
 							      NULL);
-		e_reflow_add_item(E_REFLOW(view), item);
+		gtk_signal_connect(GTK_OBJECT(item), "selected",
+				   GTK_SIGNAL_FUNC(card_selected), view);
+
+		e_reflow_add_item(E_REFLOW(view), item, &position);
+
+		e_selection_model_simple_insert_rows(view->selection, position, 1);
 	}
 }
 
@@ -152,12 +194,18 @@ modify_card(EBookView *book_view, const GList *cards, EMinicardView *view)
 	for (; cards; cards = g_list_next(cards)) {
 		ECard *card = cards->data;
 		gchar *id = e_card_get_id(card);
-		GnomeCanvasItem *item = e_reflow_sorted_get_item(E_REFLOW_SORTED(view), id);
+		GnomeCanvasItem *item = e_reflow_sorted_get_item(E_REFLOW_SORTED(view), id, NULL);
 		if (item && !GTK_OBJECT_DESTROYED(item)) {
+			int old_pos;
+			int new_pos;
+
 			gnome_canvas_item_set(item,
 					      "card", card,
 					      NULL);
-			e_reflow_sorted_reorder_item(E_REFLOW_SORTED(view), id);
+
+			e_reflow_sorted_reorder_item(E_REFLOW_SORTED(view), id, &old_pos, &new_pos);
+
+			e_selection_model_simple_move_row(view->selection, old_pos, new_pos);
 		}
 	}
 }
@@ -175,7 +223,9 @@ status_message (EBookView *book_view,
 static void
 remove_card(EBookView *book_view, const char *id, EMinicardView *view)
 {
-	e_reflow_sorted_remove_item(E_REFLOW_SORTED(view), id);
+	int position;
+	e_reflow_sorted_remove_item(E_REFLOW_SORTED(view), id, &position);
+	e_selection_model_simple_delete_rows(view->selection, position, 1);
 }
 
 static void
@@ -197,22 +247,26 @@ book_view_loaded (EBook *book, EBookStatus status, EBookView *book_view, gpointe
 		gtk_object_ref(GTK_OBJECT(view->book_view));
 	
 
-	view->create_card_id = gtk_signal_connect(GTK_OBJECT(view->book_view),
-						  "card_added",
-						  GTK_SIGNAL_FUNC(create_card),
-						  view);
-	view->remove_card_id = gtk_signal_connect(GTK_OBJECT(view->book_view),
-						  "card_removed",
-						  GTK_SIGNAL_FUNC(remove_card),
-						  view);
-	view->modify_card_id = gtk_signal_connect(GTK_OBJECT(view->book_view),
-						  "card_changed",
-						  GTK_SIGNAL_FUNC(modify_card),
-						  view);
-	view->status_message_id = gtk_signal_connect(GTK_OBJECT(view->book_view),
-						     "status_message",
-						     GTK_SIGNAL_FUNC(status_message),
-						     view);
+	view->create_card_id =
+		gtk_signal_connect(GTK_OBJECT(view->book_view),
+				   "card_added",
+				   GTK_SIGNAL_FUNC(create_card),
+				   view);
+	view->remove_card_id =
+		gtk_signal_connect(GTK_OBJECT(view->book_view),
+				   "card_removed",
+				   GTK_SIGNAL_FUNC(remove_card),
+				   view);
+	view->modify_card_id =
+		gtk_signal_connect(GTK_OBJECT(view->book_view),
+				   "card_changed",
+				   GTK_SIGNAL_FUNC(modify_card),
+				   view);
+	view->status_message_id =
+		gtk_signal_connect(GTK_OBJECT(view->book_view),
+				   "status_message",
+				   GTK_SIGNAL_FUNC(status_message),
+				   view);
 
 	g_list_foreach(E_REFLOW(view)->items, (GFunc) gtk_object_unref, NULL);
 	g_list_foreach(E_REFLOW(view)->items, (GFunc) gtk_object_destroy, NULL);
@@ -269,6 +323,17 @@ e_minicard_view_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 		if (view->get_view_idle == 0)
 			view->get_view_idle = g_idle_add((GSourceFunc)get_view, view);
 		break;
+	case ARG_EDITABLE: {
+		GList *l;
+		view->editable = GTK_VALUE_BOOL (*arg);
+		/* bit of a hack */
+		for (l = E_REFLOW (view)->items; l; l = g_list_next(l)) {
+			gtk_object_set (GTK_OBJECT (l->data),
+					"editable", view->editable,
+					NULL);
+		}
+		break;
+	}
 	}
 }
 
@@ -286,6 +351,9 @@ e_minicard_view_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 	case ARG_QUERY:
 		GTK_VALUE_STRING (*arg) = g_strdup(e_minicard_view->query);
 		break;
+	case ARG_EDITABLE:
+		GTK_VALUE_BOOL (*arg) = e_minicard_view->editable;
+		break;
 	default:
 		arg->type = GTK_TYPE_INVALID;
 		break;
@@ -352,6 +420,26 @@ editor_closed_cb (EContactEditor *ce, gpointer data)
 	gtk_object_unref (GTK_OBJECT (ce));
 }
 
+static void
+supported_fields_cb (EBook *book, EBookStatus status, EList *fields, EMinicardView *view)
+{
+	ECard *card;
+	EContactEditor *ce;
+
+	card = e_card_new("");
+
+	ce = e_contact_editor_new (card, TRUE, fields, !view->editable);
+
+	gtk_signal_connect (GTK_OBJECT (ce), "add_card",
+			    GTK_SIGNAL_FUNC (add_card_cb), book);
+	gtk_signal_connect (GTK_OBJECT (ce), "commit_card",
+			    GTK_SIGNAL_FUNC (commit_card_cb), book);
+	gtk_signal_connect (GTK_OBJECT (ce), "editor_closed",
+			    GTK_SIGNAL_FUNC (editor_closed_cb), NULL);
+
+	gtk_object_sink(GTK_OBJECT(card));
+}
+
 static gboolean
 e_minicard_view_event (GnomeCanvasItem *item, GdkEvent *event)
 {
@@ -363,25 +451,15 @@ e_minicard_view_event (GnomeCanvasItem *item, GdkEvent *event)
 	case GDK_2BUTTON_PRESS:
 		if (((GdkEventButton *)event)->button == 1)
 		{
-			ECard *card;
-			EContactEditor *ce;
 			EBook *book;
 
-			card = e_card_new("");
-
 			gtk_object_get(GTK_OBJECT(view), "book", &book, NULL);
+
 			g_assert (E_IS_BOOK (book));
 
-			ce = e_contact_editor_new (card, TRUE);
-
-			gtk_signal_connect (GTK_OBJECT (ce), "add_card",
-					    GTK_SIGNAL_FUNC (add_card_cb), book);
-			gtk_signal_connect (GTK_OBJECT (ce), "commit_card",
-					    GTK_SIGNAL_FUNC (commit_card_cb), book);
-			gtk_signal_connect (GTK_OBJECT (ce), "editor_closed",
-					    GTK_SIGNAL_FUNC (editor_closed_cb), NULL);
-
-			gtk_object_sink(GTK_OBJECT(card));
+			e_book_get_supported_fields (book,
+						     (EBookFieldsCallback)supported_fields_cb,
+						     view);
 		}
 		return TRUE;
 	default:
@@ -487,3 +565,17 @@ e_minicard_view_stop             (EMinicardView *view)
 		gtk_object_unref(GTK_OBJECT(view->book_view));
 	view->book_view = NULL;
 }
+
+static void
+e_minicard_view_update_selection (EMinicardView *view)
+{
+	int i;
+	GList *item;
+
+	for (i = 0, item = E_REFLOW(view)->items; item; item = item->next, i++) {
+		if (E_IS_MINICARD(item->data))
+			gtk_object_set(GTK_OBJECT(item->data),
+				       "selected", e_selection_model_is_row_selected(E_SELECTION_MODEL(view->selection), i),
+				       NULL);
+	}
+}
