@@ -22,6 +22,7 @@
 #define __E_MINICARD_H__
 
 #include <gnome.h>
+#include "addressbook/contact-editor/e-contact-editor.h"
 #include "addressbook/backend/ebook/e-card.h"
 #include "addressbook/backend/ebook/e-card-simple.h"
 
@@ -69,13 +70,19 @@ struct _EMinicard
 	GnomeCanvasItem *header_rect;
 	GnomeCanvasItem *header_text;
 
+	EContactEditor *editor;
+
 	GList *fields; /* Of type EMinicardField */
 	guint needs_remodeling : 1;
 
 	guint changed : 1;
+
+	guint selected : 1;
 	
-	gboolean has_focus;
-	
+	guint has_focus : 1;
+
+	guint editable : 1;
+
 	double width;
 	double height;
 };
@@ -84,7 +91,7 @@ struct _EMinicardClass
 {
 	GnomeCanvasGroupClass parent_class;
 
-	void (* resize) (EMinicard *minicard);
+	gint (* selected) (EMinicard *minicard, GdkEvent *event);
 };
 
 
@@ -92,6 +99,8 @@ GtkType    e_minicard_get_type    (void);
 char      *e_minicard_get_card_id (EMinicard *minicard);
 int        e_minicard_compare     (EMinicard *minicard1, EMinicard *minicard2);
 
+int        e_minicard_selected    (EMinicard *minicard, GdkEvent *event);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
