@@ -1,9 +1,11 @@
 /* Miscellaneous time-related utilities
  *
  * Copyright (C) 1998 The Free Software Foundation
+ * Copyright (C) 2000 Helix Code, Inc.
+ * Copyright (C) 2000 Ximian, Inc.
  *
- * Authors: Federico Mena <federico@nuclecu.unam.mx>
- *          Miguel de Icaza <miguel@nuclecu.unam.mx>
+ * Authors: Federico Mena <federico@ximian.com>
+ *          Miguel de Icaza <miguel@ximian.com>
  */
 
 #ifndef TIMEUTIL_H
@@ -14,12 +16,7 @@
 #include <ical.h>
 
 
-time_t time_from_isodate        (char *str);
-time_t time_from_start_duration (time_t start, char *duration);
 char   *isodate_from_time_t     (time_t t);
-int    get_time_t_hour          (time_t t);
-int    isodiff_to_secs          (char *str);
-char   *isodiff_from_secs       (int secs);
 
 time_t time_add_minutes (time_t time, int minutes);
 time_t time_add_day (time_t time, int days);
@@ -28,12 +25,6 @@ time_t time_add_month (time_t time, int months);
 time_t time_add_year (time_t time, int years);
 
 
-/* Returns pointer to a statically-allocated buffer with a string of the form
- * 3am, 4am, 12pm, 08h, 17h, etc.
- * The string is internationalized, hopefully correctly.
- */
-char *format_simple_hour (int hour, int use_am_pm);
-
 /* Returns the number of days in the specified month.  Years are full years (starting from year 1).
  * Months are in [0, 11].
  */
@@ -44,8 +35,6 @@ int time_days_in_month (int year, int month);
  */
 time_t time_from_day (int year, int month, int day);
 
-time_t time_day_hour     (time_t t, int hour);
-
 /* For the functions below, time ranges are considered to contain the start time, but not the end
  * time.
  */
@@ -74,7 +63,6 @@ time_t time_week_end (time_t t);
 time_t time_day_begin (time_t t);
 time_t time_day_end (time_t t);
 
-time_t parse_date (char *str);
 void print_time_t (time_t t);
 
 
