@@ -23,16 +23,9 @@
  */
 
 #include <config.h>
-#include <sys/stat.h>
-#include <utime.h>
-#include <unistd.h>
-#include <pwd.h>
-#include <signal.h>
-#include <errno.h>
 
 #include <liboaf/liboaf.h>
 #include <bonobo.h>
-#include <gnome-xml/parser.h>
 #include <cal-client/cal-client-types.h>
 #include <cal-client/cal-client.h>
 #include <cal-util/timeutil.h>
@@ -40,8 +33,8 @@
 #include <pi-socket.h>
 #include <pi-file.h>
 #include <pi-dlp.h>
-#include <pi-version.h>
 #include <libical/src/libical/icaltypes.h>
+#include <e-pilot-util.h>
 
 #define TODO_CONFIG_LOAD 1
 #define TODO_CONFIG_DESTROY 1
@@ -54,7 +47,7 @@
 GnomePilotConduit * conduit_get_gpilot_conduit (guint32);
 void conduit_destroy_gpilot_conduit (GnomePilotConduit*);
 
-#define CONDUIT_VERSION "0.1.2"
+#define CONDUIT_VERSION "0.1.4"
 #ifdef G_LOG_DOMAIN
 #undef G_LOG_DOMAIN
 #endif
@@ -84,13 +77,15 @@ print_local (EToDoLocalRecord *local)
 	}
 
 	if (local->todo && local->todo->description) {
-		sprintf (buff, "[%d %ld %d %d '%s' '%s']",
-			 local->todo->indefinite,
-			 mktime (& local->todo->due),
-			 local->todo->priority,
-			 local->todo->complete,
-			 local->todo->description,
-			 local->todo->note);
+		g_snprintf (buff, 4096, "[%d %ld %d %d '%s' '%s']",
+			    local->todo->indefinite,
+			    mktime (& local->todo->due),
+			    local->todo->priority,
+			    local->todo->complete,
+			    local->todo->description ?
+			    local->todo->description : "",
+			    local->todo->note ?
+			    local->todo->note : "");
 		return buff;
 	}
 
@@ -110,68 +105,84 @@ static char *print_remote (GnomePilotRecord *remote)
 	memset (&todo, 0, sizeof (struct ToDo));
 	unpack_ToDo (&todo, remote->record, remote->length);
 
-	sprintf (buff, "[%d %ld %d %d '%s' '%s']",
-		 todo.indefinite,
-		 mktime (& todo.due),
-		 todo.priority,
-		 todo.complete,
-		 todo.description,
-		 todo.note);
+	g_snprintf (buff, 4096, "[%d %ld %d %d '%s' '%s']",
+		    todo.indefinite,
+		    mktime (&todo.due),
+		    todo.priority,
+		    todo.complete,
+		    todo.description ?
+		    todo.description : "",
+		    todo.note ?
+		    todo.note : "");
 
 	return buff;
 }
 
 /* Context Routines */
-static void
-e_todo_context_new (EToDoConduitContext **ctxt, guint32 pilot_id) 
+static EToDoConduitContext *
+e_todo_context_new (guint32 pilot_id) 
 {
-	*ctxt = g_new0 (EToDoConduitContext,1);
-	g_assert (ctxt!=NULL);
+	EToDoConduitContext *ctxt = g_new0 (EToDoConduitContext, 1);
 
-	todoconduit_load_configuration (&(*ctxt)->cfg, pilot_id);
+	todoconduit_load_configuration (&ctxt->cfg, pilot_id);
+
+	return ctxt;
 }
 
 static void
-e_todo_context_destroy (EToDoConduitContext **ctxt)
+e_todo_context_foreach_change (gpointer key, gpointer value, gpointer data) 
 {
-	g_return_if_fail (ctxt!=NULL);
-	g_return_if_fail (*ctxt!=NULL);
+	g_free (key);
+}
 
-	if ((*ctxt)->client != NULL)
-		gtk_object_unref (GTK_OBJECT ((*ctxt)->client));
+static void
+e_todo_context_destroy (EToDoConduitContext *ctxt)
+{
+	g_return_if_fail (ctxt != NULL);
 
-	if ((*ctxt)->cfg != NULL)
-		todoconduit_destroy_configuration (&(*ctxt)->cfg);
+	if (ctxt->cfg != NULL)
+		todoconduit_destroy_configuration (&ctxt->cfg);
 
-	g_free (*ctxt);
-	*ctxt = NULL;
+	if (ctxt->client != NULL)
+		gtk_object_unref (GTK_OBJECT (ctxt->client));
+
+	if (ctxt->calendar_file)
+		g_free (ctxt->calendar_file);
+
+	if (ctxt->uids)
+		cal_obj_uid_list_free (ctxt->uids);
+
+	if (ctxt->changed_hash)
+		g_hash_table_foreach (ctxt->changed_hash, e_todo_context_foreach_change, NULL);
+
+	if (ctxt->changed)
+		cal_client_change_list_free (ctxt->changed);
+	
+	if (ctxt->map)
+		e_pilot_map_destroy (ctxt->map);
+
+	g_free (ctxt);
 }
 
 /* Calendar Server routines */
 static void
-start_calendar_server_cb (GtkWidget *cal_client,
-			  CalClientLoadStatus status,
-			  EToDoConduitContext *ctxt)
+start_calendar_server_cb (CalClient *cal_client,
+			  CalClientOpenStatus status,
+			  gpointer data)
 {
-	CalClient *client = CAL_CLIENT (cal_client);
+	EToDoConduitContext *ctxt;
 
-	LOG ("  entering start_calendar_server_load_cb, tried=%d\n",
-		ctxt->calendar_load_tried);
+	ctxt = data;
 
-	if (status == CAL_CLIENT_LOAD_SUCCESS) {
-		ctxt->calendar_load_success = TRUE;
+	LOG ("  entering start_calendar_server_cb\n");
+
+	if (status == CAL_CLIENT_OPEN_SUCCESS) {
+		ctxt->calendar_open_success = TRUE;
 		LOG ("    success\n");
-		gtk_main_quit (); /* end the sub event loop */
-	} else {
-		if (ctxt->calendar_load_tried) {
-			LOG ("    load and create of calendar failed\n");
-			gtk_main_quit (); /* end the sub event loop */
-			return;
-		}
+	} else
+		LOG ("    open of calendar failed\n");
 
-		cal_client_create_calendar (client, ctxt->calendar_file);
-		ctxt->calendar_load_tried = TRUE;
-	}
+	gtk_main_quit (); /* end the sub event loop */
 }
 
 static int
@@ -184,19 +195,20 @@ start_calendar_server (EToDoConduitContext *ctxt)
 
 	/* FIX ME */
 	ctxt->calendar_file = g_concat_dir_and_file (g_get_home_dir (),
-			       "evolution/local/Calendar/calendar.ics");
+			       "evolution/local/Tasks/tasks.ics");
 
-	gtk_signal_connect (GTK_OBJECT (ctxt->client), "cal_loaded",
+	gtk_signal_connect (GTK_OBJECT (ctxt->client), "cal_opened",
 			    start_calendar_server_cb, ctxt);
 
-	LOG ("    calling cal_client_load_calendar\n");
-	cal_client_load_calendar (ctxt->client, ctxt->calendar_file);
+	LOG ("    calling cal_client_open_calendar\n");
+	if (!cal_client_open_calendar (ctxt->client, ctxt->calendar_file, FALSE))
+		return -1;
 
 	/* run a sub event loop to turn cal-client's async load
 	   notification into a synchronous call */
 	gtk_main ();
 
-	if (ctxt->calendar_load_success)
+	if (ctxt->calendar_open_success)
 		return 0;
 
 	return -1;
@@ -208,11 +220,21 @@ map_name (EToDoConduitContext *ctxt)
 {
 	char *filename;
 	
-	filename = g_strdup_printf ("%s/evolution/local/Calendar/pilot-map-todo-%d.xml", g_get_home_dir (), ctxt->cfg->pilot_id);
+	filename = g_strdup_printf ("%s/evolution/local/Tasks/pilot-map-todo-%d.xml", g_get_home_dir (), ctxt->cfg->pilot_id);
 
 	return filename;
 }
 
+static gboolean
+is_empty_time (struct tm time) 
+{
+	if (time.tm_sec || time.tm_min || time.tm_hour 
+	    || time.tm_mday || time.tm_mon || time.tm_year) 
+		return FALSE;
+	
+	return TRUE;
+}
+
 static GList *
 next_changed_item (EToDoConduitContext *ctxt, GList *changes) 
 {
@@ -306,6 +328,7 @@ local_record_from_comp (EToDoLocalRecord *local, CalComponent *comp, EToDoCondui
 	g_return_if_fail (comp != NULL);
 
 	local->comp = comp;
+	gtk_object_ref (GTK_OBJECT (comp));
 
 	cal_component_get_uid (local->comp, &uid);
 	local->local.ID = e_pilot_map_lookup_pid (ctxt->map, uid);
@@ -318,13 +341,13 @@ local_record_from_comp (EToDoLocalRecord *local, CalComponent *comp, EToDoCondui
 	   uses free to deallocate */
 	cal_component_get_summary (comp, &summary);
 	if (summary.value) 
-		local->todo->description = strdup ((char *) summary.value);
+		local->todo->description = e_pilot_utf8_to_pchar (summary.value);
 
 	cal_component_get_description_list (comp, &d_list);
 	if (d_list) {
 		description = (CalComponentText *) d_list->data;
 		if (description && description->value)
-			local->todo->note = strdup (description->value);
+			local->todo->note = e_pilot_utf8_to_pchar (description->value);
 		else
 			local->todo->note = NULL;
 	} else {
@@ -395,11 +418,12 @@ comp_from_remote_record (GnomePilotConduitSyncAbs *conduit,
 {
 	CalComponent *comp;
 	struct ToDo todo;
-	struct icaltimetype now = icaltime_from_timet (time (NULL), FALSE, TRUE);
+	struct icaltimetype now = icaltime_from_timet (time (NULL), FALSE);
 	CalComponentText summary = {NULL, NULL};
 	CalComponentDateTime dt = {NULL, NULL};
 	struct icaltimetype due;
-
+	char *txt;
+	
 	g_return_val_if_fail (remote != NULL, NULL);
 
 	memset (&todo, 0, sizeof (struct ToDo));
@@ -415,9 +439,10 @@ comp_from_remote_record (GnomePilotConduitSyncAbs *conduit,
 
 	cal_component_set_last_modified (comp, &now);
 
-	summary.value = todo.description;
+	summary.value = txt = e_pilot_utf8_from_pchar (todo.description);
 	cal_component_set_summary (comp, &summary);
-
+	free (txt);
+	
 	/* The iCal description field */
 	if (!todo.note) {
 		cal_component_set_comment_list (comp, NULL);
@@ -425,12 +450,13 @@ comp_from_remote_record (GnomePilotConduitSyncAbs *conduit,
 		GSList l;
 		CalComponentText text;
 
-		text.value = todo.note;
+		text.value = txt = e_pilot_utf8_from_pchar (todo.note);
 		text.altrep = NULL;
 		l.data = &text;
 		l.next = NULL;
 
 		cal_component_set_description_list (comp, &l);
+		free (txt);
 	} 
 
 	if (todo.complete) {
@@ -439,12 +465,8 @@ comp_from_remote_record (GnomePilotConduitSyncAbs *conduit,
 		cal_component_set_percent (comp, &percent);
 	}
 
-	/* FIX ME This is a bit hackish, how else can we tell if there is
-	 * no due date set?
-	 */
-	if (todo.due.tm_sec || todo.due.tm_min || todo.due.tm_hour 
-	    || todo.due.tm_mday || todo.due.tm_mon || todo.due.tm_year) {
-		due = icaltime_from_timet (mktime (&todo.due), FALSE, TRUE);
+	if (!is_empty_time (todo.due)) {
+		due = icaltime_from_timet (mktime (&todo.due), FALSE);
 		dt.value = &due;
 		cal_component_set_due (comp, &dt);
 	}
@@ -602,7 +624,6 @@ post_sync (GnomePilotConduit *conduit,
 
 	filename = map_name (ctxt);
 	e_pilot_map_write (filename, ctxt->map);
-	e_pilot_map_destroy (ctxt->map);
 	g_free (filename);
 
 	/* FIX ME ugly hack - our changes musn't count, this does introduce
@@ -717,8 +738,6 @@ for_each_modified (GnomePilotConduitSyncAbs *conduit,
 		if (iterator != NULL) {
 			CalClientChange *ccc = iterator->data;
 		
-			LOG ("iterating over %d records", g_hash_table_size (ctxt->changed_hash));
-
 			*local = g_new0 (EToDoLocalRecord, 1);
 			local_record_from_comp (*local, ccc->comp, ctxt);
 		} else {
@@ -834,6 +853,7 @@ delete_record (GnomePilotConduitSyncAbs *conduit,
 
 	LOG ("delete_record: deleting %s\n", uid);
 
+	e_pilot_map_remove_by_uid (ctxt->map, uid);
 	cal_client_remove_object (ctxt->client, uid);
 	
         return 0;
@@ -953,7 +973,7 @@ conduit_get_gpilot_conduit (guint32 pilot_id)
 	gnome_pilot_conduit_construct (GNOME_PILOT_CONDUIT (retval),
 				       "e_todo_conduit");
 
-	e_todo_context_new (&ctxt, pilot_id);
+	ctxt = e_todo_context_new (pilot_id);
 	gtk_object_set_data (GTK_OBJECT (retval), "todoconduit_context", ctxt);
 
 	gtk_signal_connect (retval, "pre_sync", (GtkSignalFunc) pre_sync, ctxt);
@@ -982,12 +1002,11 @@ conduit_get_gpilot_conduit (guint32 pilot_id)
 void
 conduit_destroy_gpilot_conduit (GnomePilotConduit *conduit)
 { 
+	GtkObject *obj = GTK_OBJECT (conduit);
 	EToDoConduitContext *ctxt;
+	
+	ctxt = gtk_object_get_data (obj, "todoconduit_context");
+	e_todo_context_destroy (ctxt);
 
-	ctxt = gtk_object_get_data (GTK_OBJECT (conduit), 
-				    "todoconduit_context");
-
-	e_todo_context_destroy (&ctxt);
-
-	gtk_object_destroy (GTK_OBJECT (conduit));
+	gtk_object_destroy (obj);
 }
