@@ -23,7 +23,6 @@
  */
 
 #include <config.h>
-#include <bonobo.h>
 #include "evolution-shell-component.h"
 #include <executive-summary/evolution-services/executive-summary-component.h>
 #include "component-factory.h"
@@ -145,6 +144,7 @@ factory_fn (BonoboGenericFactory *factory,
 							 NULL, /* remove_folder_fn */
 							 NULL, /* copy_folder_fn */
 							 NULL, /* populate_folder_context_menu_fn */
+							 NULL, /* get_dnd_selection_fn */
 							 NULL  /* closure */);
 
 	gtk_signal_connect (GTK_OBJECT (shell_component), "owner_set",
