@@ -2,9 +2,10 @@
 
 /* 
  * Author : 
- *  Damon Chaplin <damon@helixcode.com>
+ *  Damon Chaplin <damon@ximian.com>
  *
  * Copyright 2000, Helix Code, Inc.
+ * Copyright 2000, Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -27,6 +28,7 @@
 #include <gtk/gtkobject.h>
 #include <libgnome/gnome-defs.h>
 #include <bonobo.h>
+#include <cal-client/cal-client.h>
 
 BEGIN_GNOME_DECLS
 
@@ -52,15 +54,15 @@ struct _TaskEditorClass
 	GtkObjectClass parent_class;
 };
 
+GtkType     task_editor_get_type        (void);
+TaskEditor* task_editor_construct       (TaskEditor   *tedit);
+TaskEditor* task_editor_new             (void);
+void        task_editor_set_cal_client  (TaskEditor   *tedit,
+					 CalClient    *client);
+void        task_editor_set_todo_object (TaskEditor   *tedit,
+					 CalComponent *comp);
+void        task_editor_focus           (TaskEditor   *tedit);
 
-GtkType	    task_editor_get_type	(void);
-TaskEditor* task_editor_construct	(TaskEditor	*tedit);
-TaskEditor* task_editor_new		(void);
-
-void	    task_editor_set_cal_client	(TaskEditor	*tedit,
-					 CalClient	*client);
-void	    task_editor_set_todo_object	(TaskEditor	*tedit,
-					 CalComponent	*comp);
 
 
 END_GNOME_DECLS
