@@ -2,9 +2,10 @@
 
 /* 
  * Author : 
- *  Damon Chaplin <damon@helixcode.com>
+ *  Damon Chaplin <damon@ximian.com>
  *
  * Copyright 1999, Helix Code, Inc.
+ * Copyright 1999, Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -135,11 +136,20 @@ typedef enum
 {
 	E_DAY_VIEW_COLOR_BG_WORKING,
 	E_DAY_VIEW_COLOR_BG_NOT_WORKING,
-	E_DAY_VIEW_COLOR_EVENT_VBAR,
+	E_DAY_VIEW_COLOR_BG_SELECTED,
+	E_DAY_VIEW_COLOR_BG_GRID,
 
+	E_DAY_VIEW_COLOR_BG_TOP_CANVAS,
+	E_DAY_VIEW_COLOR_BG_TOP_CANVAS_SELECTED,
+	E_DAY_VIEW_COLOR_BG_TOP_CANVAS_GRID,
+
+	E_DAY_VIEW_COLOR_EVENT_VBAR,
 	E_DAY_VIEW_COLOR_EVENT_BACKGROUND,
 	E_DAY_VIEW_COLOR_EVENT_BORDER,
-	
+
+	E_DAY_VIEW_COLOR_LONG_EVENT_BACKGROUND,
+	E_DAY_VIEW_COLOR_LONG_EVENT_BORDER,
+
 	E_DAY_VIEW_COLOR_LAST
 } EDayViewColors;
 
