@@ -26,9 +26,9 @@
 #ifndef _E_TASKS_H_
 #define _E_TASKS_H_
 
+#include <gtk/gtktable.h>
 #include <cal-client/cal-client.h>
 
-
 #define E_TYPE_TASKS            (e_tasks_get_type ())
 #define E_TASKS(obj)            (GTK_CHECK_CAST ((obj), E_TYPE_TASKS, ETasks))
 #define E_TASKS_CLASS(klass)    (GTK_CHECK_CAST_CLASS ((klass), E_TYPE_TASKS, \
@@ -52,17 +52,24 @@ struct _ETasksClass {
 };
 
 
-GtkType    e_tasks_get_type         	(void);
-GtkWidget *e_tasks_construct		(ETasks		*tasks);
+GtkType    e_tasks_get_type        (void);
+GtkWidget *e_tasks_construct       (ETasks            *tasks);
 
-GtkWidget *e_tasks_new			(void);
+GtkWidget *e_tasks_new             (void);
 
-gboolean   e_tasks_open			(ETasks		*tasks,
-					 char		*file);
+gboolean   e_tasks_open            (ETasks            *tasks,
+				    char              *file);
 
-CalClient *e_tasks_get_cal_client	(ETasks		*tasks);
+CalClient *e_tasks_get_cal_client  (ETasks            *tasks);
 
-void	   e_tasks_new_task		(ETasks		*tasks);
+void       e_tasks_new_task        (ETasks            *tasks);
 
+void       e_tasks_setup_menus     (ETasks            *tasks,
+				    BonoboUIComponent *uic);
+
+
+/* This updates all the preference settings for all the ETasks widgets in use.
+ */
+void	   e_tasks_update_all_config_settings	(void);
 
 #endif /* _E_TASKS_H_ */
