@@ -25,6 +25,10 @@
  */
 
 #include <config.h>
+#include <string.h>
+#include <glib.h>
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-i18n.h>
 #include <glade/glade.h>
 #include <e-util/e-dialog-widgets.h>
 #include <widgets/misc/e-dateedit.h>
@@ -231,6 +235,8 @@ static void reminder_delete_cb (GtkWidget *widget, EventEditor *ee);
 static void recurrence_exception_add_cb (GtkWidget *widget, EventEditor *ee);
 static void recurrence_exception_modify_cb (GtkWidget *widget, EventEditor *ee);
 static void recurrence_exception_delete_cb (GtkWidget *widget, EventEditor *ee);
+static void recurrence_exception_select_row_cb (GtkCList *clist, gint row, gint col, GdkEvent *event,
+						gpointer data);
 static void field_changed		(GtkWidget	*widget,
 					 EventEditor	*ee);
 static void event_editor_set_changed	(EventEditor	*ee,
@@ -1223,13 +1229,17 @@ init_widgets (EventEditor *ee)
 	gtk_signal_connect (GTK_OBJECT (priv->recurrence_exception_delete), "clicked",
 			    GTK_SIGNAL_FUNC (recurrence_exception_delete_cb), ee);
 
+	/* Selections in the exceptions list */
+
+	gtk_signal_connect (GTK_OBJECT (priv->recurrence_exception_list), "select_row",
+			    GTK_SIGNAL_FUNC (recurrence_exception_select_row_cb), ee);
 
 	/*
 	 * Connect the default signal handler to use to make sure the "changed"
 	 * field gets set whenever a field is changed.
 	 */
 
-	/* General Page. */
+	/* Appointment Page */
 	gtk_signal_connect (GTK_OBJECT (priv->general_summary), "changed",
 			    GTK_SIGNAL_FUNC (field_changed), ee);
 	gtk_signal_connect (GTK_OBJECT (priv->description), "changed",
@@ -1243,9 +1253,6 @@ init_widgets (EventEditor *ee)
 	gtk_signal_connect (GTK_OBJECT (priv->classification_confidential),
 			    "toggled",
 			    GTK_SIGNAL_FUNC (field_changed), ee);
-
-
-	/* Recurrence Page. */
 }
 
 static const int classification_map[] = {
@@ -1255,27 +1262,6 @@ static const int classification_map[] = {
 	-1
 };
 
-#if 0
-static const int alarm_unit_map[] = {
-	ALARM_MINUTES,
-	ALARM_HOURS,
-	ALARM_DAYS,
-	-1
-};
-
-static void
-alarm_unit_set (GtkWidget *widget, enum AlarmUnit unit)
-{
-	e_dialog_option_menu_set (widget, unit, alarm_unit_map);
-}
-
-static enum AlarmUnit
-alarm_unit_get (GtkWidget *widget)
-{
-	return e_dialog_option_menu_get (widget, alarm_unit_map);
-}
-#endif
-
 static const int month_pos_map[] = { 0, 1, 2, 3, 4, -1 };
 static const int weekday_map[] = { 0, 1, 2, 3, 4, 5, 6, -1 };
 
@@ -1468,6 +1454,7 @@ fill_exception_widgets (EventEditor *ee)
 {
 	EventEditorPrivate *priv;
 	GSList *list, *l;
+	gboolean added;
 
 	priv = ee->priv;
 	g_assert (priv->comp != NULL);
@@ -1476,16 +1463,23 @@ fill_exception_widgets (EventEditor *ee)
 
 	cal_component_get_exdate_list (priv->comp, &list);
 
+	added = FALSE;
+
 	for (l = list; l; l = l->next) {
 		CalComponentDateTime *cdt;
 		time_t ext;
 
+		added = TRUE;
+
 		cdt = l->data;
 		ext = icaltime_as_timet (*cdt->value);
 		append_exception (ee, ext);
 	}
 
 	cal_component_free_exdate_list (list);
+
+	if (added)
+		gtk_clist_select_row (GTK_CLIST (priv->recurrence_exception_list), 0, 0);
 }
 
 /* Computes a weekday mask for the start day of a calendar component, for use in
@@ -2336,6 +2330,7 @@ recur_to_comp_object (EventEditor *ee, CalComponent *comp)
 		cdt->tzid = NULL;
 
 		tim = gtk_clist_get_row_data (exception_list, i);
+		g_assert (tim != NULL);
 		*cdt->value = icaltime_from_timet (*tim, FALSE);
 
 		list = g_slist_prepend (list, cdt);
@@ -2477,7 +2472,6 @@ close_dialog (EventEditor *ee)
 
 
 
-
 /* Callback used when the dialog box is "applied" */
 static void
 ee_apply_event_cb (GtkWidget *widget, gint page_num, gpointer data)
@@ -3022,7 +3016,7 @@ append_reminder (EventEditor *ee, CalComponentAlarm *alarm, ReminderStatus statu
 	clist = GTK_CLIST (priv->reminder_list);
 
 	c[0] = get_alarm_string (alarm);
-	i = e_utf8_gtk_clist_append (clist, c);
+	i = gtk_clist_append (clist, c);
 
 	rdata = g_new (ReminderData, 1);
 	rdata->status = status;
@@ -3141,11 +3135,17 @@ append_exception (EventEditor *ee, time_t t)
 
 	clist = GTK_CLIST (priv->recurrence_exception_list);
 
+	gtk_signal_handler_block_by_data (GTK_OBJECT (clist), ee);
+
 	c[0] = get_exception_string (t);
-	i = e_utf8_gtk_clist_append (clist, c);
+	i = gtk_clist_append (clist, c);
 
 	gtk_clist_set_row_data (clist, i, tt);
+
 	gtk_clist_select_row (clist, i, 0);
+	gtk_signal_handler_unblock_by_data (GTK_OBJECT (clist), ee);
+
+	e_date_edit_set_time (E_DATE_EDIT (priv->recurrence_exception_date), t);
 
 	gtk_widget_set_sensitive (priv->recurrence_exception_modify, TRUE);
 	gtk_widget_set_sensitive (priv->recurrence_exception_delete, TRUE);
@@ -3189,7 +3189,7 @@ recurrence_exception_modify_cb (GtkWidget *widget, EventEditor *ee)
 	t = gtk_clist_get_row_data (clist, sel);
 	*t = e_date_edit_get_time (E_DATE_EDIT (priv->recurrence_exception_date));
 
-	e_utf8_gtk_clist_set_text (clist, sel, 0, get_exception_string (*t));
+	gtk_clist_set_text (clist, sel, 0, get_exception_string (*t));
 
 	preview_recur (ee);
 }
@@ -3201,6 +3201,7 @@ recurrence_exception_delete_cb (GtkWidget *widget, EventEditor *ee)
 	EventEditorPrivate *priv;
 	GtkCList *clist;
 	int sel;
+	time_t *t;
 
 	priv = ee->priv;
 
@@ -3212,7 +3213,9 @@ recurrence_exception_delete_cb (GtkWidget *widget, EventEditor *ee)
 
 	sel = GPOINTER_TO_INT (clist->selection->data);
 
-	g_free (gtk_clist_get_row_data (clist, sel)); /* free the time_t stored there */
+	t = gtk_clist_get_row_data (clist, sel);
+	g_assert (t != NULL);
+	g_free (t);
 
 	gtk_clist_remove (clist, sel);
 	if (sel >= clist->rows)
@@ -3228,6 +3231,25 @@ recurrence_exception_delete_cb (GtkWidget *widget, EventEditor *ee)
 	preview_recur (ee);
 }
 
+/* Callback used when a row is selected in the list of exception dates.  We must
+ * update the date/time widgets to reflect the exception's value.
+ */
+static void
+recurrence_exception_select_row_cb (GtkCList *clist, gint row, gint col, GdkEvent *event,
+				    gpointer data)
+{
+	EventEditor *ee;
+	EventEditorPrivate *priv;
+	time_t *t;
+
+	ee = EVENT_EDITOR (data);
+	priv = ee->priv;
+
+	t = gtk_clist_get_row_data (clist, row);
+	g_assert (t != NULL);
+
+	e_date_edit_set_time (E_DATE_EDIT (priv->recurrence_exception_date), *t);
+}
 
 GtkWidget *
 make_date_edit (void)
