@@ -29,6 +29,11 @@
 #include <signal.h>
 #include <sys/wait.h>
 #include <fcntl.h>
+#include <glib.h>
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-i18n.h>
+#include <libgnomeui/gnome-dialog.h>
+#include <libgnomeui/gnome-dialog-util.h>
 #include <gal/e-paned/e-hpaned.h>
 #include <gal/e-paned/e-vpaned.h>
 #include <cal-util/timeutil.h>
@@ -252,6 +257,7 @@ setup_widgets (GnomeCalendar *gcal)
 
 	/* The ToDo list. */
 	priv->todo = e_calendar_table_new ();
+	calendar_config_configure_e_calendar_table (E_CALENDAR_TABLE (priv->todo));
 	model = e_calendar_table_get_model (E_CALENDAR_TABLE (priv->todo));
 	calendar_model_set_new_comp_vtype (model, CAL_COMPONENT_TODO);
 	e_paned_pack2 (E_PANED (priv->vpane), priv->todo, TRUE, TRUE);
@@ -886,7 +892,7 @@ gnome_calendar_get_cal_client (GnomeCalendar *gcal)
 }
 
 gboolean
-gnome_calendar_open (GnomeCalendar *gcal, char *str_uri)
+gnome_calendar_open (GnomeCalendar *gcal, const char *str_uri)
 {
 	GnomeCalendarPrivate *priv;
 	char *tasks_uri;
@@ -1011,6 +1017,8 @@ gnome_calendar_update_config_settings (GnomeCalendar *gcal,
 
 	calendar_config_configure_e_calendar (E_CALENDAR (priv->date_navigator));
 
+	calendar_config_configure_e_calendar_table (E_CALENDAR_TABLE (priv->todo));
+
 	if (initializing) {
 		priv->hpane_pos = calendar_config_get_hpane_pos ();
 		priv->vpane_pos = calendar_config_get_vpane_pos ();
@@ -1148,23 +1156,30 @@ gnome_calendar_edit_object (GnomeCalendar *gcal, CalComponent *comp)
 /**
  * gnome_calendar_new_appointment:
  * @gcal: An Evolution calendar.
+ * @dtstart: a Unix time_t that marks the beginning of the appointment.
+ * @dtend: a Unix time_t that marks the end of the appointment.
+ * @all_day: if true, the dtstart and dtend are expanded to cover the entire day.
+ *
+ * Opens an event editor dialog for a new appointment.
  *
- * Opens an event editor dialog for a new appointment.  The appointment's start
- * and end times are set to the currently selected time range in the calendar
- * views.
  **/
 void
-gnome_calendar_new_appointment (GnomeCalendar *gcal)
+gnome_calendar_new_appointment_for (GnomeCalendar *cal,
+				    time_t dtstart, time_t dtend,
+				    gboolean all_day)
 {
-	CalComponent *comp;
-	time_t dtstart, dtend;
-	CalComponentDateTime dt;
 	struct icaltimetype itt;
+	CalComponentDateTime dt;
+	CalComponent *comp;
 
-	g_return_if_fail (gcal != NULL);
-	g_return_if_fail (GNOME_IS_CALENDAR (gcal));
+	g_return_if_fail (cal != NULL);
+	g_return_if_fail (GNOME_IS_CALENDAR (cal));
 
-	gnome_calendar_get_current_time_range (gcal, &dtstart, &dtend);
+	if (all_day){
+		dtstart = time_day_begin (dtstart);
+		dtend = time_day_end (dtend);
+	}
+	
 	dt.value = &itt;
 	dt.tzid = NULL;
 
@@ -1179,9 +1194,28 @@ gnome_calendar_new_appointment (GnomeCalendar *gcal)
 
 	cal_component_commit_sequence (comp);
 
-	gnome_calendar_edit_object (gcal, comp);
+	gnome_calendar_edit_object (cal, comp);
 	gtk_object_unref (GTK_OBJECT (comp));
+}
 
+/**
+ * gnome_calendar_new_appointment:
+ * @gcal: An Evolution calendar.
+ *
+ * Opens an event editor dialog for a new appointment.  The appointment's start
+ * and end times are set to the currently selected time range in the calendar
+ * views.
+ **/
+void
+gnome_calendar_new_appointment (GnomeCalendar *gcal)
+{
+	time_t dtstart, dtend;
+
+	g_return_if_fail (gcal != NULL);
+	g_return_if_fail (GNOME_IS_CALENDAR (gcal));
+
+	gnome_calendar_get_current_time_range (gcal, &dtstart, &dtend);
+	gnome_calendar_new_appointment_for (gcal, dtstart, dtend, FALSE);
 }
 
 /* Returns the selected time range for the current view. Note that this may be
