@@ -25,15 +25,18 @@
 #include <config.h>
 #endif
 
-#include <glade/glade.h>
-#include <bonobo.h>
+#include <glib.h>
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-i18n.h>
+#include <libgnomeui/gnome-init.h>
 #include <glade/glade.h>
 
 #include <liboaf/liboaf.h>
+#include <bonobo/bonobo-main.h>
 
 #include <gal/widgets/e-cursors.h>
 
-#include "alarm.h"
+#include "alarm-notify/alarm.h"
 #include "calendar-commands.h"
 #include "calendar-config.h"
 #include "component-factory.h"
