@@ -22,6 +22,15 @@
  *  USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#include <string.h>
+#include <time.h>
+
+#include <glib.h>
+
 #include "camel-filter-driver.h"
 #include "camel-filter-search.h"
 
@@ -29,10 +38,6 @@
 #include "camel-service.h"
 #include "camel-mime-message.h"
 
-#include <glib.h>
-
-#include <time.h>
-
 #include "e-util/e-sexp.h"
 #include "e-util/e-memory.h"
 #include "e-util/e-msgport.h"	/* for edlist */
@@ -71,7 +76,7 @@ struct _CamelFilterDriverPrivate {
 	
 	/* run-time data */
 	GHashTable *folders;       /* folders that message has been copied to */
-	int closed;		/* close count */
+	int closed;		   /* close count */
 	GHashTable *forwards;      /* addresses that have been forwarded the message */
 	
 	gboolean terminated;       /* message processing was terminated */
@@ -80,10 +85,12 @@ struct _CamelFilterDriverPrivate {
 	
 	CamelMimeMessage *message; /* input message */
 	CamelMessageInfo *info;    /* message summary info */
+	const char *uid;           /* message uid */
+	CamelFolder *source;       /* message source folder */
 	
 	FILE *logfile;             /* log file */
 	
-	EDList rules;		/* list of _filter_rule structs */
+	EDList rules;		   /* list of _filter_rule structs */
 
 	CamelException *ex;
 	
@@ -369,10 +376,19 @@ do_copy (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFilterDriv
 				break;
 			
 			p->copied = TRUE;
-			camel_folder_append_message (outbox, p->message, p->info, p->ex);
+			if (p->uid && p->source && camel_folder_has_summary_capability (p->source)) {
+				GPtrArray *uids;
+				
+				uids = g_ptr_array_new ();
+				g_ptr_array_add (uids, (char *) p->uid);
+				camel_folder_copy_messages_to (p->source, uids, outbox, p->ex);
+				g_ptr_array_free (uids, TRUE);
+			} else
+				camel_folder_append_message (outbox, p->message, p->info, p->ex);
 			
 			service_url = camel_service_get_url (CAMEL_SERVICE (camel_folder_get_parent_store (outbox)));
-			camel_filter_driver_log (driver, FILTER_LOG_ACTION, "Copy to folder %s", service_url);
+			camel_filter_driver_log (driver, FILTER_LOG_ACTION, "Copy to folder %s",
+						 service_url);
 			g_free (service_url);
 		}
 	}
@@ -402,10 +418,19 @@ do_move (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFilterDriv
 			p->copied = TRUE;
 			p->deleted = TRUE;  /* a 'move' is a copy & delete */
 			
-			camel_folder_append_message (outbox, p->message, p->info, p->ex);
+			if (p->uid && p->source && camel_folder_has_summary_capability (p->source)) {
+				GPtrArray *uids;
+				
+				uids = g_ptr_array_new ();
+				g_ptr_array_add (uids, (char *) p->uid);
+				camel_folder_copy_messages_to (p->source, uids, outbox, p->ex);
+				g_ptr_array_free (uids, TRUE);
+			} else
+				camel_folder_append_message (outbox, p->message, p->info, p->ex);
 			
 			service_url = camel_service_get_url (CAMEL_SERVICE (camel_folder_get_parent_store (outbox)));
-			camel_filter_driver_log (driver, FILTER_LOG_ACTION, "Move to folder %s", service_url);
+			camel_filter_driver_log (driver, FILTER_LOG_ACTION, "Move to folder %s",
+						 service_url);
 			g_free (service_url);
 		}
 	}
@@ -506,7 +531,7 @@ close_folder (void *key, void *value, void *data)
 	camel_folder_thaw (folder);
 	camel_object_unref (CAMEL_OBJECT (folder));
 
-	report_status(driver, CAMEL_FILTER_STATUS_PROGRESS, g_hash_table_size(p->folders)* 100 / p->closed, "Syncing folders");
+	report_status(driver, CAMEL_FILTER_STATUS_PROGRESS, g_hash_table_size(p->folders)* 100 / p->closed, _("Syncing folders"));
 }
 
 /* flush/close all folders */
@@ -515,7 +540,7 @@ close_folders (CamelFilterDriver *driver)
 {
 	struct _CamelFilterDriverPrivate *p = _PRIVATE (driver);
 
-	report_status(driver, CAMEL_FILTER_STATUS_PROGRESS, 0, "Syncing folders");
+	report_status(driver, CAMEL_FILTER_STATUS_PROGRESS, 0, _("Syncing folders"));
 
 	p->closed = 0;
 	g_hash_table_foreach (p->folders, close_folder, driver);
@@ -588,8 +613,21 @@ camel_filter_driver_log (CamelFilterDriver *driver, enum filter_log_t status, co
 }
 
 
-/* will filter only an mbox - is more efficient as it doesn't need to open the folder through camel directly */
-void
+/**
+ * camel_filter_driver_filter_mbox:
+ * @driver: CamelFilterDriver
+ * @mbox: mbox filename to be filtered
+ * @ex: exception
+ *
+ * Filters an mbox file based on rules defined in the FilterDriver
+ * object. Is more efficient as it doesn't need to open the folder
+ * through Camel directly.
+ *
+ * Returns -1 if errors were encountered during filtering,
+ * otherwise returns 0.
+ *
+ **/
+int
 camel_filter_driver_filter_mbox (CamelFilterDriver *driver, const char *mbox, CamelException *ex)
 {
 	struct _CamelFilterDriverPrivate *p = _PRIVATE (driver);
@@ -598,10 +636,11 @@ camel_filter_driver_filter_mbox (CamelFilterDriver *driver, const char *mbox, Ca
 	int fd = -1;
 	int i = 0;
 	struct stat st;
+	int status;
 	
 	fd = open (mbox, O_RDONLY);
 	if (fd == -1) {
-		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, "Unable to open spool folder");
+		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Unable to open spool folder"));
 		goto fail;
 	}
 	/* to get the filesize */
@@ -610,7 +649,7 @@ camel_filter_driver_filter_mbox (CamelFilterDriver *driver, const char *mbox, Ca
 	mp = camel_mime_parser_new ();
 	camel_mime_parser_scan_from (mp, TRUE);
 	if (camel_mime_parser_init_with_fd (mp, fd) == -1) {
-		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, "Unable to process spool folder");
+		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Unable to process spool folder"));
 		goto fail;
 	}
 	fd = -1;
@@ -624,20 +663,20 @@ camel_filter_driver_filter_mbox (CamelFilterDriver *driver, const char *mbox, Ca
 		if (st.st_size > 0)
 			pc = (int)(100.0 * ((double)camel_mime_parser_tell (mp) / (double)st.st_size));
 		
-		report_status (driver, CAMEL_FILTER_STATUS_START, pc, "Getting message %d (%d%%)", i, pc);
+		report_status (driver, CAMEL_FILTER_STATUS_START, pc, _("Getting message %d (%d%%)"), i, pc);
 		
 		msg = camel_mime_message_new ();
 		if (camel_mime_part_construct_from_parser (CAMEL_MIME_PART (msg), mp) == -1) {
-			report_status (driver, CAMEL_FILTER_STATUS_END, 100, "Failed message %d", i);
-			camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, "Cannot open message");
+			report_status (driver, CAMEL_FILTER_STATUS_END, 100, _("Failed message %d"), i);
+			camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot open message"));
 			camel_object_unref (CAMEL_OBJECT (msg));
 			goto fail;
 		}
 		
-		camel_filter_driver_filter_message (driver, msg, NULL, source_url, ex);
+		status = camel_filter_driver_filter_message (driver, msg, NULL, NULL, NULL, source_url, ex);
 		camel_object_unref (CAMEL_OBJECT (msg));
-		if (camel_exception_is_set (ex)) {
-			report_status (driver, CAMEL_FILTER_STATUS_END, 100, "Failed message %d", i);
+		if (camel_exception_is_set (ex) || status == -1) {
+			report_status (driver, CAMEL_FILTER_STATUS_END, 100, _("Failed message %d"), i);
 			goto fail;
 		}
 		
@@ -648,11 +687,13 @@ camel_filter_driver_filter_mbox (CamelFilterDriver *driver, const char *mbox, Ca
 	}
 
 	if (p->defaultfolder) {
-		report_status(driver, CAMEL_FILTER_STATUS_PROGRESS, 100, "Syncing folder");
+		report_status(driver, CAMEL_FILTER_STATUS_PROGRESS, 100, _("Syncing folder"));
 		camel_folder_sync(p->defaultfolder, FALSE, ex);
 	}
 
-	report_status (driver, CAMEL_FILTER_STATUS_END, 100, "Complete");
+	report_status (driver, CAMEL_FILTER_STATUS_END, 100, _("Complete"));
+	
+	return 0;
 	
 fail:
 	g_free (source_url);
@@ -660,10 +701,27 @@ fail:
 		close (fd);
 	if (mp)
 		camel_object_unref (CAMEL_OBJECT (mp));
+	
+	return -1;
 }
 
-/* will filter a folder */
-void
+
+/**
+ * camel_filter_driver_filter_folder:
+ * @driver: CamelFilterDriver
+ * @folder: CamelFolder to be filtered
+ * @uids: message uids to be filtered or NULL (as a shortcut to filter all messages)
+ * @remove: TRUE to mark filtered messages as deleted
+ * @ex: exception
+ *
+ * Filters a folder based on rules defined in the FilterDriver
+ * object.
+ *
+ * Returns -1 if errors were encountered during filtering,
+ * otherwise returns 0.
+ *
+ **/
+int
 camel_filter_driver_filter_folder (CamelFilterDriver *driver, CamelFolder *folder,
 				   GPtrArray *uids, gboolean remove, CamelException *ex)
 {
@@ -673,6 +731,7 @@ camel_filter_driver_filter_folder (CamelFilterDriver *driver, CamelFolder *folde
 	CamelMimeMessage *message;
 	CamelMessageInfo *info;
 	char *source_url, *service_url;
+	int status = 0;
 	
 	service_url = camel_service_get_url (CAMEL_SERVICE (camel_folder_get_parent_store (folder)));
 	source_url = g_strdup_printf ("%s%s", service_url, camel_folder_get_full_name (folder));
@@ -686,11 +745,14 @@ camel_filter_driver_filter_folder (CamelFilterDriver *driver, CamelFolder *folde
 	for (i = 0; i < uids->len; i++) {
 		int pc = (100 * i)/uids->len;
 
-		report_status (driver, CAMEL_FILTER_STATUS_START, pc, "Getting message %d of %d", i+1, uids->len);
+		report_status (driver, CAMEL_FILTER_STATUS_START, pc, _("Getting message %d of %d"), i+1,
+			       uids->len);
 		
 		message = camel_folder_get_message (folder, uids->pdata[i], ex);
-		if (camel_exception_is_set (ex)) {
-			report_status (driver, CAMEL_FILTER_STATUS_END, 100, "Failed at message %d of %d", i+1, uids->len);
+		if (!message || camel_exception_is_set (ex)) {
+			report_status (driver, CAMEL_FILTER_STATUS_END, 100, _("Failed at message %d of %d"),
+				       i+1, uids->len);
+			status = -1;
 			break;
 		}
 		
@@ -699,49 +761,75 @@ camel_filter_driver_filter_folder (CamelFilterDriver *driver, CamelFolder *folde
 		else
 			info = NULL;
 		
-		camel_filter_driver_filter_message (driver, message, info, source_url, ex);
-
+		status = camel_filter_driver_filter_message (driver, message, info, uids->pdata[i],
+							     folder, source_url, ex);
+		
 		if (camel_folder_has_summary_capability (folder))
 			camel_folder_free_message_info (folder, info);
 
-		if (camel_exception_is_set (ex)) {
-			report_status (driver, CAMEL_FILTER_STATUS_END, 100, "Failed at message %d of %d", i+1, uids->len);
+		if (camel_exception_is_set (ex) || status == -1) {
+			report_status (driver, CAMEL_FILTER_STATUS_END, 100, _("Failed at message %d of %d"),
+				       i+1, uids->len);
+			status = -1;
 			break;
 		}
 		
 		if (remove)
 			camel_folder_set_message_flags (folder, uids->pdata[i],
-							CAMEL_MESSAGE_DELETED, CAMEL_MESSAGE_DELETED);
+							CAMEL_MESSAGE_DELETED|CAMEL_MESSAGE_SEEN, CAMEL_MESSAGE_DELETED|CAMEL_MESSAGE_SEEN);
 		
 		camel_object_unref (CAMEL_OBJECT (message));
 	}
 
 	if (freeuids)
 		camel_folder_free_uids (folder, uids);
-
 	
 	if (p->defaultfolder) {
-		report_status(driver, CAMEL_FILTER_STATUS_PROGRESS, 100, "Syncing folder");
+		report_status(driver, CAMEL_FILTER_STATUS_PROGRESS, 100, _("Syncing folder"));
 		camel_folder_sync (p->defaultfolder, FALSE, ex);
 	}
 
 	if (i == uids->len)
-		report_status (driver, CAMEL_FILTER_STATUS_END, 100, "Complete");
-	
+		report_status (driver, CAMEL_FILTER_STATUS_END, 100, _("Complete"));
 	
 	g_free (source_url);
+	
+	return status;
 }
 
-void
-camel_filter_driver_filter_message (CamelFilterDriver *driver, CamelMimeMessage *message, CamelMessageInfo *info,
-				    const char *source_url, CamelException *ex)
+
+/**
+ * camel_filter_driver_filter_message:
+ * @driver: CamelFilterDriver
+ * @message: message to filter
+ * @info: message info or NULL
+ * @uid: message uid or NULL
+ * @source: source folder or NULL
+ * @source_url: url of source folder or NULL
+ * @ex: exception
+ *
+ * Filters a message based on rules defined in the FilterDriver
+ * object. If the source folder (@source) and the uid (@uid) are
+ * provided, the filter will operate on the CamelFolder (which in
+ * certain cases is more efficient than using the default
+ * camel_folder_append_message() function).
+ *
+ * Returns -1 if errors were encountered during filtering,
+ * otherwise returns 0.
+ *
+ **/
+int
+camel_filter_driver_filter_message (CamelFilterDriver *driver, CamelMimeMessage *message,
+				    CamelMessageInfo *info, const char *uid,
+				    CamelFolder *source, const char *source_url,
+				    CamelException *ex)
 {
 	struct _CamelFilterDriverPrivate *p = _PRIVATE (driver);
 	ESExpResult *r;
 	struct _filter_rule *node;
 	gboolean freeinfo = FALSE;
 	gboolean filtered = FALSE;
-	
+
 	if (info == NULL) {
 		struct _header_raw *h = CAMEL_MIME_PART (message)->headers;
 		
@@ -749,7 +837,7 @@ camel_filter_driver_filter_message (CamelFilterDriver *driver, CamelMimeMessage
 		freeinfo = TRUE;
 	} else {
 		if (info->flags & CAMEL_MESSAGE_DELETED)
-			return;
+			return 0;
 	}
 	
 	p->ex = ex;
@@ -758,7 +846,11 @@ camel_filter_driver_filter_message (CamelFilterDriver *driver, CamelMimeMessage
 	p->copied = FALSE;
 	p->message = message;
 	p->info = info;
-
+	p->uid = uid;
+	p->source = source;
+	
+	/* camel_mime_message_set_identity (message, source_url); */
+	
 	node = (struct _filter_rule *)p->rules.head;
 	while (node->next) {
 		d(fprintf (stderr, "applying rule %s\n action %s\n", node->match, node->action));
@@ -766,7 +858,7 @@ camel_filter_driver_filter_message (CamelFilterDriver *driver, CamelMimeMessage
 		if (camel_filter_search_match(p->message, p->info, source_url, node->match, p->ex)) {
 			filtered = TRUE;
 			camel_filter_driver_log (driver, FILTER_LOG_START, node->name);
-
+			
 			/* perform necessary filtering actions */
 			e_sexp_input_text (p->eval, node->action, strlen (node->action));
 			if (e_sexp_parse (p->eval) == -1) {
@@ -787,20 +879,34 @@ camel_filter_driver_filter_message (CamelFilterDriver *driver, CamelMimeMessage
 	
 	/* *Now* we can set the DELETED flag... */
 	if (p->deleted)
-		info->flags = info->flags | CAMEL_MESSAGE_DELETED | CAMEL_MESSAGE_FOLDER_FLAGGED;
+		info->flags = info->flags | CAMEL_MESSAGE_DELETED | CAMEL_MESSAGE_FOLDER_FLAGGED | CAMEL_MESSAGE_SEEN;
 	
 	/* Logic: if !Moved and there exists a default folder... */
 	if (!(p->copied && p->deleted) && p->defaultfolder) {
 		/* copy it to the default inbox */
 		filtered = TRUE;
 		camel_filter_driver_log (driver, FILTER_LOG_ACTION, "Copy to default folder");
-		camel_folder_append_message (p->defaultfolder, p->message, p->info, p->ex);
+		if (p->uid && p->source && camel_folder_has_summary_capability (p->source)) {
+			GPtrArray *uids;
+				
+			uids = g_ptr_array_new ();
+			g_ptr_array_add (uids, (char *) p->uid);
+			camel_folder_copy_messages_to (p->source, uids, p->defaultfolder, p->ex);
+			g_ptr_array_free (uids, TRUE);
+		} else
+			camel_folder_append_message (p->defaultfolder, p->message, p->info, p->ex);
 	}
-
+	
+	if (freeinfo)
+		camel_message_info_free (info);
+	return 0;
+	
 error:	
 	if (filtered)
 		camel_filter_driver_log (driver, FILTER_LOG_END, NULL);
 	
 	if (freeinfo)
 		camel_message_info_free (info);
+	
+	return -1;
 }
