@@ -23,13 +23,18 @@
    It implements enough to do basic searches on folders that can provide
    an in-memory summary and a body index. */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
-#include <glib.h>
 #include <sys/types.h>
 #include <regex.h>
 
+#include <glib.h>
+
 #include "camel-folder-search.h"
 #include "string-utils.h"
 
@@ -41,8 +46,8 @@
 #include "e-util/e-memory.h"
 #include "camel-search-private.h"
 
-#define d(x)
-#define r(x)
+#define d(x) 
+#define r(x) 
 
 struct _CamelFolderSearchPrivate {
 	GHashTable *mempool_hash;
@@ -64,6 +69,7 @@ static ESExpResult *search_system_flag(struct _ESExp *f, int argc, struct _ESExp
 static ESExpResult *search_get_sent_date(struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolderSearch *s);
 static ESExpResult *search_get_received_date(struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolderSearch *s);
 static ESExpResult *search_get_current_date(struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolderSearch *s);
+static ESExpResult *search_uid(struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolderSearch *s);
 
 static ESExpResult *search_dummy(struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolderSearch *search);
 
@@ -91,6 +97,7 @@ camel_folder_search_class_init (CamelFolderSearchClass *klass)
 	klass->get_sent_date = search_get_sent_date;
 	klass->get_received_date = search_get_received_date;
 	klass->get_current_date = search_get_current_date;
+	klass->uid = search_uid;
 }
 
 static void
@@ -188,6 +195,7 @@ struct {
 	{ "get-sent-date", CAMEL_STRUCT_OFFSET(CamelFolderSearchClass, get_sent_date), 1 },
 	{ "get-received-date", CAMEL_STRUCT_OFFSET(CamelFolderSearchClass, get_received_date), 1 },
 	{ "get-current-date", CAMEL_STRUCT_OFFSET(CamelFolderSearchClass, get_current_date), 1 },
+	{ "uid", CAMEL_STRUCT_OFFSET(CamelFolderSearchClass, uid), 1 },
 };
 
 void
@@ -465,6 +473,8 @@ search_match_all(struct _ESExp *f, int argc, struct _ESExpTerm **argv, CamelFold
 	if (search->match1) {
 		search->current = search->match1;
 
+		d(printf("matching against 1 message: %s\n", camel_message_info_subject(search->current)));
+
 		if (argc>0) {
 			r1 = e_sexp_term_eval(f, argv[0]);
 			if (r1->type == ESEXP_RES_BOOL) {
@@ -518,7 +528,7 @@ check_header(struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolder
 	ESExpResult *r;
 	int truth = FALSE;
 
-	r(printf("executing check-header\n"));
+	r(printf("executing check-header %d\n", how));
 
 	/* are we inside a match-all? */
 	if (search->current && argc>1
@@ -542,6 +552,8 @@ check_header(struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolder
 			header = camel_message_info_to(search->current);
 		} else if (!strcasecmp(headername, "cc")) {
 			header = camel_message_info_cc(search->current);
+		} else if (!strcasecmp(headername, "x-camel-mlist")) {
+			header = camel_message_info_mlist(search->current);
 		} else {
 			e_sexp_resultv_free(f, argc, argv);
 			e_sexp_fatal_error(f, _("Performing query on unknown header: %s"), headername);
@@ -646,7 +658,9 @@ search_body_contains(struct _ESExp *f, int argc, struct _ESExpResult **argv, Cam
 	if (search->current) {
 		int truth = FALSE;
 
-		if (search->body_index) {
+		if (argc == 1 && argv[0]->value.string[0] == 0 && search->folder) {
+			truth = TRUE;
+		} else if (search->body_index) {
 			for (i=0;i<argc && !truth;i++) {
 				if (argv[i]->type == ESEXP_RES_STRING) {
 					truth = ibex_find_name(search->body_index, (char *)camel_message_info_uid(search->current),
@@ -670,7 +684,14 @@ search_body_contains(struct _ESExp *f, int argc, struct _ESExpResult **argv, Cam
 	} else {
 		r = e_sexp_result_new(f, ESEXP_RES_ARRAY_PTR);
 
-		if (search->body_index) {
+		if (argc == 1 && argv[0]->value.string[0] == 0 && search->folder) {
+			/* optimise the match "" case - match everything */
+			r->value.ptrarray = g_ptr_array_new();
+			for (i=0;i<search->summary->len;i++) {
+				CamelMessageInfo *info = g_ptr_array_index(search->summary, i);
+				g_ptr_array_add(r->value.ptrarray, (char *)camel_message_info_uid(info));
+			}
+		} else if (search->body_index) {
 			if (argc==1) {
 				/* common case */
 				r->value.ptrarray = ibex_find(search->body_index, argv[0]->value.string);
@@ -848,3 +869,37 @@ search_get_current_date(struct _ESExp *f, int argc, struct _ESExpResult **argv,
 	return r;
 }
 
+static ESExpResult *
+search_uid(struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolderSearch *search)
+{
+	ESExpResult *r;
+	int i;
+
+	r(printf("executing uid\n"));
+
+	/* are we inside a match-all? */
+	if (search->current) {
+		int truth = FALSE;
+		const char *uid = camel_message_info_uid(search->current);
+
+		/* performs an OR of all words */
+		for (i=0;i<argc && !truth;i++) {
+			if (argv[i]->type == ESEXP_RES_STRING
+			    && !strcmp(uid, argv[i]->value.string)) {
+				truth = TRUE;
+				break;
+			}
+		}
+		r = e_sexp_result_new(f, ESEXP_RES_BOOL);
+		r->value.bool = truth;
+	} else {
+		r = e_sexp_result_new(f, ESEXP_RES_ARRAY_PTR);
+		r->value.ptrarray = g_ptr_array_new();
+		for (i=0;i<argc;i++) {
+			if (argv[i]->type == ESEXP_RES_STRING)
+				g_ptr_array_add(r->value.ptrarray, argv[i]->value.string);
+		}
+	}
+
+	return r;
+}
