@@ -19,6 +19,10 @@
  *  USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <unistd.h>
 #include <netinet/in.h>
 #include <ctype.h>
@@ -36,6 +40,7 @@
 #include <camel/camel-mime-filter-charset.h>
 #include <camel/camel-mime-filter-save.h>
 #include <camel/camel-mime-filter-basic.h>
+#include <camel/camel-mime-filter-html.h>
 #include <camel/camel-mime-message.h>
 #include <camel/camel-stream-mem.h>
 
@@ -68,7 +73,7 @@ static pthread_mutex_t info_lock = PTHREAD_MUTEX_INITIALIZER;
 extern int strdup_count, malloc_count, free_count;
 #endif
 
-#define CAMEL_FOLDER_SUMMARY_VERSION (11)
+#define CAMEL_FOLDER_SUMMARY_VERSION (12)
 
 #define _PRIVATE(o) (((CamelFolderSummary *)(o))->priv)
 
@@ -198,13 +203,15 @@ camel_folder_summary_finalize (CamelObject *obj)
 		e_memchunk_destroy(s->content_info_chunks);
 
 	if (p->filter_index)
-		camel_object_unref ((CamelObject *)p->filter_index);
+		camel_object_unref((CamelObject *)p->filter_index);
 	if (p->filter_64)
-		camel_object_unref ((CamelObject *)p->filter_64);
+		camel_object_unref((CamelObject *)p->filter_64);
 	if (p->filter_qp)
-		camel_object_unref ((CamelObject *)p->filter_qp);
+		camel_object_unref((CamelObject *)p->filter_qp);
 	if (p->filter_save)
-		camel_object_unref ((CamelObject *)p->filter_save);
+		camel_object_unref((CamelObject *)p->filter_save);
+	if (p->filter_html)
+		camel_object_unref((CamelObject *)p->filter_html);
 
 #ifdef ENABLE_THREADS
 	g_mutex_free(p->summary_lock);
@@ -1044,19 +1051,18 @@ void camel_folder_summary_remove_uid(CamelFolderSummary *s, const char *uid)
  **/
 void camel_folder_summary_remove_index(CamelFolderSummary *s, int index)
 {
-	CAMEL_SUMMARY_LOCK(s, ref_lock);
 	CAMEL_SUMMARY_LOCK(s, summary_lock);
 	if (index < s->messages->len) {
 		CamelMessageInfo *info = s->messages->pdata[index];
-		/* make sure it doesn't vanish while we're not looking */
-		info->refcount++;
+
+		g_hash_table_remove(s->messages_uid, camel_message_info_uid(info));
+		g_ptr_array_remove_index(s->messages, index);
+		s->flags |= CAMEL_SUMMARY_DIRTY;
+
 		CAMEL_SUMMARY_UNLOCK(s, summary_lock);
-		CAMEL_SUMMARY_UNLOCK(s, ref_lock);
-		camel_folder_summary_remove(s, info);
 		camel_folder_summary_info_free(s, info);
 	} else {
 		CAMEL_SUMMARY_UNLOCK(s, summary_lock);
-		CAMEL_SUMMARY_UNLOCK(s, ref_lock);
 	}
 }
 
@@ -1717,11 +1723,14 @@ message_info_new(CamelFolderSummary *s, struct _header_raw *h)
 	e_strv_set_ref_free(mi->strings, CAMEL_MESSAGE_INFO_TO, msgid);
 	msgid = camel_folder_summary_format_address(h, "cc");
 	e_strv_set_ref_free(mi->strings, CAMEL_MESSAGE_INFO_CC, msgid);
+	msgid = header_raw_check_mailing_list(&h);
+	e_strv_set_ref_free(mi->strings, CAMEL_MESSAGE_INFO_MLIST, msgid);
 #else
 	mi->subject = camel_folder_summary_format_string(h, "subject");
 	mi->from = camel_folder_summary_format_address(h, "from");
 	mi->to = camel_folder_summary_format_address(h, "to");
 	mi->cc = camel_folder_summary_format_address(h, "cc");
+	mi->mlist = header_raw_check_mailing_list(&h);
 #endif
 	mi->user_flags = NULL;
 	mi->user_tags = NULL;
@@ -1793,6 +1802,8 @@ message_info_load(CamelFolderSummary *s, FILE *in)
 	e_strv_set_ref_free(mi->strings, CAMEL_MESSAGE_INFO_TO, tmp);
 	camel_folder_summary_decode_string(in, &tmp);
 	e_strv_set_ref_free(mi->strings, CAMEL_MESSAGE_INFO_CC, tmp);
+	camel_folder_summary_decode_string(in, &tmp);
+	e_strv_set_ref_free(mi->strings, CAMEL_MESSAGE_INFO_MLIST, tmp);
 #else
 	camel_folder_summary_decode_string(in, &mi->uid);
 	camel_folder_summary_decode_uint32(in, &mi->flags);
@@ -1803,6 +1814,7 @@ message_info_load(CamelFolderSummary *s, FILE *in)
 	camel_folder_summary_decode_string(in, &mi->from);
 	camel_folder_summary_decode_string(in, &mi->to);
 	camel_folder_summary_decode_string(in, &mi->cc);
+	camel_folder_summary_decode_string(in, &mi->mlist);
 #endif
 	mi->content = NULL;
 
@@ -1871,6 +1883,7 @@ message_info_save(CamelFolderSummary *s, FILE *out, CamelMessageInfo *mi)
 	camel_folder_summary_encode_string(out, camel_message_info_from(mi));
 	camel_folder_summary_encode_string(out, camel_message_info_to(mi));
 	camel_folder_summary_encode_string(out, camel_message_info_cc(mi));
+	camel_folder_summary_encode_string(out, camel_message_info_mlist(mi));
 
 	camel_folder_summary_encode_fixed_int32(out, mi->message_id.id.part.hi);
 	camel_folder_summary_encode_fixed_int32(out, mi->message_id.id.part.lo);
@@ -1916,6 +1929,7 @@ message_info_free(CamelFolderSummary *s, CamelMessageInfo *mi)
 	g_free(mi->from);
 	g_free(mi->to);
 	g_free(mi->cc);
+	g_free(mi->mlist);
 #endif
 	g_free(mi->references);
 	camel_flag_list_free(&mi->user_flags);
@@ -2045,7 +2059,7 @@ summary_build_content_info(CamelFolderSummary *s, CamelMessageInfo *msginfo, Cam
 	CamelMessageContentInfo *info = NULL;
 	struct _header_content_type *ct;
 	int body;
-	int enc_id = -1, chr_id = -1, idx_id = -1;
+	int enc_id = -1, chr_id = -1, html_id = -1, idx_id = -1;
 	struct _CamelFolderSummaryPrivate *p = _PRIVATE(s);
 	CamelMimeFilterCharset *mfc;
 	CamelMessageContentInfo *part;
@@ -2079,18 +2093,22 @@ summary_build_content_info(CamelFolderSummary *s, CamelMessageInfo *msginfo, Cam
 					d(printf(" decoding base64\n"));
 					if (p->filter_64 == NULL)
 						p->filter_64 = camel_mime_filter_basic_new_type(CAMEL_MIME_FILTER_BASIC_BASE64_DEC);
+					else
+						camel_mime_filter_reset((CamelMimeFilter *)p->filter_64);
 					enc_id = camel_mime_parser_filter_add(mp, (CamelMimeFilter *)p->filter_64);
 				} else if (!strcasecmp(encoding, "quoted-printable")) {
 					d(printf(" decoding quoted-printable\n"));
 					if (p->filter_qp == NULL)
 						p->filter_qp = camel_mime_filter_basic_new_type(CAMEL_MIME_FILTER_BASIC_QP_DEC);
+					else
+						camel_mime_filter_reset((CamelMimeFilter *)p->filter_qp);
 					enc_id = camel_mime_parser_filter_add(mp, (CamelMimeFilter *)p->filter_qp);
 				} else {
 					d(printf(" ignoring encoding %s\n", encoding));
 				}
 				g_free(encoding);
 			}
-				
+
 			charset = header_content_type_param(ct, "charset");
 			if (charset!=NULL
 			    && !(strcasecmp(charset, "us-ascii")==0
@@ -2101,6 +2119,8 @@ summary_build_content_info(CamelFolderSummary *s, CamelMessageInfo *msginfo, Cam
 					mfc = camel_mime_filter_charset_new_convert(charset, "UTF-8");
 					if (mfc)
 						g_hash_table_insert(p->filter_charset, g_strdup(charset), mfc);
+				} else {
+					camel_mime_filter_reset((CamelMimeFilter *)mfc);
 				}
 				if (mfc) {
 					chr_id = camel_mime_parser_filter_add(mp, (CamelMimeFilter *)mfc);
@@ -2109,6 +2129,18 @@ summary_build_content_info(CamelFolderSummary *s, CamelMessageInfo *msginfo, Cam
 				}
 			}
 
+#if 0
+			/* this is disabled for now */
+			/* also, should this go before/instead of the charset decoding?  xml interfaces are utf8 already? */
+			if (header_content_type_is(ct, "text", "html")) {
+				if (p->filter_html == NULL)
+					p->filter_html = camel_mime_filter_html_new();
+				else
+					camel_mime_filter_reset((CamelMimeFilter *)p->filter_html);
+				html_id = camel_mime_parser_filter_add(mp, (CamelMimeFilter *)p->filter_html);
+			}
+#endif
+			
 			/* and this filter actually does the indexing */
 			idx_id = camel_mime_parser_filter_add(mp, (CamelMimeFilter *)p->filter_index);
 		}
@@ -2118,6 +2150,7 @@ summary_build_content_info(CamelFolderSummary *s, CamelMessageInfo *msginfo, Cam
 		/* and remove the filters */
 		camel_mime_parser_filter_remove(mp, enc_id);
 		camel_mime_parser_filter_remove(mp, chr_id);
+		camel_mime_parser_filter_remove(mp, html_id);
 		camel_mime_parser_filter_remove(mp, idx_id);
 		break;
 	case HSCAN_MULTIPART:
@@ -2515,19 +2548,21 @@ CamelMessageInfo *
 camel_message_info_new_from_header (struct _header_raw *header)
 {
 	CamelMessageInfo *info;
-	char *subject, *from, *to, *cc;
-	
-	subject = camel_folder_summary_format_string (header, "subject");
-	from = camel_folder_summary_format_address (header, "from");
-	to = camel_folder_summary_format_address (header, "to");
-	cc = camel_folder_summary_format_address (header, "cc");
+	char *subject, *from, *to, *cc, *mlist;
 	
+	subject = camel_folder_summary_format_string(header, "subject");
+	from = camel_folder_summary_format_address(header, "from");
+	to = camel_folder_summary_format_address(header, "to");
+	cc = camel_folder_summary_format_address(header, "cc");
+	mlist = header_raw_check_mailing_list(&header);
+
 	info = camel_message_info_new();
 
-	camel_message_info_set_subject (info, subject);
-	camel_message_info_set_from (info, from);
-	camel_message_info_set_to (info, to);
-	camel_message_info_set_cc (info, cc);
+	camel_message_info_set_subject(info, subject);
+	camel_message_info_set_from(info, from);
+	camel_message_info_set_to(info, to);
+	camel_message_info_set_cc(info, cc);
+	camel_message_info_set_mlist(info, mlist);
 
 	return info;
 }
@@ -2563,12 +2598,14 @@ camel_message_info_dup_to(const CamelMessageInfo *from, CamelMessageInfo *to)
 	e_strv_set(to->strings, CAMEL_MESSAGE_INFO_TO, camel_message_info_to(from));
 	e_strv_set(to->strings, CAMEL_MESSAGE_INFO_CC, camel_message_info_cc(from));
 	e_strv_set(to->strings, CAMEL_MESSAGE_INFO_UID, camel_message_info_uid(from));
+	e_strv_set(to->strings, CAMEL_MESSAGE_INFO_UID, camel_message_info_mlist(from));
 #else
 	to->subject = g_strdup(from->subject);
 	to->from = g_strdup(from->from);
 	to->to = g_strdup(from->to);
 	to->cc = g_strdup(from->cc);
 	to->uid = g_strdup(from->uid);
+	to->mlist = g_strdup(from->mlist);
 #endif
 	memcpy(&to->message_id, &from->message_id, sizeof(from->message_id));
 
@@ -2632,6 +2669,7 @@ camel_message_info_free(CamelMessageInfo *mi)
 	g_free(mi->from);
 	g_free(mi->to);
 	g_free(mi->cc);
+	g_free(mi->mlist);
 #endif
 	g_free(mi->references);
 	camel_flag_list_free(&mi->user_flags);
@@ -2693,6 +2731,7 @@ message_info_dump(CamelMessageInfo *mi)
 	printf("Subject: %s\n", camel_message_info_subject(mi));
 	printf("To: %s\n", camel_message_info_to(mi));
 	printf("Cc: %s\n", camel_message_info_cc(mi));
+	printf("mailing list: %s\n", camel_message_info_mlist(mi));
 	printf("From: %s\n", camel_message_info_from(mi));
 	printf("UID: %s\n", camel_message_info_uid(mi));
 	printf("Flags: %04x\n", mi->flags & 0xffff);
