@@ -34,6 +34,8 @@
 
 #include "camel-private.h"
 
+#define d(x) 
+
 static CamelObjectClass *parent_class = NULL;
 
 /* Returns the class for a CamelFolder */
@@ -84,6 +86,7 @@ static CamelMimeMessage *get_message         (CamelFolder *folder,
 
 static CamelMessageInfo *get_message_info	(CamelFolder *folder, const char *uid);
 static void		 free_message_info	(CamelFolder *folder, CamelMessageInfo *info);
+static void		 ref_message_info	(CamelFolder *folder, CamelMessageInfo *info);
 
 static GPtrArray      *search_by_expression  (CamelFolder *folder,
 					      const char *exp,
@@ -91,15 +94,15 @@ static GPtrArray      *search_by_expression  (CamelFolder *folder,
 static void            search_free           (CamelFolder * folder, 
 					      GPtrArray * result);
 
-static void            copy_message_to       (CamelFolder *source,
-					      const char *uid,
-					      CamelFolder *dest,
-					      CamelException *ex);
+static void            copy_messages_to       (CamelFolder *source,
+					       GPtrArray *uids,
+					       CamelFolder *dest,
+					       CamelException *ex);
 
-static void            move_message_to       (CamelFolder *source,
-					      const char *uid,
-					      CamelFolder *dest,
-					      CamelException *ex);
+static void            move_messages_to       (CamelFolder *source,
+					       GPtrArray *uids,
+					       CamelFolder *dest,
+					       CamelException *ex);
 
 static void            freeze                (CamelFolder *folder);
 static void            thaw                  (CamelFolder *folder);
@@ -142,9 +145,10 @@ camel_folder_class_init (CamelFolderClass *camel_folder_class)
 	camel_folder_class->search_by_expression = search_by_expression;
 	camel_folder_class->search_free = search_free;
 	camel_folder_class->get_message_info = get_message_info;
+	camel_folder_class->ref_message_info = ref_message_info;
 	camel_folder_class->free_message_info = free_message_info;
-	camel_folder_class->copy_message_to = copy_message_to;
-	camel_folder_class->move_message_to = move_message_to;
+	camel_folder_class->copy_messages_to = copy_messages_to;
+	camel_folder_class->move_messages_to = move_messages_to;
 	camel_folder_class->freeze = freeze;
 	camel_folder_class->thaw = thaw;
 
@@ -798,6 +802,30 @@ camel_folder_free_message_info(CamelFolder *folder, CamelMessageInfo *info)
 	CF_CLASS (folder)->free_message_info(folder, info);
 }
 
+static void
+ref_message_info (CamelFolder *folder, CamelMessageInfo *info)
+{
+	g_return_if_fail(folder->summary != NULL);
+
+	camel_folder_summary_info_ref(folder->summary, info);
+}
+
+/**
+ * camel_folder_ref_message_info:
+ * @folder: 
+ * @info: 
+ * 
+ * Ref a CamelMessageInfo, previously obtained with get_message_info().
+ **/
+void
+camel_folder_ref_message_info(CamelFolder *folder, CamelMessageInfo *info)
+{
+	g_return_if_fail(CAMEL_IS_FOLDER (folder));
+	g_return_if_fail(info != NULL);
+
+	CF_CLASS (folder)->ref_message_info(folder, info);
+}
+
 /* TODO: is this function required anyway? */
 gboolean
 camel_folder_has_summary_capability (CamelFolder *folder)
@@ -924,9 +952,6 @@ camel_folder_free_uids (CamelFolder *folder, GPtrArray *array)
 static GPtrArray *
 get_summary(CamelFolder *folder)
 {
-	GPtrArray *res = g_ptr_array_new();
-	int i, count;
-
 	g_assert(folder->summary != NULL);
 
 	return camel_folder_summary_array(folder->summary);
@@ -957,8 +982,6 @@ camel_folder_get_summary (CamelFolder *folder)
 static void
 free_summary(CamelFolder *folder, GPtrArray *summary)
 {
-	int i;
-
 	g_assert(folder->summary != NULL);
 
 	camel_folder_summary_array_free(folder->summary, summary);
@@ -1062,49 +1085,69 @@ static void
 copy_message_to (CamelFolder *source, const char *uid, CamelFolder *dest, CamelException *ex)
 {
 	CamelMimeMessage *msg;
-	CamelMessageInfo *info;
+	CamelMessageInfo *info = NULL;
 
 	/* Default implementation. */
 	
 	/* we alredy have the lock, dont deadlock */
-	msg = CF_CLASS(source)->get_message(source, uid, ex);
+	msg = CF_CLASS (source)->get_message (source, uid, ex);
 	if (!msg)
 		return;
-	info = CF_CLASS(source)->get_message_info (source, uid);
+	
+	if (source->has_summary_capability)
+		info = CF_CLASS (source)->get_message_info (source, uid);
+	else
+		info = camel_message_info_new_from_header (((CamelMimePart *)msg)->headers);
+	
+	/* we don't want to retain the deleted flag */
+	if (info && info->flags & CAMEL_MESSAGE_DELETED)
+		info->flags = info->flags & ~CAMEL_MESSAGE_DELETED;
+	
 	camel_folder_append_message (dest, msg, info, ex);
 	camel_object_unref (CAMEL_OBJECT (msg));
-	if (info)
-		CF_CLASS(source)->free_message_info(source, info);
+	if (info) {
+		if (source->has_summary_capability)
+			CF_CLASS (source)->free_message_info (source, info);
+		else
+			camel_message_info_free (info);
+	}
+}
+
+static void
+copy_messages_to (CamelFolder *source, GPtrArray *uids, CamelFolder *dest, CamelException *ex)
+{
+	int i;
+	
+	for (i = 0; i < uids->len && !camel_exception_is_set (ex); i++)
+		copy_message_to (source, uids->pdata[i], dest, ex);
 }
 
 /**
- * camel_folder_copy_message_to:
+ * camel_folder_copy_messages_to:
  * @source: source folder
- * @uid: UID of message in @source
+ * @uids: message UIDs in @source
  * @dest: destination folder
  * @ex: a CamelException
  *
- * This copies a message from one folder to another. If the @source and
+ * This copies messages from one folder to another. If the @source and
  * @dest folders have the same parent_store, this may be more efficient
  * than a camel_folder_append_message().
- *
- * This function is still depcreated, it is not the same as move_message_to.
  **/
 void
-camel_folder_copy_message_to (CamelFolder *source, const char *uid,
-			      CamelFolder *dest, CamelException *ex)
+camel_folder_copy_messages_to (CamelFolder *source, GPtrArray *uids,
+			       CamelFolder *dest, CamelException *ex)
 {
 	g_return_if_fail (CAMEL_IS_FOLDER (source));
 	g_return_if_fail (CAMEL_IS_FOLDER (dest));
-	g_return_if_fail (uid != NULL);
-
+	g_return_if_fail (uids != NULL);
+	
 	CAMEL_FOLDER_LOCK(source, lock);
-
+	
 	if (source->parent_store == dest->parent_store)
-		CF_CLASS (source)->copy_message_to (source, uid, dest, ex);
+		CF_CLASS (source)->copy_messages_to (source, uids, dest, ex);
 	else
-		copy_message_to (source, uid, dest, ex);
-
+		copy_messages_to (source, uids, dest, ex);
+	
 	CAMEL_FOLDER_UNLOCK(source, lock);
 }
 
@@ -1114,27 +1157,50 @@ move_message_to (CamelFolder *source, const char *uid,
 		 CamelFolder *dest, CamelException *ex)
 {
 	CamelMimeMessage *msg;
-	CamelMessageInfo *info;
-
+	CamelMessageInfo *info = NULL;
+	
 	/* Default implementation. */
 	
-	msg = CF_CLASS(source)->get_message (source, uid, ex);
+	msg = CF_CLASS (source)->get_message (source, uid, ex);
 	if (!msg)
 		return;
-	info = CF_CLASS(source)->get_message_info (source, uid);
+	
+	if (source->has_summary_capability)
+		info = CF_CLASS (source)->get_message_info (source, uid);
+	else
+		info = camel_message_info_new_from_header (((CamelMimePart *)msg)->headers);
+	
+	/* we don't want to retain the deleted flag */
+	if (info && info->flags & CAMEL_MESSAGE_DELETED)
+		info->flags = info->flags & ~CAMEL_MESSAGE_DELETED;
+	
 	camel_folder_append_message (dest, msg, info, ex);
 	camel_object_unref (CAMEL_OBJECT (msg));
-	if (!camel_exception_is_set(ex))
-		CF_CLASS(source)->set_message_flags(source, uid, CAMEL_MESSAGE_DELETED, CAMEL_MESSAGE_DELETED);
+	if (!camel_exception_is_set (ex))
+		CF_CLASS (source)->set_message_flags (source, uid, CAMEL_MESSAGE_DELETED,
+						      CAMEL_MESSAGE_DELETED);
+	
+	if (info) {
+		if (source->has_summary_capability)
+			CF_CLASS (source)->free_message_info (source, info);
+		else
+			camel_message_info_free (info);
+	}
+}
 
-	if (info)
-		CF_CLASS(source)->free_message_info(source, info);
+static void
+move_messages_to (CamelFolder *source, GPtrArray *uids, CamelFolder *dest, CamelException *ex)
+{
+	int i;
+	
+	for (i = 0; i < uids->len && !camel_exception_is_set (ex); i++)
+		move_message_to (source, uids->pdata[i], dest, ex);
 }
 
 /**
- * camel_folder_move_message_to:
+ * camel_folder_move_messages_to:
  * @source: source folder
- * @uid: UID of message in @source
+ * @uids: message UIDs in @source
  * @dest: destination folder
  * @ex: a CamelException
  *
@@ -1144,20 +1210,25 @@ move_message_to (CamelFolder *source, const char *uid,
  * camel_folder_delete_message().
  **/
 void
-camel_folder_move_message_to (CamelFolder *source, const char *uid,
+camel_folder_move_messages_to (CamelFolder *source, GPtrArray *uids,
 			      CamelFolder *dest, CamelException *ex)
 {
 	g_return_if_fail (CAMEL_IS_FOLDER (source));
 	g_return_if_fail (CAMEL_IS_FOLDER (dest));
-	g_return_if_fail (uid != NULL);
-
+	g_return_if_fail (uids != NULL);
+	
+	if (source == dest) {
+		/* source and destination folders are the same, nothing to do. */
+		return;
+	}
+	
 	CAMEL_FOLDER_LOCK(source, lock);
-
+	
 	if (source->parent_store == dest->parent_store)
-		CF_CLASS (source)->move_message_to (source, uid, dest, ex);
+		CF_CLASS (source)->move_messages_to (source, uids, dest, ex);
 	else
-		move_message_to (source, uid, dest, ex);
-
+		move_messages_to (source, uids, dest, ex);
+	
 	CAMEL_FOLDER_UNLOCK(source, lock);
 }
 
@@ -1168,6 +1239,7 @@ freeze (CamelFolder *folder)
 
 	folder->priv->frozen++;
 
+	d(printf ("freeze(%p) = %d\n", folder, folder->priv->frozen));
 	CAMEL_FOLDER_UNLOCK(folder, change_lock);
 }
 
@@ -1197,6 +1269,9 @@ thaw (CamelFolder * folder)
 	CAMEL_FOLDER_LOCK(folder, change_lock);
 
 	folder->priv->frozen--;
+
+	d(printf ("thaw(%p) = %d\n", folder, folder->priv->frozen));
+
 	if (folder->priv->frozen == 0) {
 		/* If we have more or less messages, do a folder changed, otherwise just
 		   do a message changed for each one.
@@ -1240,6 +1315,8 @@ folder_changed (CamelObject *obj, gpointer event_data)
 	CamelFolderChangeInfo *changed = event_data;
 	gboolean ret = TRUE;
 
+	d(printf ("folder_changed(%p, %p), frozen=%d\n", obj, event_data, folder->priv->frozen));
+
 	if (folder->priv->frozen) {
 		CAMEL_FOLDER_LOCK(folder, change_lock);
 
@@ -1262,6 +1339,8 @@ message_changed (CamelObject *obj, /*const char *uid*/gpointer event_data)
 	CamelFolder *folder = CAMEL_FOLDER (obj);
 	gboolean ret = TRUE;
 
+	d(printf ("message_changed(%p, %p), frozen=%d\n", folder, event_data, folder->priv->frozen));
+
 	if (folder->priv->frozen) {
 		CAMEL_FOLDER_LOCK(folder, change_lock);
 	
@@ -1325,6 +1404,12 @@ camel_folder_free_deep (CamelFolder *folder, GPtrArray *array)
 	g_ptr_array_free (array, TRUE);
 }
 
+struct _CamelFolderChangeInfoPrivate {
+	GHashTable *uid_stored;	/* what we have stored, which array they're in */
+	GHashTable *uid_source;	/* used to create unique lists */
+	struct _EMemPool *uid_pool;	/* pool used to store copies of uid strings */
+};
+
 /**
  * camel_folder_change_info_new:
  * @void: 
@@ -1345,38 +1430,14 @@ camel_folder_change_info_new(void)
 	info->uid_added = g_ptr_array_new();
 	info->uid_removed = g_ptr_array_new();
 	info->uid_changed = g_ptr_array_new();
-	info->uid_source = NULL;
-	info->uid_pool = e_mempool_new(512, 256, E_MEMPOOL_ALIGN_BYTE);
+	info->priv = g_malloc0(sizeof(*info->priv));
+	info->priv->uid_stored = g_hash_table_new(g_str_hash, g_str_equal);
+	info->priv->uid_source = NULL;
+	info->priv->uid_pool = e_mempool_new(512, 256, E_MEMPOOL_ALIGN_BYTE);
 
 	return info;
 }
 
-static void
-change_info_add_uid(CamelFolderChangeInfo *info, GPtrArray *uids, const char *uid, int copy)
-{
-	int i;
-
-	/* TODO: Check that it is in the other arrays and remove it from them/etc? */
-	for (i=0;i<uids->len;i++) {
-		if (!strcmp(uids->pdata[i], uid))
-			return;
-	}
-	if (copy)
-		g_ptr_array_add(uids, e_mempool_strdup(info->uid_pool, uid));
-	else
-		g_ptr_array_add(uids, (char *)uid);
-}
-
-static void
-change_info_cat(CamelFolderChangeInfo *info, GPtrArray *uids, GPtrArray *source)
-{
-	int i;
-
-	for (i=0;i<source->len;i++) {
-		change_info_add_uid(info, uids, source->pdata[i], TRUE);
-	}
-}
-
 /**
  * camel_folder_change_info_add_source:
  * @info: 
@@ -1387,11 +1448,13 @@ change_info_cat(CamelFolderChangeInfo *info, GPtrArray *uids, GPtrArray *source)
 void
 camel_folder_change_info_add_source(CamelFolderChangeInfo *info, const char *uid)
 {
-	if (info->uid_source == NULL)
-		info->uid_source = g_hash_table_new(g_str_hash, g_str_equal);
+	struct _CamelFolderChangeInfoPrivate *p = info->priv;
 
-	if (g_hash_table_lookup(info->uid_source, uid) == NULL)
-		g_hash_table_insert(info->uid_source, e_mempool_strdup(info->uid_pool, uid), (void *)1);
+	if (p->uid_source == NULL)
+		p->uid_source = g_hash_table_new(g_str_hash, g_str_equal);
+
+	if (g_hash_table_lookup(p->uid_source, uid) == NULL)
+		g_hash_table_insert(p->uid_source, e_mempool_strdup(p->uid_pool, uid), (void *)1);
 }
 
 /**
@@ -1405,15 +1468,16 @@ void
 camel_folder_change_info_add_source_list(CamelFolderChangeInfo *info, const GPtrArray *list)
 {
 	int i;
+	struct _CamelFolderChangeInfoPrivate *p = info->priv;
 
-	if (info->uid_source == NULL)
-		info->uid_source = g_hash_table_new(g_str_hash, g_str_equal);
+	if (p->uid_source == NULL)
+		p->uid_source = g_hash_table_new(g_str_hash, g_str_equal);
 
 	for (i=0;i<list->len;i++) {
 		char *uid = list->pdata[i];
 
-		if (g_hash_table_lookup(info->uid_source, uid) == NULL)
-			g_hash_table_insert(info->uid_source, e_mempool_strdup(info->uid_pool, uid), (void *)1);
+		if (g_hash_table_lookup(p->uid_source, uid) == NULL)
+			g_hash_table_insert(p->uid_source, e_mempool_strdup(p->uid_pool, uid), (void *)1);
 	}
 }
 
@@ -1429,16 +1493,17 @@ camel_folder_change_info_add_update(CamelFolderChangeInfo *info, const char *uid
 {
 	char *key;
 	int value;
+	struct _CamelFolderChangeInfoPrivate *p = info->priv;
 
-	if (info->uid_source == NULL) {
-		change_info_add_uid(info, info->uid_added, uid, TRUE);
+	if (p->uid_source == NULL) {
+		camel_folder_change_info_add_uid(info, uid);
 		return;
 	}
 
-	if (g_hash_table_lookup_extended(info->uid_source, uid, (void **)&key, (void **)&value)) {
-		g_hash_table_remove(info->uid_source, key);
+	if (g_hash_table_lookup_extended(p->uid_source, uid, (void **)&key, (void **)&value)) {
+		g_hash_table_remove(p->uid_source, key);
 	} else {
-		change_info_add_uid(info, info->uid_added, uid, TRUE);
+		camel_folder_change_info_add_uid(info, uid);
 	}
 }
 
@@ -1454,16 +1519,31 @@ camel_folder_change_info_add_update_list(CamelFolderChangeInfo *info, const GPtr
 {
 	int i;
 
-	for (i=0;i<list->len;i++) {
+	for (i=0;i<list->len;i++)
 		camel_folder_change_info_add_update(info, list->pdata[i]);
-	}
 }
 
 static void
 change_info_remove(char *key, void *value, CamelFolderChangeInfo *info)
 {
+	struct _CamelFolderChangeInfoPrivate *p = info->priv;
+	GPtrArray *olduids;
+	char *olduid;
+
+	if (g_hash_table_lookup_extended(p->uid_stored, key, (void **)&olduid, (void **)&olduids)) {
+		/* if it was added/changed them removed, then remove it */
+		if (olduids != info->uid_removed) {
+			g_ptr_array_remove_fast(olduids, olduid);
+			g_ptr_array_add(info->uid_removed, olduid);
+			g_hash_table_insert(p->uid_stored, olduid, info->uid_removed);
+		}
+		return;
+	}
+
+
 	/* we dont need to copy this, as they've already been copied into our pool */
-	change_info_add_uid(info, info->uid_removed, key, FALSE);
+	g_ptr_array_add(info->uid_removed, key);
+	g_hash_table_insert(p->uid_stored, key, info->uid_removed);
 }
 
 /**
@@ -1476,13 +1556,24 @@ change_info_remove(char *key, void *value, CamelFolderChangeInfo *info)
 void
 camel_folder_change_info_build_diff(CamelFolderChangeInfo *info)
 {
-	if (info->uid_source) {
-		g_hash_table_foreach(info->uid_source, (GHFunc)change_info_remove, info);
-		g_hash_table_destroy(info->uid_source);
-		info->uid_source = NULL;
+	struct _CamelFolderChangeInfoPrivate *p = info->priv;
+
+	if (p->uid_source) {
+		g_hash_table_foreach(p->uid_source, (GHFunc)change_info_remove, info);
+		g_hash_table_destroy(p->uid_source);
+		p->uid_source = NULL;
 	}
 }
 
+static void
+change_info_cat(CamelFolderChangeInfo *info, GPtrArray *source, void (*add)(CamelFolderChangeInfo *info, const char *uid))
+{
+	int i;
+
+	for (i=0;i<source->len;i++)
+		add(info, source->pdata[i]);
+}
+
 /**
  * camel_folder_change_info_cat:
  * @info: 
@@ -1494,9 +1585,9 @@ camel_folder_change_info_build_diff(CamelFolderChangeInfo *info)
 void
 camel_folder_change_info_cat(CamelFolderChangeInfo *info, CamelFolderChangeInfo *source)
 {
-	change_info_cat(info, info->uid_added, source->uid_added);
-	change_info_cat(info, info->uid_removed, source->uid_removed);
-	change_info_cat(info, info->uid_changed, source->uid_changed);
+	change_info_cat(info, source->uid_added, camel_folder_change_info_add_uid);
+	change_info_cat(info, source->uid_removed, camel_folder_change_info_remove_uid);
+	change_info_cat(info, source->uid_changed, camel_folder_change_info_change_uid);
 }
 
 /**
@@ -1509,7 +1600,24 @@ camel_folder_change_info_cat(CamelFolderChangeInfo *info, CamelFolderChangeInfo
 void
 camel_folder_change_info_add_uid(CamelFolderChangeInfo *info, const char *uid)
 {
-	change_info_add_uid(info, info->uid_added, uid, TRUE);
+	struct _CamelFolderChangeInfoPrivate *p = info->priv;
+	GPtrArray *olduids;
+	char *olduid;
+
+	if (g_hash_table_lookup_extended(p->uid_stored, uid, (void **)&olduid, (void **)&olduids)) {
+		/* if it was removed then added, promote it to a changed */
+		/* if it was changed then added, leave as changed */
+		if (olduids == info->uid_removed) {
+			g_ptr_array_remove_fast(olduids, olduid);
+			g_ptr_array_add(info->uid_changed, olduid);
+			g_hash_table_insert(p->uid_stored, olduid, info->uid_changed);
+		}
+		return;
+	}
+
+	olduid = e_mempool_strdup(p->uid_pool, uid);
+	g_ptr_array_add(info->uid_added, olduid);
+	g_hash_table_insert(p->uid_stored, olduid, info->uid_added);
 }
 
 /**
@@ -1522,7 +1630,23 @@ camel_folder_change_info_add_uid(CamelFolderChangeInfo *info, const char *uid)
 void
 camel_folder_change_info_remove_uid(CamelFolderChangeInfo *info, const char *uid)
 {
-	change_info_add_uid(info, info->uid_removed, uid, TRUE);
+	struct _CamelFolderChangeInfoPrivate *p = info->priv;
+	GPtrArray *olduids;
+	char *olduid;
+
+	if (g_hash_table_lookup_extended(p->uid_stored, uid, (void **)&olduid, (void **)&olduids)) {
+		/* if it was added/changed them removed, then remove it */
+		if (olduids != info->uid_removed) {
+			g_ptr_array_remove_fast(olduids, olduid);
+			g_ptr_array_add(info->uid_removed, olduid);
+			g_hash_table_insert(p->uid_stored, olduid, info->uid_removed);
+		}
+		return;
+	}
+
+	olduid = e_mempool_strdup(p->uid_pool, uid);
+	g_ptr_array_add(info->uid_removed, olduid);
+	g_hash_table_insert(p->uid_stored, olduid, info->uid_removed);
 }
 
 /**
@@ -1535,7 +1659,18 @@ camel_folder_change_info_remove_uid(CamelFolderChangeInfo *info, const char *uid
 void
 camel_folder_change_info_change_uid(CamelFolderChangeInfo *info, const char *uid)
 {
-	change_info_add_uid(info, info->uid_changed, uid, TRUE);
+	struct _CamelFolderChangeInfoPrivate *p = info->priv;
+	GPtrArray *olduids;
+	char *olduid;
+
+	if (g_hash_table_lookup_extended(p->uid_stored, uid, (void **)&olduid, (void **)&olduids)) {
+		/* if we have it already, leave it as that */
+		return;
+	}
+
+	olduid = e_mempool_strdup(p->uid_pool, uid);
+	g_ptr_array_add(info->uid_changed, olduid);
+	g_hash_table_insert(p->uid_stored, olduid, info->uid_changed);
 }
 
 /**
@@ -1561,14 +1696,18 @@ camel_folder_change_info_changed(CamelFolderChangeInfo *info)
 void
 camel_folder_change_info_clear(CamelFolderChangeInfo *info)
 {
+	struct _CamelFolderChangeInfoPrivate *p = info->priv;
+
 	g_ptr_array_set_size(info->uid_added, 0);
 	g_ptr_array_set_size(info->uid_removed, 0);
 	g_ptr_array_set_size(info->uid_changed, 0);
-	if (info->uid_source) {
-		g_hash_table_destroy(info->uid_source);
-		info->uid_source = NULL;
+	if (p->uid_source) {
+		g_hash_table_destroy(p->uid_source);
+		p->uid_source = NULL;
 	}
-	e_mempool_flush(info->uid_pool, TRUE);
+	g_hash_table_destroy(p->uid_stored);
+	p->uid_stored = g_hash_table_new(g_str_hash, g_str_equal);
+	e_mempool_flush(p->uid_pool, TRUE);
 }
 
 /**
@@ -1580,10 +1719,14 @@ camel_folder_change_info_clear(CamelFolderChangeInfo *info)
 void
 camel_folder_change_info_free(CamelFolderChangeInfo *info)
 {
-	if (info->uid_source)
-		g_hash_table_destroy(info->uid_source);
+	struct _CamelFolderChangeInfoPrivate *p = info->priv;
 
-	e_mempool_destroy(info->uid_pool);
+	if (p->uid_source)
+		g_hash_table_destroy(p->uid_source);
+
+	g_hash_table_destroy(p->uid_stored);
+	e_mempool_destroy(p->uid_pool);
+	g_free(p);
 
 	g_ptr_array_free(info->uid_added, TRUE);
 	g_ptr_array_free(info->uid_removed, TRUE);
