@@ -29,7 +29,7 @@
 
 #define CAMEL_MIME_PARSER(obj)         CAMEL_CHECK_CAST (obj, camel_mime_parser_get_type (), CamelMimeParser)
 #define CAMEL_MIME_PARSER_CLASS(klass) CAMEL_CHECK_CLASS_CAST (klass, camel_mime_parser_get_type (), CamelMimeParserClass)
-#define IS_CAMEL_MIME_PARSER(obj)      CAMEL_CHECK_TYPE (obj, camel_mime_parser_get_type ())
+#define CAMEL_IS_MIME_PARSER(obj)      CAMEL_CHECK_TYPE (obj, camel_mime_parser_get_type ())
 
 typedef struct _CamelMimeParserClass CamelMimeParserClass;
 
@@ -38,6 +38,7 @@ typedef struct _CamelMimeParserClass CamelMimeParserClass;
    the same as the matching start tag, with a bit difference */
 enum _header_state {
 	HSCAN_INITIAL,
+	HSCAN_PRE_FROM,		/* data before a 'From' line */
 	HSCAN_FROM,		/* got 'From' line */
 	HSCAN_HEADER,		/* toplevel header */
 	HSCAN_BODY,		/* scanning body of message */
@@ -49,6 +50,7 @@ enum _header_state {
 	HSCAN_END = 8,		/* bit mask for 'end' flags */
 
 	HSCAN_EOF = 8,		/* end of file */
+	HSCAN_PRE_FROM_END,	/* pre from end */
 	HSCAN_FROM_END,		/* end of whole from bracket */
 	HSCAN_HEADER_END,	/* dummy value */
 	HSCAN_BODY_END,		/* end of message */
@@ -84,6 +86,8 @@ int		camel_mime_parser_fd(CamelMimeParser *m);
 
 /* scan 'From' separators? */
 void camel_mime_parser_scan_from(CamelMimeParser *, int);
+/* Do we want to know about the pre-from data? */
+void camel_mime_parser_scan_pre_from(CamelMimeParser *, int);
 
 /* what headers to save, MUST include ^Content-Type: */
 int camel_mime_parser_set_header_regex(CamelMimeParser *m, char *matchstr);
@@ -94,6 +98,9 @@ void camel_mime_parser_unstep(CamelMimeParser *);
 void camel_mime_parser_drop_step(CamelMimeParser *m);
 enum _header_state camel_mime_parser_state(CamelMimeParser *);
 
+/* read through the parser */
+int camel_mime_parser_read(CamelMimeParser *m, const char **databuffer, int len);
+
 /* get content type for the current part/header */
 struct _header_content_type *camel_mime_parser_content_type(CamelMimeParser *);
 
@@ -103,6 +110,13 @@ const char *camel_mime_parser_header(CamelMimeParser *, const char *, int *offse
 /* get all raw headers. READ ONLY! */
 struct _header_raw *camel_mime_parser_headers_raw(CamelMimeParser *);
 
+/* get multipart pre/postface */
+const char *camel_mime_parser_preface(CamelMimeParser *m);
+const char *camel_mime_parser_postface(CamelMimeParser *m);
+
+/* return the from line content */
+const char *camel_mime_parser_from_line(CamelMimeParser *m);
+
 /* add a processing filter for body contents */
 int camel_mime_parser_filter_add(CamelMimeParser *, CamelMimeFilter *);
 void camel_mime_parser_filter_remove(CamelMimeParser *, int);
