@@ -21,29 +21,41 @@
 
 /* dont touch this file without my permission - Michael */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
 
 #include <stdio.h>
-
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/param.h>  /* for MAXHOSTNAMELEN */
 #include <fcntl.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
+#ifndef MAXHOSTNAMELEN
+#define MAXHOSTNAMELEN 1024
+#endif
+
 #include <unicode.h>
 #include <iconv.h>
 
-#include <glib.h>
 #include <time.h>
 
 #include <ctype.h>
 #include <errno.h>
+#include <regex.h>
+
+#include <glib.h>
 
 #include "camel-mime-utils.h"
 #include "camel-charset-map.h"
 
+#ifdef ENABLE_THREADS
+#include <pthread.h>
+#endif
+
 #ifndef CLEAN_DATE
 #include "broken-date-parser.h"
 #endif
@@ -454,6 +466,30 @@ base64_decode_step(unsigned char *in, int len, unsigned char *out, int *state, u
 	return outptr-out;
 }
 
+char *
+base64_encode_simple (const char *data, int len)
+{
+	unsigned char *out;
+	int state = 0, outlen;
+	unsigned int save = 0;
+
+	out = g_malloc (len * 4 / 3 + 5);
+	outlen = base64_encode_close ((unsigned char *)data, len, FALSE,
+				      out, &state, &save);
+	out[outlen] = '\0';
+	return (char *)out;
+}
+
+int
+base64_decode_simple (char *data, int len)
+{
+	int state = 0;
+	unsigned int save = 0;
+
+	return base64_decode_step ((unsigned char *)data, len,
+				   (unsigned char *)data, &state, &save);
+}
+
 
 /**
  * uudecode_step: uudecode a chunk of data
@@ -867,29 +903,28 @@ rfc2047_decode_word(const char *in, int len)
 {
 	const char *inptr = in+2;
 	const char *inend = in+len-2;
+	char *inbuf;
 	char *encname;
 	int tmplen;
 	int ret;
 	char *decword = NULL;
 	char *decoded = NULL;
 	char *outbase = NULL;
-	char *inbuf, *outbuf;
+	char *outbuf;
 	int inlen, outlen;
 	iconv_t ic;
 
 	d(printf("rfc2047: decoding '%.*s'\n", len, in));
 
-	/* just make sure we're not passed shit */
-	if (len<7
-	    || !(in[0]=='=' && in[1]=='?' && in[len-1]=='=' && in[len-2]=='?')) {
+	/* quick check to see if this could possibly be a real encoded word */
+	if (len < 8 || !(in[0] == '=' && in[1] == '?' && in[len-1] == '=' && in[len-2] == '?')) {
 		d(printf("invalid\n"));
 		return NULL;
 	}
-
-	inptr = memchr(inptr, '?', inend-inptr);
-	if (inptr!=NULL
-	    && inptr<inend+2
-	    && inptr[2]=='?') {
+	
+	/* skip past the charset to the encoding type */
+	inptr = memchr (inptr, '?', inend-inptr);
+	if (inptr != NULL && inptr < inend + 2 && inptr[2] == '?') {
 		d(printf("found ?, encoding is '%c'\n", inptr[0]));
 		inptr++;
 		tmplen = inend-inptr-2;
@@ -905,6 +940,9 @@ rfc2047_decode_word(const char *in, int len)
 			/* if state != 0 then error? */
 			break;
 		}
+		default:
+			/* uhhh, unknown encoding type - probably an invalid encoded word string */
+			return NULL;
 		}
 		d(printf("The encoded length = %d\n", inlen));
 		if (inlen>0) {
@@ -923,7 +961,7 @@ rfc2047_decode_word(const char *in, int len)
 			/* TODO: Should this cache iconv converters? */
 			ic = iconv_open("UTF-8", encname);
 			if (ic != (iconv_t)-1) {
-				ret = iconv(ic, (const char **)&inbuf, &inlen, &outbuf, &outlen);
+				ret = iconv(ic, &inbuf, &inlen, &outbuf, &outlen);
 				if (ret>=0) {
 					iconv(ic, NULL, 0, &outbuf, &outlen);
 					*outbuf = 0;
@@ -931,7 +969,8 @@ rfc2047_decode_word(const char *in, int len)
 				}
 				iconv_close(ic);
 			} else {
-				w(g_warning("Cannot decode charset, header display may be corrupt: %s: %s", encname, strerror(errno)));
+				w(g_warning("Cannot decode charset, header display may be corrupt: %s: %s",
+					    encname, strerror(errno)));
 				/* TODO: Should this do this, or just leave the encoded strings? */
 				decword[inlen] = 0;
 				decoded = g_strdup(decword);
@@ -983,65 +1022,69 @@ static char *
 header_decode_text (const char *in, int inlen)
 {
 	GString *out;
-	char *inptr, *inend, *start;
+	char *inptr, *inend, *start, *word_start;
 	char *decoded;
-	unsigned char lastc = 0;
-	int wasdword = FALSE;
-
+	gboolean wasdword = FALSE;
+	gboolean wasspace = FALSE;
+	
 	out = g_string_new ("");
 	start = inptr = (char *) in;
 	inend = inptr + inlen;
 	
+	word_start = NULL;
 	while (inptr && inptr < inend) {
 		unsigned char c = *inptr++;
 		
-		if (is_lwsp(c)) {
+		if (is_lwsp (c) && !wasspace) {
 			char *word, *dword;
-			guint len;
 			
-			len = inptr - start - 1;
-			word = start;
+			if (word_start)
+				word = word_start;
+			else
+				word = start;
+			
+			dword = rfc2047_decode_word (word, inptr - word - 1);
 			
-			dword = rfc2047_decode_word (word, len);
-
 			if (dword) {
-				if (!wasdword && lastc)
-					g_string_append_c(out, lastc);
-					
+				if (!wasdword && word_start)
+					g_string_append_len (out, start, word_start - start);
+				
 				g_string_append (out, dword);
 				g_free (dword);
-				lastc = c;
 				wasdword = TRUE;
 			} else {
-				if (lastc)
-					g_string_append_c(out, lastc);
-				out = append_latin1 (out, word, len);
-				lastc = c;
+				out = append_latin1 (out, start, inptr - start - 1);
 				wasdword = FALSE;
 			}
-						
-			start = inptr;
+			
+			start = inptr - 1;
+			word_start = NULL;
+			wasspace = TRUE;
+		} else if (!is_lwsp (c)) {
+			wasspace = FALSE;
+			if (!word_start)
+				word_start = inptr - 1;
 		}
 	}
 	
 	if (inptr - start) {
 		char *word, *dword;
-		guint len;
 		
-		len = inptr - start;
-		word = start;
+		if (word_start)
+			word = word_start;
+		else
+			word = start;
 		
-		dword = rfc2047_decode_word (word, len);
+		dword = rfc2047_decode_word (word, inptr - word);
 		
 		if (dword) {
-			if (!wasdword && lastc)
-				g_string_append_c(out, lastc);
+			if (!wasdword && word_start)
+				g_string_append_len (out, start, word_start - start);
+			
 			g_string_append (out, dword);
 			g_free (dword);
 		} else {
-			if (lastc)
-				g_string_append_c(out, lastc);
-			out = g_string_append_len (out, word, len);
+			out = append_latin1 (out, start, inptr - start);
 		}
 	}
 	
@@ -1107,7 +1150,7 @@ rfc2047_encode_word(GString *outstring, const char *in, int len, const char *typ
 	iconv_t ic = (iconv_t *)-1;
 	char *buffer, *out, *ascii;
 	size_t inlen, outlen, enclen, bufflen;
-	const char *inptr, *p;
+	char *inptr, *p;
 	int first = 1;
 
 	d(printf("Converting [%d] '%.*s' to %s\n", len, len, in, type));
@@ -1116,7 +1159,7 @@ rfc2047_encode_word(GString *outstring, const char *in, int len, const char *typ
 	bufflen = len*6+16;
 	buffer = alloca(bufflen);
 	inlen = len;
-	inptr = in;
+	inptr = (char *) in;
 
 	ascii = alloca(bufflen);
 
@@ -1204,82 +1247,125 @@ rfc2047_encode_word(GString *outstring, const char *in, int len, const char *typ
 
 /* TODO: Should this worry about quotes?? */
 char *
-header_encode_string(const unsigned char *in)
+header_encode_string (const unsigned char *in)
 {
-	GString *out;
-	const unsigned char *inptr = in, *start;
+	const unsigned char *inptr = in, *start, *word;
+	gboolean last_was_encoded = FALSE;
+	gboolean last_was_space = FALSE;
 	int encoding;
+	GString *out;
 	char *outstr;
-
+	
 	if (in == NULL)
 		return NULL;
-
+	
 	/* do a quick us-ascii check (the common case?) */
 	while (*inptr) {
 		if (*inptr > 127)
 			break;
 		inptr++;
 	}
-	if (*inptr == 0)
-		return g_strdup(in);
-
+	if (*inptr == '\0')
+		return g_strdup (in);
+	
 	/* This gets each word out of the input, and checks to see what charset
 	   can be used to encode it. */
 	/* TODO: Work out when to merge subsequent words, or across word-parts */
-	out = g_string_new("");
+	out = g_string_new ("");
 	inptr = in;
 	encoding = 0;
+	word = NULL;
 	start = inptr;
 	while (inptr && *inptr) {
 		unicode_char_t c;
 		const char *newinptr;
-		newinptr = unicode_get_utf8(inptr, &c);
+		
+		newinptr = unicode_get_utf8 (inptr, &c);
 		if (newinptr == NULL) {
-			w(g_warning("Invalid UTF-8 sequence encountered (pos %d, char '%c'): %s", (inptr-in), inptr[0], in));
+			w(g_warning ("Invalid UTF-8 sequence encountered (pos %d, char '%c'): %s",
+				     (inptr-in), inptr[0], in));
 			inptr++;
 			continue;
 		}
-		inptr = newinptr;
-		if (unicode_isspace(c)) {
+		
+		if (unicode_isspace (c) && !last_was_space) {
 			/* we've reached the end of a 'word' */
+			if (word && !(last_was_encoded && encoding)) {
+				g_string_append_len (out, start, word - start);
+				start = word;
+			}
+			
 			switch (encoding) {
 			case 0:
-				out = g_string_append_len(out, start, inptr-start);
+				out = g_string_append_len (out, word, inptr - start);
+				last_was_encoded = FALSE;
 				break;
 			case 1:
-				rfc2047_encode_word(out, start, inptr-start-1, "ISO-8859-1", IS_ESAFE);
-				out = g_string_append_c (out, c);
+				if (last_was_encoded)
+					g_string_append_c (out, ' ');
+				
+				rfc2047_encode_word (out, start, inptr - start, "ISO-8859-1", IS_ESAFE);
+				last_was_encoded = TRUE;
 				break;
 			case 2:
-				rfc2047_encode_word(out, start, inptr-start-1,
-						    camel_charset_best(start, inptr-start-1), IS_ESAFE);
-				out = g_string_append_c(out, c);
+				if (last_was_encoded)
+					g_string_append_c (out, ' ');
+				
+				rfc2047_encode_word (out, start, inptr - start,
+						     camel_charset_best (start, inptr - start), IS_ESAFE);
+				last_was_encoded = TRUE;
 				break;
 			}
+			
+			last_was_space = TRUE;
 			start = inptr;
+			word = NULL;
 			encoding = 0;
 		} else if (c > 127 && c < 256) {
-			encoding = MAX(encoding, 1);
+			encoding = MAX (encoding, 1);
+			last_was_space = FALSE;
 		} else if (c >= 256) {
-			encoding = MAX(encoding, 2);
+			encoding = MAX (encoding, 2);
+			last_was_space = FALSE;
+		} else if (!unicode_isspace (c)) {
+			last_was_space = FALSE;
 		}
+		
+		if (!unicode_isspace (c) && !word)
+			word = inptr;
+		
+		inptr = newinptr;
 	}
-	if (inptr-start) {
+	
+	if (inptr - start) {
+		if (word && !(last_was_encoded && encoding)) {
+			g_string_append_len (out, start, word - start);
+			start = word;
+		}
+		
 		switch (encoding) {
 		case 0:
-			out = g_string_append_len(out, start, inptr-start);
+			out = g_string_append_len (out, start, inptr - start);
 			break;
 		case 1:
-			rfc2047_encode_word(out, start, inptr-start, "ISO-8859-1", IS_ESAFE);
+			if (last_was_encoded)
+				g_string_append_c (out, ' ');
+			
+			rfc2047_encode_word (out, start, inptr - start, "ISO-8859-1", IS_ESAFE);
 			break;
 		case 2:
-			rfc2047_encode_word(out, start, inptr-start,
-					    camel_charset_best(start, inptr-start-1), IS_ESAFE);
+			if (last_was_encoded)
+				g_string_append_c (out, ' ');
+			
+			rfc2047_encode_word (out, start, inptr - start,
+					     camel_charset_best (start, inptr - start - 1), IS_ESAFE);
 			break;
 		}
 	}
+	
 	outstr = out->str;
-	g_string_free(out, FALSE);
+	g_string_free (out, FALSE);
+	
 	return outstr;
 }
 
@@ -1320,110 +1406,102 @@ struct _phrase_word {
    merge common word types
    clean up
 */
-/* encodes a phrase sequence (different quoting/encoding rules to strings) */
-char *
-header_encode_phrase(const unsigned char *in)
+
+static GList *
+header_encode_phrase_get_words (const unsigned char *in)
 {
-	GString *out;
 	const unsigned char *inptr = in, *start, *last;
-	int encoding;
-	char *outstr;
-	struct _phrase_word *word, *next;
+	struct _phrase_word *word;
 	enum _phrase_word_t type;
-	GList *words = NULL, *wordl, *nextl;
-	int count;
-
-	if (in == NULL)
-		return NULL;
-
-	out = g_string_new("");
-
-#if 0
-	{
-		int i;
-
-		printf("encoding phrase: %s\n", in);
-		for (i=0;in[i];i++) {
-			printf(" %02x", in[i]);
-			if (((i) & 15) == 15)
-				printf("\n");
-		}
-		printf("\n");
-	}
-#endif
-
+	int encoding, count = 0;
+	GList *words = NULL;
+	
 	/* break the input into words */
 	type = WORD_ATOM;
-	count = 0;
 	last = inptr;
 	start = inptr;
 	encoding = 0;
 	while (inptr && *inptr) {
 		unicode_char_t c;
 		const char *newinptr;
-		newinptr = unicode_get_utf8(inptr, &c);
+		
+		newinptr = unicode_get_utf8 (inptr, &c);
 		if (newinptr == NULL) {
-			w(g_warning("Invalid UTF-8 sequence encountered (pos %d, char '%c'): %s", (inptr-in), inptr[0], in));
+			w(g_warning ("Invalid UTF-8 sequence encountered (pos %d, char '%c'): %s",
+				     (inptr - in), inptr[0], in));
 			inptr++;
 			continue;
 		}
+		
 		inptr = newinptr;
-		/* save this word out, multiple whitespace is not explicitly counted (?) */
-		if (unicode_isspace(c)) {
+		if (unicode_isspace (c)) {
 			if (count > 0) {
-				word = g_malloc0(sizeof(*word));
+				word = g_new0 (struct _phrase_word, 1);
 				word->start = start;
 				word->end = last;
 				word->type = type;
 				word->encoding = encoding;
-				words = g_list_append(words, word);
+				words = g_list_append (words, word);
 				count = 0;
 			}
+			
 			start = inptr;
 			type = WORD_ATOM;
 			encoding = 0;
 		} else {
 			count++;
-			if (c<128) {
-				if (!is_atom(c))
-					type = MAX(type, WORD_QSTRING);
-			} else if (c>127 && c < 256) {
+			if (c < 128) {
+				if (!is_atom (c))
+					type = MAX (type, WORD_QSTRING);
+			} else if (c > 127 && c < 256) {
 				type = WORD_2047;
-				encoding = MAX(encoding, 1);
-			} else if (c >=256) {
+				encoding = MAX (encoding, 1);
+			} else if (c >= 256) {
 				type = WORD_2047;
-				encoding = MAX(encoding, 2);
+				encoding = MAX (encoding, 2);
 			}
 		}
+		
 		last = inptr;
 	}
+	
 	if (count > 0) {
-		word = g_malloc0(sizeof(*word));
+		word = g_new0 (struct _phrase_word, 1);
 		word->start = start;
 		word->end = last;
 		word->type = type;
 		word->encoding = encoding;
-		words = g_list_append(words, word);
+		words = g_list_append (words, word);
 	}
+	
+	return words;
+}
 
-	/* now scan the list, checking for words of similar types that can be merged */
+static void
+header_encode_phrase_merge_words (GList **wordsp)
+{
+	GList *wordl, *nextl, *words = *wordsp;
+	struct _phrase_word *word, *next;
+	
+	/* scan the list, checking for words of similar types that can be merged */
 	wordl = words;
 	while (wordl) {
 		word = wordl->data;
 		/* leave atoms as atoms (unless they're surrounded by quoted words??) */
 		if (word->type != WORD_ATOM) {
-			nextl = g_list_next(wordl);
+			nextl = g_list_next (wordl);
 			while (nextl) {
 				next = nextl->data;
 				/* merge nodes of the same type AND we are not creating too long a string */
 				if (word->type == next->type) {
 					if (next->end - word->start < CAMEL_FOLD_PREENCODED) {
 						word->end = next->end;
-						words = g_list_remove_link(words, nextl);
-						g_free(next);
-						nextl = g_list_next(wordl);
+						words = g_list_remove_link (words, nextl);
+						g_free (next);
+						nextl = g_list_next (wordl);
 					} else {
-						/* if it is going to be too long, make sure we include the separating whitespace */
+						/* if it is going to be too long, make sure we include the
+						   separating whitespace */
 						word->end = next->start;
 						break;
 					}
@@ -1432,51 +1510,89 @@ header_encode_phrase(const unsigned char *in)
 				}
 			}
 		}
-		wordl = g_list_next(wordl);
+		wordl = g_list_next (wordl);
 	}
+	
+	*wordsp = words;
+}
 
+/* encodes a phrase sequence (different quoting/encoding rules to strings) */
+char *
+header_encode_phrase (const unsigned char *in)
+{
+	struct _phrase_word *word = NULL, *last_word = NULL;
+	GList *words, *wordl;
+	GString *out;
+	char *outstr;
+	
+	if (in == NULL)
+		return NULL;
+	
+	words = header_encode_phrase_get_words (in);
+	if (!words)
+		return NULL;
+	
+	header_encode_phrase_merge_words (&words);
+	
+	out = g_string_new ("");
+	
 	/* output words now with spaces between them */
 	wordl = words;
 	while (wordl) {
+		const char *start;
+		int len;
+		
 		word = wordl->data;
+		
+		/* append correct number of spaces between words */
+		if (last_word && !(last_word->type == WORD_2047 && word->type == WORD_2047)) {
+			/* one or both of the words are not encoded so we write the spaces out untouched */
+			len = word->start - last_word->end;
+			out = g_string_append_len (out, last_word->end, len);
+		}
+		
 		switch (word->type) {
 		case WORD_ATOM:
-			out = g_string_append_len(out, word->start, word->end-word->start);
+			out = g_string_append_len (out, word->start, word->end - word->start);
 			break;
 		case WORD_QSTRING:
-			quote_word(out, TRUE, word->start, word->end-word->start);
+			quote_word (out, TRUE, word->start, word->end - word->start);
 			break;
 		case WORD_2047:
+			if (last_word && last_word->type == WORD_2047) {
+				/* include the whitespace chars between these 2 words in the
+                                   resulting rfc2047 encoded word. */
+				len = word->end - last_word->end;
+				start = last_word->end;
+				
+				/* encoded words need to be separated by linear whitespace */
+				g_string_append_c (out, ' ');
+			} else {
+				len = word->end - word->start;
+				start = word->start;
+			}
+			
 			if (word->encoding == 1)
-				rfc2047_encode_word(out, word->start, word->end-word->start, "ISO-8859-1", IS_PSAFE);
+				rfc2047_encode_word (out, start, len, "ISO-8859-1", IS_PSAFE);
 			else
-				rfc2047_encode_word(out, word->start, word->end-word->start,
-						    camel_charset_best(word->start, word->end-word->start), IS_PSAFE);
+				rfc2047_encode_word (out, start, len,
+						     camel_charset_best (start, len), IS_PSAFE);
 			break;
 		}
-
-		/* copy across the right number of spaces between words */
-		nextl = g_list_next(wordl);
-		if (nextl) {
-			int i;
-			next = nextl->data;
-			/* if they are adjacent, it means we already had the spaces encoded internally,
-			   so now we just need to output 1 space */
-			i=next->start-word->end;
-			if (i==0)
-				i=1;
-			for (;i>0;i--)
-				out = g_string_append_c(out, ' ');
-		}
-
-		g_free(word);
-		wordl = g_list_next(wordl);
+		
+		g_free (last_word);
+		wordl = g_list_next (wordl);
+		
+		last_word = word;
 	}
+	
 	/* and we no longer need the list */
-	g_list_free(words);
-
+	g_free (word);
+	g_list_free (words);
+	
 	outstr = out->str;
-	g_string_free(out, FALSE);
+	g_string_free (out, FALSE);
+	
 	return outstr;
 }
 
@@ -2353,7 +2469,7 @@ header_param_list_format_append(GString *out, struct _header_param *p)
 		g_string_sprintfa(out, "%s=", p->name);
 
 		for (ch = p->value; *ch; ch++) {
-			if (is_tspecial(*ch))
+			if (is_tspecial (*ch) || is_lwsp (*ch))
 				break;
 		}
 		if (!*ch)
@@ -2619,10 +2735,10 @@ header_decode_date(const char *in, int *saveoffset)
 				char *newdate;
 
 				w(g_warning("day not followed by ',' it's probably a broken mail client, so we'll ignore its date entirely"));
-				printf ("Giving it one last chance...\n");
+				w(printf ("Giving it one last chance...\n"));
 				newdate = parse_broken_date (in);
 				if (newdate) {
-					printf ("Got: %s\n", newdate);
+					w(printf ("Got: %s\n", newdate));
 					t = header_decode_date (newdate, saveoffset);
 					g_free (newdate);
 					return t;
@@ -2646,10 +2762,12 @@ header_decode_date(const char *in, int *saveoffset)
 		g_free(monthname);
 	}
 	year = header_decode_int(&inptr);
-	if (year<100) {
+	if (year < 69 || (year >= 100 && year < 1900)) {
+		tm.tm_year = 100 + year;
+	} else if (year < 100) {
 		tm.tm_year = year;
 	} else {
-		tm.tm_year = year-1900;
+		tm.tm_year = year - 1900;
 	}
 	/* get the time ... yurck */
 	tm.tm_hour = header_decode_int(&inptr);
@@ -2892,6 +3010,70 @@ header_raw_clear(struct _header_raw **list)
 	*list = NULL;
 }
 
+char *
+header_msgid_generate (void)
+{
+	char host[MAXHOSTNAMELEN];
+#ifdef ENABLE_THREADS
+	static pthread_mutex_t count_lock = PTHREAD_MUTEX_INITIALIZER;
+#define COUNT_LOCK() pthread_mutex_lock (&count_lock)
+#define COUNT_UNLOCK() pthread_mutex_unlock (&count_lock)
+#else
+#define COUNT_LOCK()
+#define COUNT_UNLOCK()
+#endif /* ENABLE_THREADS */
+	static gint count = 0;
+	gint hrv;
+	char *ret;
+	
+	hrv = gethostname (host, sizeof (host));
+	
+	COUNT_LOCK ();
+	ret = g_strdup_printf ("%d.%d.%d.camel@%s", (gint) time (NULL), getpid (), count++,
+			       (hrv == 0 && host && *host) ? host : "unknown.host");
+	COUNT_UNLOCK ();
+	
+	return ret;
+}
+
+
+static struct {
+	char *name;
+	char *pattern;
+} mail_list_magic[] = {
+	{ "Sender", " *owner-([^@]+)" },
+	{ "Return-Path", " *owner-([^@]+)" },
+	{ "X-BeenThere", " *([^@]+)" },
+	{ "Delivered-To", " *mailing list ([^@]+)" },
+	{ "X-Mailing-List", " *([^@]+)" },
+	{ "X-Loop", " *([^@]+)" },
+	{ "List-Id", " *([^<]+)" },
+};
+
+char *
+header_raw_check_mailing_list(struct _header_raw **list)
+{
+	const char *v;
+	regex_t pattern;
+	regmatch_t match[2];
+	int i;
+
+	for (i=0;i<sizeof(mail_list_magic)/sizeof(mail_list_magic[0]);i++) {
+		if (regcomp(&pattern, mail_list_magic[i].pattern, REG_EXTENDED|REG_ICASE) == -1) {
+			g_warning("Internal error, compiling regex failed: %s: %s", mail_list_magic[i].pattern, strerror(errno));
+			continue;
+		}
+
+		v = header_raw_find(list, mail_list_magic[i].name, NULL);
+		if (v != NULL && regexec(&pattern, v, 2, match, 0) == 0 && match[1].rm_so != -1) {
+			regfree(&pattern);
+			return g_strndup(v+match[1].rm_so, match[1].rm_eo-match[1].rm_so);
+		}
+		regfree(&pattern);
+	}
+
+	return NULL;
+}
 
 /* ok, here's the address stuff, what a mess ... */
 struct _header_address *header_address_new(void)
@@ -3103,6 +3285,132 @@ header_address_list_format(struct _header_address *a)
 	return ret;
 }
 
+#if 0
+static const char *
+header_fold_next_space (const char *in)
+{
+	register const char *inptr = in;
+	gboolean escaped = FALSE;
+	
+	if (is_lwsp (*inptr))
+		return inptr;
+	
+	do {
+		if (*inptr == '\\') {
+			escaped = TRUE;
+		} else if (*inptr == '"' && !escaped) {
+			/* find the end of this quoted section */
+			for (inptr++; *inptr; inptr++) {
+				if (*inptr == '"' && *(inptr-1) != '\\')
+					break;
+			}
+		} else {
+			escaped = FALSE;
+		}
+		
+		inptr++;
+	} while (*inptr && !is_lwsp (*inptr));
+	
+	if (*inptr)
+		return inptr;
+	else
+		return NULL;
+}
+
+/* I wonder if this might be better for folding headers? */
+char *
+header_fold (const char *in, int headerlen, gboolean force)
+{
+	const char *inptr = in, *space, *p, *n;
+	gboolean needunfold = FALSE;
+	int len, outlen, i;
+	GString *out;
+	char *ret;
+	
+	if (in == NULL)
+		return NULL;
+	
+	/* first, check to see if we even need to fold */
+	len = headerlen + 2;
+	p = in;
+	while (*p) {
+		n = strchr (p, '\n');
+		if (n == NULL)
+			n = p + strlen (p);
+		else
+			needunfold = TRUE;
+		
+		len += n - p;
+		
+		if (len >= CAMEL_FOLD_SIZE)
+			break;
+		len = 0;
+		p = n + 1;
+	}
+	
+	if (len < CAMEL_FOLD_SIZE)
+		return g_strdup (in);
+	
+	/* we need to fold, so first unfold (if we need to), then process */
+	if (needunfold)
+		inptr = in = header_unfold (in);
+	
+	out = g_string_new ("");
+	outlen = headerlen + 2;
+	while (*inptr) {
+		if (force)
+			space = strchr (inptr, ' ');
+		else
+			space = header_fold_next_space (inptr);
+		
+		if (space) {
+			len = space - inptr + 1;
+		} else {
+			len = strlen (inptr);
+		}
+		
+		d(printf ("next word '%.*s'\n", len, inptr));
+		
+		if (outlen + len > CAMEL_FOLD_SIZE) {
+			d(printf("outlen = %d wordlen = %d\n", outlen, len));
+			/* strip trailing space */
+			if (out->len > 0 && out->str[out->len-1] == ' ')
+				g_string_truncate (out, out->len-1);
+			g_string_append (out, "\n\t");
+			outlen = 1;
+			
+			if (force) {
+				/* check for very long words, just cut them up */
+				while (outlen + len > CAMEL_FOLD_SIZE) {
+					for (i = 0; i < CAMEL_FOLD_SIZE - outlen; i++)
+						g_string_append_c (out, inptr[i]);
+					inptr += CAMEL_FOLD_SIZE - outlen;
+					len -= CAMEL_FOLD_SIZE - outlen;
+					g_string_append (out, "\n\t");
+					outlen = 1;
+				}
+			}
+		}
+		
+		outlen += len;
+		
+		for (i = 0; i < len; i++)
+			g_string_append_c (out, inptr[i]);
+		
+		inptr += len;
+	}
+	
+	ret = out->str;
+	g_string_free (out, FALSE);
+	
+	if (needunfold)
+		g_free ((char *)in);
+	
+	return ret;	
+}
+#endif
+
+
 /* simple header folding */
 /* will work even if the header is already folded */
 char *
@@ -3123,10 +3431,11 @@ header_fold(const char *in, int headerlen)
 	while (*p) {
 		n = strchr(p, '\n');
 		if (n == NULL) {
-			n = p+strlen(p);
-		} else {
-			needunfold = TRUE;
+			len += strlen (p);
+			break;
 		}
+
+		needunfold = TRUE;
 		len += n-p;
 		
 		if (len >= CAMEL_FOLD_SIZE)
@@ -3296,18 +3605,3 @@ void run_test(void)
 }
 
 #endif /* BUILD_TABLE */
-
-char *
-header_msgid_generate (void)
-{
-	gchar host [256], domain [768];
-	static gint count = 0;
-	gint hrv, drv;
-
-	hrv = gethostname (host, sizeof (host));
-	drv = getdomainname (domain, sizeof (domain));
-
-	return g_strdup_printf ("%d.%d.%d.camel@%s.%s", (gint) time (NULL), getpid (), count++,
-				(hrv == 0 && host && *host) ? host : "unknown.host",
-				(drv && domain && *domain) ? domain : "unknown.domain");
-}
