@@ -38,15 +38,16 @@ static CamelObjectClass *parent_class = NULL;
 /* Returns the class for a CamelService */
 #define CSERV_CLASS(so) CAMEL_SERVICE_CLASS (CAMEL_OBJECT_GET_CLASS(so))
 
+static void construct (CamelService *service, CamelSession *session,
+		       CamelProvider *provider, CamelURL *url,
+		       CamelException *ex);
 static gboolean service_connect(CamelService *service, CamelException *ex);
 static gboolean service_disconnect(CamelService *service, gboolean clean,
 				   CamelException *ex);
 /*static gboolean is_connected (CamelService *service);*/
-static GList *  query_auth_types (CamelService *service, gboolean connect, CamelException *ex);
-static void     free_auth_types (CamelService *service, GList *authtypes);
+static GList *  query_auth_types (CamelService *service, CamelException *ex);
 static char *   get_name (CamelService *service, gboolean brief);
 static char *   get_path (CamelService *service);
-static gboolean check_url (CamelService *service, CamelException *ex);
 
 
 static void
@@ -55,11 +56,10 @@ camel_service_class_init (CamelServiceClass *camel_service_class)
 	parent_class = camel_type_get_global_classfuncs (CAMEL_OBJECT_TYPE);
 
 	/* virtual method definition */
+	camel_service_class->construct = construct;
 	camel_service_class->connect = service_connect;
 	camel_service_class->disconnect = service_disconnect;
-	/*camel_service_class->is_connected = is_connected;*/
 	camel_service_class->query_auth_types = query_auth_types;
-	camel_service_class->free_auth_types = free_auth_types;
 	camel_service_class->get_name = get_name;
 	camel_service_class->get_path = get_path;
 }
@@ -125,82 +125,66 @@ camel_service_get_type (void)
 	return camel_service_type;
 }
 
-static gboolean
-check_url (CamelService *service, CamelException *ex)
+
+static void
+construct (CamelService *service, CamelSession *session,
+	   CamelProvider *provider, CamelURL *url, CamelException *ex)
 {
 	char *url_string;
 
-	if (((service->provider->url_flags & CAMEL_URL_NEED_USER)
-	     == CAMEL_URL_NEED_USER) &&
-	    (service->url->user == NULL || service->url->user[0] == '\0')) {
-		url_string = camel_url_to_string (service->url, FALSE);
+	if (CAMEL_PROVIDER_NEEDS (provider, CAMEL_URL_PART_USER) &&
+	    (url->user == NULL || url->user[0] == '\0')) {
+		url_string = camel_url_to_string (url, CAMEL_URL_HIDE_PASSWORD);
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_URL_INVALID,
 				      _("URL '%s' needs a username component"),
 				      url_string);
 		g_free (url_string);
-		return FALSE;
-	} else if (((service->provider->url_flags & CAMEL_URL_NEED_HOST)
-		    == CAMEL_URL_NEED_HOST) &&
-		   (service->url->host == NULL || service->url->host[0] == '\0')) {
-		url_string = camel_url_to_string (service->url, FALSE);
+		return;
+	} else if (CAMEL_PROVIDER_NEEDS (provider, CAMEL_URL_PART_HOST) &&
+		   (url->host == NULL || url->host[0] == '\0')) {
+		url_string = camel_url_to_string (url, CAMEL_URL_HIDE_PASSWORD);
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_URL_INVALID,
 				      _("URL '%s' needs a host component"),
 				      url_string);
 		g_free (url_string);
-		return FALSE;
-	} else if (((service->provider->url_flags & CAMEL_URL_NEED_PATH)
-		    == CAMEL_URL_NEED_PATH) &&
-		   (service->url->path == NULL || service->url->path[0] == '\0')) {
-		url_string = camel_url_to_string (service->url, FALSE);
+		return;
+	} else if (CAMEL_PROVIDER_NEEDS (provider, CAMEL_URL_PART_PATH) &&
+		   (url->path == NULL || url->path[0] == '\0')) {
+		url_string = camel_url_to_string (url, CAMEL_URL_HIDE_PASSWORD);
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_URL_INVALID,
 				      _("URL '%s' needs a path component"),
 				      url_string);
 		g_free (url_string);
-		return FALSE;
+		return;
 	}
 
-	return TRUE;
+	service->provider = provider;
+	service->url = url;
+	service->session = session;
+	camel_object_ref (CAMEL_OBJECT (session));
+
+	service->connected = FALSE;
 }
 
 /**
- * camel_service_new: create a new CamelService or subtype
- * @type: the CamelType of the class to create
+ * camel_service_construct:
+ * @service: the CamelService
  * @session: the session for the service
  * @provider: the service's provider
  * @url: the default URL for the service (may be NULL)
  * @ex: a CamelException
  *
- * Creates a new CamelService (or one of its subtypes), initialized
- * with the given parameters.
- *
- * Return value: the CamelService, or NULL.
+ * Constructs a CamelService initialized with the given parameters.
  **/
-CamelService *
-camel_service_new (CamelType type, CamelSession *session,
-		   CamelProvider *provider, CamelURL *url,
-		   CamelException *ex)
+void
+camel_service_construct (CamelService *service, CamelSession *session,
+			 CamelProvider *provider, CamelURL *url,
+			 CamelException *ex)
 {
-	CamelService *service;
+	g_return_if_fail (CAMEL_IS_SERVICE (service));
+	g_return_if_fail (CAMEL_IS_SESSION (session));
 
-	g_return_val_if_fail (CAMEL_IS_SESSION (session), NULL);
-
-	service = CAMEL_SERVICE (camel_object_new (type));
-
-	/*service->connect_level = 0;*/
-
-	service->provider = provider;
-	service->url = url;
-	if (!check_url (service, ex)) {
-		camel_object_unref (CAMEL_OBJECT (service));
-		return NULL;
-	}
-
-	service->session = session;
-	camel_object_ref (CAMEL_OBJECT (session));
-
-	service->connected = FALSE;
-
-	return service;
+	CSERV_CLASS (service)->construct (service, session, provider, url, ex);
 }
 
 
@@ -346,29 +330,35 @@ get_path (CamelService *service)
 	GString *gpath;
 	char *path;
 	CamelURL *url = service->url;
-	int flags = service->provider->url_flags;
+	CamelProvider *prov = service->provider;
 
 	/* A sort of ad-hoc default implementation that works for our
 	 * current set of services.
 	 */
 
 	gpath = g_string_new (service->provider->protocol);
-	if (flags & CAMEL_URL_ALLOW_USER) {
-		if (flags & CAMEL_URL_ALLOW_HOST) {
+	if (CAMEL_PROVIDER_ALLOWS (prov, CAMEL_URL_PART_USER)) {
+		if (CAMEL_PROVIDER_ALLOWS (prov, CAMEL_URL_PART_HOST)) {
 			g_string_sprintfa (gpath, "/%s@%s",
 					   url->user ? url->user : "",
 					   url->host ? url->host : "");
+			
+			if (url->port)
+				g_string_sprintfa (gpath, ":%d", url->port);
 		} else {
 			g_string_sprintfa (gpath, "/%s%s",
-			   url->user ? url->user : "",
-			   ((flags & CAMEL_URL_NEED_USER) == CAMEL_URL_NEED_USER) ? "" : "@");
+					   url->user ? url->user : "",
+					   CAMEL_PROVIDER_NEEDS (prov, CAMEL_URL_PART_USER) ? "" : "@");
 		}
-	} else if (flags & CAMEL_URL_ALLOW_HOST) {
+	} else if (CAMEL_PROVIDER_ALLOWS (prov, CAMEL_URL_PART_HOST)) {
 		g_string_sprintfa (gpath, "/%s%s",
-		   ((flags & CAMEL_URL_NEED_HOST) == CAMEL_URL_NEED_HOST) ? "" : "@",
-		   url->host ? url->host : "");
+				   CAMEL_PROVIDER_NEEDS (prov, CAMEL_URL_PART_HOST) ? "" : "@",
+				   url->host ? url->host : "");
+		
+		if (url->port)
+			g_string_sprintfa (gpath, ":%d", url->port);
 	}
-	if ((flags & CAMEL_URL_NEED_PATH) == CAMEL_URL_NEED_PATH) {
+	if (CAMEL_PROVIDER_NEEDS (prov, CAMEL_URL_PART_PATH)) {
 		g_string_sprintfa (gpath, "%s%s",
 				   *url->path == '/' ? "" : "/",
 				   url->path);
@@ -430,7 +420,7 @@ camel_service_get_provider (CamelService *service)
 }
 
 static GList *
-query_auth_types (CamelService *service, gboolean connect, CamelException *ex)
+query_auth_types (CamelService *service, CamelException *ex)
 {
 	return NULL;
 }
@@ -438,58 +428,29 @@ query_auth_types (CamelService *service, gboolean connect, CamelException *ex)
 /**
  * camel_service_query_auth_types:
  * @service: a CamelService
- * @connect: specifies whether or not to connect
  * @ex: a CamelException
  *
  * This is used by the mail source wizard to get the list of
  * authentication types supported by the protocol, and information
  * about them.
  *
- * This may be called on a service with or without an associated URL.
- * If there is no URL, the routine must return a generic answer. If
- * the service does have a URL, the routine should connect to the
- * server and query what authentication mechanisms it supports only if
- * @connect is TRUE. If it cannot do that for any reason, it should
- * set @ex accordingly.
- *
  * Return value: a list of CamelServiceAuthType records. The caller
- * must free the list by calling camel_service_free_auth_types when
- * it is done.
+ * must free the list with g_list_free() when it is done with it.
  **/
 GList *
-camel_service_query_auth_types (CamelService *service, gboolean connect, CamelException *ex)
+camel_service_query_auth_types (CamelService *service, CamelException *ex)
 {
 	GList *ret;
 
 	/* note that we get the connect lock here, which means the callee
 	   must not call the connect functions itself */
 	CAMEL_SERVICE_LOCK(service, connect_lock);
-	ret = CSERV_CLASS (service)->query_auth_types (service, connect, ex);
+	ret = CSERV_CLASS (service)->query_auth_types (service, ex);
 	CAMEL_SERVICE_UNLOCK(service, connect_lock);
 
 	return ret;
 }
 
-static void
-free_auth_types (CamelService *service, GList *authtypes)
-{
-	;
-}
-
-/**
- * camel_service_free_auth_types:
- * @service: the service
- * @authtypes: the list of authtypes
- *
- * This frees the data allocated by camel_service_query_auth_types().
- **/
-void
-camel_service_free_auth_types (CamelService *service, GList *authtypes)
-{
-	CSERV_CLASS (service)->free_auth_types (service, authtypes);
-}
-
-
 /* URL utility routines */
 
 /**
