@@ -32,7 +32,6 @@
 #include <string.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <gal/util/e-util.h>
 #include "camel-session.h"
 #include "camel-store.h"
 #include "camel-transport.h"
@@ -40,6 +39,8 @@
 #include "string-utils.h"
 #include "camel-url.h"
 #include "hash-table-utils.h"
+#include <gal/util/e-util.h>
+#include "camel-vee-store.h"
 
 #include "camel-private.h"
 
@@ -47,6 +48,22 @@
 
 static CamelObjectClass *parent_class;
 
+/* The vfolder provider is always avilable */
+static CamelProvider vee_provider = {
+	"vfolder",
+	N_("Virtual folder email provider"),
+
+	N_("For reading mail as a query of another set of folders"),
+
+	"vfolder",
+
+	0, /* flags */
+
+	0, /* url_flags */
+
+	/* ... */
+};
+
 static void
 camel_session_init (CamelSession *session)
 {
@@ -132,9 +149,14 @@ camel_session_new (const char *storage_path,
 	CamelSession *session = CAMEL_SESSION (camel_object_new (CAMEL_SESSION_TYPE));
 
 	session->storage_path = g_strdup (storage_path);
-	session->authenticator = authenticator;
+	session->authenticator = authenticator;	
 	session->registrar = registrar;
 	session->remover = remover;
+
+	vee_provider.object_types[CAMEL_PROVIDER_STORE] = camel_vee_store_get_type();
+	vee_provider.service_cache = g_hash_table_new(camel_url_hash, camel_url_equal);
+	camel_session_register_provider(session, &vee_provider);
+
 	return session;
 }
 
@@ -224,6 +246,63 @@ camel_session_list_providers (CamelSession *session, gboolean load)
 	return list;
 }
 
+static CamelProvider *
+get_provider_locked (CamelSession *session, const char *protocol,
+		     CamelException *ex)
+{
+	CamelProvider *provider;
+
+	provider = g_hash_table_lookup (session->providers, protocol);
+	if (!provider) {
+		/* See if there's one we can load. */
+		char *path;
+
+		path = g_hash_table_lookup (session->modules, protocol);
+		if (path) {
+			camel_provider_load (session, path, ex);
+			if (camel_exception_is_set (ex))
+				return NULL;
+		}
+		provider = g_hash_table_lookup (session->providers, protocol);
+	}
+
+	if (!provider) {
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_URL_INVALID,
+				      _("No provider available for protocol `%s'"),
+				      protocol);
+	}
+
+	return provider;
+}
+
+/**
+ * camel_session_get_provider:
+ * @session: the session
+ * @url_string: the URL for the service whose provider you want
+ * @ex: a CamelException
+ *
+ * This returns the CamelProvider that would be used to handle
+ * @url_string, loading it in from disk if necessary.
+ *
+ * Return value: the provider, or %NULL, in which case @ex will be set.
+ **/
+CamelProvider *
+camel_session_get_provider (CamelSession *session, const char *url_string,
+			    CamelException *ex)
+{
+	CamelProvider *provider;
+	char *protocol;
+
+	protocol = g_strndup (url_string, strcspn (url_string, ":"));
+	CAMEL_SESSION_LOCK(session, lock);
+	provider = get_provider_locked (session, protocol, ex);
+	CAMEL_SESSION_UNLOCK(session, lock);
+	g_free (protocol);
+
+	return provider;
+}
+
+
 static void
 service_cache_remove (CamelService *service, gpointer event_data, gpointer user_data)
 {
@@ -276,28 +355,14 @@ camel_session_get_service (CamelSession *session, const char *url_string,
 	/* We need to look up the provider so we can then lookup
 	   the service in the provider's cache */
 	CAMEL_SESSION_LOCK(session, lock);
-	provider = g_hash_table_lookup (session->providers, url->protocol);
-	if (!provider) {
-		/* See if there's one we can load. */
-		char *path;
-
-		path = g_hash_table_lookup (session->modules, url->protocol);
-		if (path) {
-			camel_provider_load (session, path, ex);
-			if (camel_exception_get_id (ex) !=
-			    CAMEL_EXCEPTION_NONE) {
-				camel_url_free (url);
-				CAMEL_SESSION_UNLOCK(session, lock);
-				return NULL;
-			}
-		}
-		provider = g_hash_table_lookup (session->providers, url->protocol);
-	}
-
-	if (!provider || !provider->object_types[type]) {
+	provider = get_provider_locked (session, url->protocol, ex);
+	if (provider && !provider->object_types[type]) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_URL_INVALID,
 				      _("No provider available for protocol `%s'"),
 				      url->protocol);
+		provider = NULL;
+	}
+	if (!provider) {
 		camel_url_free (url);
 		CAMEL_SESSION_UNLOCK(session, lock);
 		return NULL;
@@ -312,8 +377,12 @@ camel_session_get_service (CamelSession *session, const char *url_string,
 		return service;
 	}
 
-	service = camel_service_new (provider->object_types[type], session, provider, url, ex);
-	if (service) {
+	service = (CamelService *)camel_object_new (provider->object_types[type]);
+	camel_service_construct (service, session, provider, url, ex);
+	if (camel_exception_is_set (ex)) {
+		camel_object_unref (CAMEL_OBJECT (service));
+		service = NULL;
+	} else {
 		g_hash_table_insert (provider->service_cache, url, service);
 		camel_object_hook_event (CAMEL_OBJECT (service), "finalize", (CamelObjectEventHookFunc) service_cache_remove, session);
 	}
@@ -399,7 +468,7 @@ camel_session_get_storage_path (CamelSession *session, CamelService *service,
 /**
  * camel_session_query_authenticator: query the session authenticator
  * @session: session object
- * @mode: %CAMEL_AUTHENTICATOR_ASK or %CAMEL_AUTHENTICATOR_TELL
+ * @mode: %CAMEL_AUTHENTICATOR_ASK, %CAMEL_AUTHENTICATOR_TELL or CAMEL_AUTHENTICATOR_ACCEPT
  * @data: prompt to query user with, or data to cache
  * @secret: whether or not the data is secret (eg, a password)
  * @service: the service this query is being made by
@@ -425,9 +494,14 @@ camel_session_get_storage_path (CamelSession *session, CamelService *service,
  * caching anything about that datum (eg, because the data is a
  * password that turned out to be incorrect).
  *
+ * If @mode is %CAMEL_AUTHENTICATOR_ACCEPT, then @data is a YES/NO
+ * question to ask the user (if the application doesn't already have
+ * the answer cached). Return GINT_TO_POINTER(TRUE) on accept, or
+ * GINT_TO_POINTER(FALSE) to decline.
+ *
  * Return value: the authentication information or %NULL.
  **/
-char *
+gpointer
 camel_session_query_authenticator (CamelSession *session,
 				   CamelAuthCallbackMode mode,
 				   char *prompt, gboolean secret,
@@ -438,30 +512,6 @@ camel_session_query_authenticator (CamelSession *session,
 				       service, item, ex);
 }
 
-#ifdef U_CANT_TOUCH_THIS
-/**
- * camel_session_query_cert_authenticator:
- * @session: session object
- * @prompt: prompt to query user with
- *
- * This function is used by SSL to discuss certificate authentication
- * information with the application and/or user. Allows the user to
- * override the SSL certificate authenticator, which, at this point,
- * must have failed to authenticate the server.
- *
- * UI should be a Yes/No prompt probably defaulting to No.
- *
- * Return value: TRUE if the user decides that the SSL connection
- * should continue with the untrusted server or FALSE otherwise.
- **/
-gboolean
-camel_session_query_cert_authenticator (CamelSession *session,
-					char *prompt)
-{
-	return session->cert_authenticator (prompt);
-}
-#endif /* U_CANT_TOUCH_THIS */
-
 
 /**
  * camel_session_register_timeout: Register a timeout to be called
@@ -509,256 +559,3 @@ camel_session_remove_timeout (CamelSession *session, guint handle)
 {
 	return session->remover (handle);
 }
-
-/* ********************************************************************** */
-
-struct _CamelCancel {
-	pthread_t id;		/* id of running thread */
-	guint32 flags;		/* cancelled ? */
-	int blocked;		/* cancellation blocked depth */
-	int refcount;
-#ifdef ENABLE_THREADS
-	EMsgPort *cancel_port;
-	int cancel_fd;
-	pthread_mutex_t lock;
-#endif
-};
-
-#define CAMEL_CANCEL_CANCELLED (1<<0)
-
-#ifdef ENABLE_THREADS
-#define CAMEL_CANCEL_LOCK(cc) pthread_mutex_lock(&cc->lock)
-#define CAMEL_CANCEL_UNLOCK(cc) pthread_mutex_unlock(&cc->lock)
-#define CAMEL_ACTIVE_LOCK() pthread_mutex_lock(&cancel_active_lock)
-#define CAMEL_ACTIVE_UNLOCK() pthread_mutex_unlock(&cancel_active_lock)
-static pthread_mutex_t cancel_active_lock = PTHREAD_MUTEX_INITIALIZER;
-#else
-#define CAMEL_CANCEL_LOCK(cc)
-#define CAMEL_CANCEL_UNLOCK(cc)
-#define CAMEL_ACTIVE_LOCK()
-#define CAMEL_ACTIVE_UNLOCK()
-#endif
-
-static GHashTable *cancel_active;
-
-typedef struct _CamelCancelMsg {
-	EMsg msg;
-} CamelCancelMsg ;
-
-/* creates a new cancel handle */
-CamelCancel *camel_cancel_new(void)
-{
-	CamelCancel *cc;
-
-	cc = g_malloc0(sizeof(*cc));
-
-	cc->flags = 0;
-	cc->blocked = 0;
-	cc->refcount = 1;
-#ifdef ENABLE_THREADS
-	cc->id = ~0;
-	cc->cancel_port = e_msgport_new();
-	cc->cancel_fd = e_msgport_fd(cc->cancel_port);
-	pthread_mutex_init(&cc->lock, NULL);
-#endif
-
-	return cc;
-}
-
-void camel_cancel_reset(CamelCancel *cc)
-{
-#ifdef ENABLE_THREADS
-	CamelCancelMsg *msg;
-
-	while ((msg = (CamelCancelMsg *)e_msgport_get(cc->cancel_port)))
-		g_free(msg);
-#endif
-
-	cc->flags = 0;
-	cc->blocked = 0;
-}
-
-void camel_cancel_ref(CamelCancel *cc)
-{
-	CAMEL_CANCEL_LOCK(cc);
-	cc->refcount++;
-	CAMEL_CANCEL_UNLOCK(cc);
-}
-
-void camel_cancel_unref(CamelCancel *cc)
-{
-#ifdef ENABLE_THREADS
-	CamelCancelMsg *msg;
-
-	if (cc->refcount == 1) {
-		while ((msg = (CamelCancelMsg *)e_msgport_get(cc->cancel_port)))
-			g_free(msg);
-
-		e_msgport_destroy(cc->cancel_port);
-#endif
-		g_free(cc);
-	} else {
-		CAMEL_CANCEL_LOCK(cc);
-		cc->refcount--;
-		CAMEL_CANCEL_UNLOCK(cc);
-	}
-}
-
-/* block cancellation */
-void camel_cancel_block(CamelCancel *cc)
-{
-	CAMEL_CANCEL_LOCK(cc);
-
-	cc->blocked++;
-
-	CAMEL_CANCEL_UNLOCK(cc);
-}
-
-/* unblock cancellation */
-void camel_cancel_unblock(CamelCancel *cc)
-{
-	CAMEL_CANCEL_LOCK(cc);
-
-	cc->blocked--;
-
-	CAMEL_CANCEL_UNLOCK(cc);
-}
-
-static void
-cancel_thread(void *key, CamelCancel *cc, void *data)
-{
-	if (cc)
-		camel_cancel_cancel(cc);
-}
-
-/* cancels an operation */
-void camel_cancel_cancel(CamelCancel *cc)
-{
-	CamelCancelMsg *msg;
-
-	if (cc == NULL) {
-		if (cancel_active) {
-			CAMEL_ACTIVE_LOCK();
-			g_hash_table_foreach(cancel_active, (GHFunc)cancel_thread, NULL);
-			CAMEL_ACTIVE_UNLOCK();
-		}
-	} else if ((cc->flags & CAMEL_CANCEL_CANCELLED) == 0) {
-		d(printf("cancelling thread %d\n", cc->id));
-
-		CAMEL_CANCEL_LOCK(cc);
-		msg = g_malloc0(sizeof(*msg));
-		e_msgport_put(cc->cancel_port, (EMsg *)msg);
-		cc->flags |= CAMEL_CANCEL_CANCELLED;
-		CAMEL_CANCEL_UNLOCK(cc);
-	}
-}
-
-/* register a thread for cancellation */
-void camel_cancel_register(CamelCancel *cc)
-{
-	pthread_t id = pthread_self();
-
-	CAMEL_ACTIVE_LOCK();
-
-	if (cancel_active == NULL)
-		cancel_active = g_hash_table_new(NULL, NULL);
-
-	if (cc == NULL) {
-		cc = g_hash_table_lookup(cancel_active, (void *)id);
-		if (cc == NULL) {
-			cc = camel_cancel_new();
-		}
-	}
-
-	cc->id = id;
-	g_hash_table_insert(cancel_active, (void *)id, cc);
-
-	d(printf("registering thread %d for cancellation\n", id));
-
-	CAMEL_ACTIVE_UNLOCK();
-
-	camel_cancel_ref(cc);
-}
-
-/* remove a thread from being able to be cancelled */
-void camel_cancel_unregister(CamelCancel *cc)
-{
-	CAMEL_ACTIVE_LOCK();
-
-	if (cancel_active == NULL)
-		cancel_active = g_hash_table_new(NULL, NULL);
-
-	if (cc == NULL) {
-		cc = g_hash_table_lookup(cancel_active, (void *)pthread_self());
-		if (cc == NULL) {
-			g_warning("Trying to unregister a thread that was never registered for cancellation");
-		}
-	}
-
-	if (cc)
-		g_hash_table_remove(cancel_active, (void *)cc->id);
-
-	CAMEL_ACTIVE_UNLOCK();
-
-	d({if (cc) printf("unregistering thread %d for cancellation\n", cc->id)});
-
-	if (cc)
-		camel_cancel_unref(cc);
-}
-
-/* test for cancellation */
-gboolean camel_cancel_check(CamelCancel *cc)
-{
-	CamelCancelMsg *msg;
-
-	d(printf("checking for cancel in thread %d\n", pthread_self()));
-
-	if (cc == NULL) {
-		if (cancel_active) {
-			CAMEL_ACTIVE_LOCK();
-			cc = g_hash_table_lookup(cancel_active, (void *)pthread_self());
-			CAMEL_ACTIVE_UNLOCK();
-		}
-		if (cc == NULL)
-			return FALSE;
-	}
-
-	if (cc->blocked > 0) {
-		d(printf("ahah!  cancellation is blocked\n"));
-		return FALSE;
-	}
-
-	if (cc->flags & CAMEL_CANCEL_CANCELLED) {
-		d(printf("previously cancelled\n"));
-		return TRUE;
-	}
-
-	msg = (CamelCancelMsg *)e_msgport_get(cc->cancel_port);
-	if (msg) {
-		d(printf("Got cancellation message\n"));
-		CAMEL_CANCEL_LOCK(cc);
-		cc->flags |= CAMEL_CANCEL_CANCELLED;
-		CAMEL_CANCEL_UNLOCK(cc);
-		return TRUE;
-	}
-	return FALSE;
-}
-
-/* get the fd for cancellation waiting */
-int camel_cancel_fd(CamelCancel *cc)
-{
-	if (cc == NULL) {
-		if (cancel_active) {
-			CAMEL_ACTIVE_LOCK();
-			cc = g_hash_table_lookup(cancel_active, (void *)pthread_self());
-			CAMEL_ACTIVE_UNLOCK();
-		}
-		if (cc == NULL)
-			return -1;
-	}
-	if (cc->blocked)
-		return -1;
-
-	return cc->cancel_fd;
-}
-
