@@ -44,18 +44,15 @@ extern "C" {
 
 
 typedef enum {
-	CAMEL_AUTHENTICATOR_ASK, CAMEL_AUTHENTICATOR_TELL
+	CAMEL_AUTHENTICATOR_ASK,
+	CAMEL_AUTHENTICATOR_TELL,
+	CAMEL_AUTHENTICATOR_ACCEPT
 } CamelAuthCallbackMode;
 
-typedef char *(*CamelAuthCallback) (CamelAuthCallbackMode mode,
-				    char *data, gboolean secret,
-				    CamelService *service, char *item,
-				    CamelException *ex);
-
-#ifdef U_CANT_TOUCH_THIS
-/* this is just a guess as to what we'll actually need */
-typedef gboolean (*CamelBadCertCallback) (char *data);
-#endif
+typedef gpointer (*CamelAuthCallback) (CamelAuthCallbackMode mode,
+				       char *data, gboolean secret,
+				       CamelService *service, char *item,
+				       CamelException *ex);
 
 typedef gboolean (*CamelTimeoutCallback) (gpointer data);
 typedef guint (*CamelTimeoutRegisterCallback) (guint32 interval,
@@ -70,9 +67,6 @@ struct _CamelSession
 
 	char *storage_path;
 	CamelAuthCallback authenticator;
-#ifdef U_CANT_TOUCH_THIS
-	CamelBadCertCallback cert_authenticator;
-#endif
 	CamelTimeoutRegisterCallback registrar;
 	CamelTimeoutRemoveCallback remover;
 
@@ -101,6 +95,10 @@ void            camel_session_register_provider       (CamelSession *session,
 GList *         camel_session_list_providers          (CamelSession *session,
 						       gboolean load);
 
+CamelProvider * camel_session_get_provider            (CamelSession *session,
+						       const char *url_string,
+						       CamelException *ex);
+
 CamelService *  camel_session_get_service             (CamelSession *session,
 						       const char *url_string,
 						       CamelProviderType type,
@@ -119,7 +117,7 @@ char *          camel_session_get_storage_path (CamelSession *session,
 						CamelService *service,
 						CamelException *ex);
 
-char *          camel_session_query_authenticator (CamelSession *session,
+gpointer        camel_session_query_authenticator (CamelSession *session,
 						   CamelAuthCallbackMode mode,
 						   char *prompt,
 						   gboolean secret,
@@ -127,11 +125,6 @@ char *          camel_session_query_authenticator (CamelSession *session,
 						   char *item,
 						   CamelException *ex);
 
-#ifdef U_CANT_TOUCH_THIS
-gboolean        camel_session_query_cert_authenticator (CamelSession *session,
-							char *prompt);
-#endif
-
 guint           camel_session_register_timeout (CamelSession *session,
 						guint32 interval,
 						CamelTimeoutCallback callback,
@@ -140,24 +133,6 @@ guint           camel_session_register_timeout (CamelSession *session,
 gboolean        camel_session_remove_timeout (CamelSession *session,
 					      guint handle);
 
-
-/* cancellation helper stuff, not yet finalised */
-typedef struct _CamelCancel CamelCancel;
-/* main thread functions */
-CamelCancel *camel_cancel_new(void);
-void camel_cancel_ref(CamelCancel *cc);
-void camel_cancel_unref(CamelCancel *cc);
-void camel_cancel_reset(CamelCancel *cc);
-void camel_cancel_cancel(CamelCancel *cc);
-/* subthread functions */
-void camel_cancel_register(CamelCancel *cc);
-void camel_cancel_unregister(CamelCancel *cc);
-/* called internally by camel, for the current thread */
-void camel_cancel_block(CamelCancel *cc);
-void camel_cancel_unblock(CamelCancel *cc);
-gboolean camel_cancel_check(CamelCancel *cc);
-int camel_cancel_fd(CamelCancel *cc);
-
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
