@@ -50,10 +50,12 @@ typedef enum {
 #define IMAP_CAPABILITY_IMAP4REV1		(1 << 1)
 #define IMAP_CAPABILITY_STATUS			(1 << 2)
 #define IMAP_CAPABILITY_NAMESPACE		(1 << 3)
-#define IMAP_CAPABILITY_AUTH_KERBEROS_V4	(1 << 4)
-#define IMAP_CAPABILITY_AUTH_GSSAPI		(1 << 5)
-#define IMAP_CAPABILITY_UIDPLUS			(1 << 6)
-#define IMAP_CAPABILITY_LITERALPLUS		(1 << 7)
+#define IMAP_CAPABILITY_UIDPLUS			(1 << 4)
+#define IMAP_CAPABILITY_LITERALPLUS		(1 << 5)
+
+#define IMAP_PARAM_OVERRIDE_NAMESPACE		(1 << 0)
+#define IMAP_PARAM_CHECK_ALL			(1 << 1)
+#define IMAP_PARAM_FILTER_INBOX			(1 << 2)
 
 struct _CamelImapStore {
 	CamelRemoteStore parent_object;	
@@ -64,7 +66,8 @@ struct _CamelImapStore {
 	guint32 command;
 	
 	CamelImapServerLevel server_level;
-	guint32 capabilities;
+	guint32 capabilities, parameters;
+	GHashTable *authtypes;
 	
 	char *namespace, dir_sep, *storage_path, *base_url;
 	
@@ -84,6 +87,8 @@ typedef struct {
 /* Standard Camel function */
 CamelType camel_imap_store_get_type (void);
 
+gboolean camel_imap_store_check_online (CamelImapStore *store, CamelException *ex);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
