@@ -37,9 +37,8 @@ static CamelProvider mh_provider = {
 	N_("For storing local mail in MH-like mail directories"),
 	"mail",
 	CAMEL_PROVIDER_IS_STORAGE,
-	CAMEL_URL_NEED_PATH,
-	{0, 0},
-	NULL
+	CAMEL_URL_NEED_PATH | CAMEL_URL_PATH_IS_ABSOLUTE,
+	/* ... */
 };
 
 static CamelProvider mbox_provider = {
@@ -48,9 +47,8 @@ static CamelProvider mbox_provider = {
 	N_("For storing local mail in standard mbox format"),
 	"mail",
 	CAMEL_PROVIDER_IS_SOURCE | CAMEL_PROVIDER_IS_STORAGE,
-	CAMEL_URL_NEED_PATH,
-	{ 0, 0 },
-	NULL
+	CAMEL_URL_NEED_PATH | CAMEL_URL_PATH_IS_ABSOLUTE,
+	/* ... */
 };
 
 static CamelProvider maildir_provider = {
@@ -59,9 +57,8 @@ static CamelProvider maildir_provider = {
 	N_("For storing local mail in qmail maildir directories"),
 	"mail",
 	CAMEL_PROVIDER_IS_SOURCE | CAMEL_PROVIDER_IS_STORAGE,
-	CAMEL_URL_NEED_PATH,
-	{ 0, 0 },
-	NULL
+	CAMEL_URL_NEED_PATH | CAMEL_URL_PATH_IS_ABSOLUTE,
+	/* ... */
 };
 
 void camel_provider_module_init(CamelSession * session)
