@@ -20,8 +20,9 @@
  *  USA
  */
 
-#include "camel-mbox-summary.h"
-#include <camel/camel-mime-message.h>
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <sys/stat.h>
 #include <sys/uio.h>
@@ -30,6 +31,10 @@
 #include <string.h>
 #include <stdlib.h>
 
+#include "camel-mbox-summary.h"
+#include "camel/camel-mime-message.h"
+#include "camel/camel-operation.h"
+
 #define io(x)
 #define d(x) /*(printf("%s(%d): ", __FILE__, __LINE__),(x))*/
 
@@ -230,18 +235,26 @@ summary_rebuild(CamelMboxSummary *mbs, off_t offset, CamelException *ex)
 	CamelMimeParser *mp;
 	int fd;
 	int ok = 0;
+	struct stat st;
+	off_t size = 0;
 
 	/* FIXME: If there is a failure, it shouldn't clear the summary and restart,
 	   it should try and merge the summary info's.  This is a bit tricky. */
 
+	camel_operation_start(NULL, _("Summarising folder"));
+
 	fd = open(cls->folder_path, O_RDONLY);
 	if (fd == -1) {
 		printf("%s failed to open: %s", cls->folder_path, strerror(errno));
 		camel_exception_setv(ex, 1, _("Could not open folder: %s: summarising from position %ld: %s"),
 				     cls->folder_path, offset, strerror(errno));
+		camel_operation_end(NULL);
 		return -1;
 	}
 	
+	if (fstat(fd, &st) == 0)
+		size = st.st_size;
+
 	mp = camel_mime_parser_new();
 	camel_mime_parser_init_with_fd(mp, fd);
 	camel_mime_parser_scan_from(mp, TRUE);
@@ -262,13 +275,18 @@ summary_rebuild(CamelMboxSummary *mbs, off_t offset, CamelException *ex)
 			d(printf("mime parser state ran out? state is %d\n", camel_mime_parser_state(mp)));
 			camel_object_unref(CAMEL_OBJECT(mp));
 			/* end of file - no content? no error either */
+			camel_operation_end(NULL);
 			return 0;
 		}
 	}
 
 	while (camel_mime_parser_step(mp, NULL, NULL) == HSCAN_FROM) {
 		CamelMessageInfo *info;
+		unsigned int pc;
 
+		pc = ((unsigned int)camel_mime_parser_tell(mp) + 1) * 100 / size;		
+		camel_operation_progress(NULL, pc);
+		
 		info = camel_folder_summary_add_from_parser(s, mp);
 		if (info == NULL) {
 			camel_exception_setv(ex, 1, _("Fatal mail parser error near position %ld in folder %s"),
@@ -284,8 +302,6 @@ summary_rebuild(CamelMboxSummary *mbs, off_t offset, CamelException *ex)
 	
 	/* update the file size/mtime in the summary */
 	if (ok != -1) {
-		struct stat st;
-
 		if (stat(cls->folder_path, &st) == 0) {
 			camel_folder_summary_touch(s);
 			mbs->folder_size = st.st_size;
@@ -293,6 +309,8 @@ summary_rebuild(CamelMboxSummary *mbs, off_t offset, CamelException *ex)
 		}
 	}
 
+	camel_operation_end(NULL);
+
 	return ok;
 }
 
@@ -477,11 +495,14 @@ mbox_summary_sync_full(CamelLocalSummary *cls, gboolean expunge, CamelFolderChan
 
 	d(printf("performing full summary/sync\n"));
 
+	camel_operation_start(NULL, _("Synchronising folder"));
+
 	fd = open(cls->folder_path, O_RDONLY);
 	if (fd == -1) {
 		camel_exception_setv(ex, CAMEL_EXCEPTION_SYSTEM,
 				     _("Could not open folder to summarise: %s: %s"),
 				     cls->folder_path, strerror(errno));
+		camel_operation_end(NULL);
 		return -1;
 	}
 
@@ -502,6 +523,10 @@ mbox_summary_sync_full(CamelLocalSummary *cls, gboolean expunge, CamelFolderChan
 
 	count = camel_folder_summary_count(s);
 	for (i = 0; i < count; i++) {
+		int pc = (i+1)*100/count;
+
+		camel_operation_progress(NULL, pc);
+
 		info = (CamelMboxMessageInfo *)camel_folder_summary_index(s, i);
 
 		g_assert(info);
@@ -630,7 +655,8 @@ mbox_summary_sync_full(CamelLocalSummary *cls, gboolean expunge, CamelFolderChan
 	tmpname = NULL;
 
 	camel_object_unref((CamelObject *)mp);
-	
+	camel_operation_end(NULL);
+		
 	return 0;
  error:
 	if (fd != -1)
@@ -648,6 +674,8 @@ mbox_summary_sync_full(CamelLocalSummary *cls, gboolean expunge, CamelFolderChan
 	if (info)
 		camel_folder_summary_info_free(s, (CamelMessageInfo *)info);
 
+	camel_operation_end(NULL);
+
 	return -1;
 }
 
@@ -668,11 +696,15 @@ mbox_summary_sync_quick(CamelLocalSummary *cls, gboolean expunge, CamelFolderCha
 
 	d(printf("Performing quick summary sync\n"));
 
+	camel_operation_start(NULL, _("Synchronising folder"));
+
 	fd = open(cls->folder_path, O_RDWR);
 	if (fd == -1) {
 		camel_exception_setv(ex, CAMEL_EXCEPTION_SYSTEM,
 				     _("Could not open folder to summarise: %s: %s"),
 				     cls->folder_path, strerror(errno));
+
+		camel_operation_end(NULL);
 		return -1;
 	}
 
@@ -684,6 +716,9 @@ mbox_summary_sync_quick(CamelLocalSummary *cls, gboolean expunge, CamelFolderCha
 	count = camel_folder_summary_count(s);
 	for (i = 0; i < count; i++) {
 		int xevoffset;
+		int pc = (i+1)*100/count;
+
+		camel_operation_progress(NULL, pc);
 
 		info = (CamelMboxMessageInfo *)camel_folder_summary_index(s, i);
 
@@ -769,6 +804,8 @@ mbox_summary_sync_quick(CamelLocalSummary *cls, gboolean expunge, CamelFolderCha
 	}
 
 	camel_object_unref((CamelObject *)mp);
+
+	camel_operation_end(NULL);
 	
 	return 0;
  error:
@@ -779,6 +816,8 @@ mbox_summary_sync_quick(CamelLocalSummary *cls, gboolean expunge, CamelFolderCha
 	if (info)
 		camel_folder_summary_info_free(s, (CamelMessageInfo *)info);
 
+	camel_operation_end(NULL);
+
 	return -1;
 }
 
