@@ -35,6 +35,8 @@
 #include <unistd.h>
 #include <errno.h>
 
+#include "camel-operation.h"
+
 #ifdef HAVE_KRB4
 /* Specified nowhere */
 #define KPOP_PORT 1109
@@ -70,13 +72,13 @@ static void finalize (CamelObject *object);
 
 static gboolean pop3_connect (CamelService *service, CamelException *ex);
 static gboolean pop3_disconnect (CamelService *service, gboolean clean, CamelException *ex);
-static GList *query_auth_types (CamelService *service, gboolean connect, CamelException *ex);
+static GList *query_auth_types (CamelService *service, CamelException *ex);
 
 static CamelFolder *get_folder (CamelStore *store, const char *folder_name, 
 				guint32 flags, CamelException *ex);
-static char *get_folder_name (CamelStore *store, const char *folder_name, 
-			      CamelException *ex);
-static char *get_root_folder_name (CamelStore *store, CamelException *ex);
+
+static void init_trash (CamelStore *store);
+static CamelFolder *get_trash  (CamelStore *store, CamelException *ex);
 
 static int pop3_get_response (CamelPop3Store *store, char **ret, CamelException *ex);
 
@@ -88,9 +90,6 @@ camel_pop3_store_class_init (CamelPop3StoreClass *camel_pop3_store_class)
 		CAMEL_SERVICE_CLASS (camel_pop3_store_class);
 	CamelStoreClass *camel_store_class =
 		CAMEL_STORE_CLASS (camel_pop3_store_class);
-	/*CamelRemoteStoreClass *camel_remote_store_class =
-	 *	CAMEL_STORE_CLASS (camel_pop3_store_class);
-	 */
 
 	parent_class = CAMEL_REMOTE_STORE_CLASS(camel_type_get_global_classfuncs 
 						(camel_remote_store_get_type ()));
@@ -101,8 +100,8 @@ camel_pop3_store_class_init (CamelPop3StoreClass *camel_pop3_store_class)
 	camel_service_class->disconnect = pop3_disconnect;
 
 	camel_store_class->get_folder = get_folder;
-	camel_store_class->get_folder_name = get_folder_name;
-	camel_store_class->get_root_folder_name = get_root_folder_name;
+	camel_store_class->init_trash = init_trash;
+	camel_store_class->get_trash = get_trash;
 }
 
 
@@ -113,6 +112,8 @@ camel_pop3_store_init (gpointer object, gpointer klass)
 	CamelRemoteStore *remote_store = CAMEL_REMOTE_STORE (object);
 
 	remote_store->default_port = 110;
+	/* FIXME: what should this port be?? */
+	remote_store->default_ssl_port = 995;
 }
 
 CamelType
@@ -140,52 +141,40 @@ finalize (CamelObject *object)
 
 	if (pop3_store->apop_timestamp)
 		g_free (pop3_store->apop_timestamp);
+	if (pop3_store->implementation)
+		g_free (pop3_store->implementation);
 }
 
-static CamelServiceAuthType password_authtype = {
-	N_("Password"),
-
-	N_("This option will connect to the POP server using a plaintext "
-	   "password. This is the only option supported by many POP servers."),
-
-	"",
-	TRUE
-};
-
-static CamelServiceAuthType apop_authtype = {
-	"APOP",
-
-	N_("This option will connect to the POP server using an encrypted "
-	   "password via the APOP protocol. This may not work for all users "
-	   "even on servers that claim to support it."),
-
-	"+APOP",
-	TRUE
-};
-
-#ifdef HAVE_KRB4
-static CamelServiceAuthType kpop_authtype = {
-	"Kerberos 4 (KPOP)",
-
-	N_("This will connect to the POP server and use Kerberos 4 "
-	   "to authenticate to it."),
-
-	"+KPOP",
-	FALSE
-};
-#endif
-
 static gboolean
-connect_to_server (CamelService *service, /*gboolean real, */CamelException *ex)
+connect_to_server (CamelService *service, CamelException *ex)
 {
 	CamelPop3Store *store = CAMEL_POP3_STORE (service);
 	char *buf, *apoptime, *apopend;
-	gint status;
+	int status;
+	gboolean result;
+
 #ifdef HAVE_KRB4
-	gboolean kpop = service->url->authmech &&
-		!strcmp (service->url->authmech, "+KPOP");
+	gboolean set_port = FALSE;
+
+	kpop = (service->url->authmech &&
+		!strcmp (service->url->authmech, "+KPOP"));
+
+	if (kpop && service->url->port == 0) {
+		set_port = TRUE;
+		service->url->port = KPOP_PORT;
+	}
 #endif
 
+  	result = CAMEL_SERVICE_CLASS (parent_class)->connect (service, ex);
+
+#ifdef HAVE_KRB4
+	if (set_port)
+		service->url->port = 0;
+#endif
+
+	if (result == FALSE)
+		return FALSE;
+
 #ifdef HAVE_KRB4
 	if (kpop) {
 		KTEXT_ST ticket_st;
@@ -225,8 +214,16 @@ connect_to_server (CamelService *service, /*gboolean real, */CamelException *ex)
 
 	/* Read the greeting, check status */
 	status = pop3_get_response (store, &buf, ex);
-	if (status != CAMEL_POP3_OK)
+	switch (status) {
+	case CAMEL_POP3_ERR:
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
+				      _("Could not connect to server: %s"),
+				      buf);
+		g_free (buf);
+		/* fall through */
+	case CAMEL_POP3_FAIL:
 		return FALSE;
+	}
 
 	if (buf) {
 		apoptime = strchr (buf, '<');
@@ -250,7 +247,7 @@ connect_to_server (CamelService *service, /*gboolean real, */CamelException *ex)
 		char *p;
 		int len;
 
-		buf = camel_pop3_command_get_additional_data (store, ex);
+		buf = camel_pop3_command_get_additional_data (store, 0, ex);
 		if (camel_exception_is_set (ex))
 			return FALSE;
 
@@ -283,63 +280,50 @@ connect_to_server (CamelService *service, /*gboolean real, */CamelException *ex)
 	return TRUE;
 }
 
+extern CamelServiceAuthType camel_pop3_password_authtype;
+extern CamelServiceAuthType camel_pop3_apop_authtype;
+#ifdef HAVE_KRB4
+extern CamelServiceAuthType camel_pop3_kpop_authtype;
+#endif
+
 static GList *
-query_auth_types (CamelService *service, gboolean connect, CamelException *ex)
+query_auth_types (CamelService *service, CamelException *ex)
 {
 	CamelPop3Store *store = CAMEL_POP3_STORE (service);
 	GList *types = NULL;
 	gboolean passwd = TRUE, apop = TRUE;
+
+        types = CAMEL_SERVICE_CLASS (parent_class)->query_auth_types (service, ex);
+	if (camel_exception_is_set (ex))
+		return types;
+
+	passwd = connect_to_server (service, NULL);
+	apop = store->apop_timestamp != NULL;
+	if (passwd)
+		pop3_disconnect (service, TRUE, NULL);
+
 #ifdef HAVE_KRB4
-	gboolean kpop = TRUE;
-	int saved_port;
+	service->url->authtype = "+KPOP";
+	kpop = connect_to_server (service, NULL);
+	service->url->authtype = NULL;
+	if (kpop)
+		pop3_disconnect (service, TRUE, NULL);
 #endif
-	
-        types = CAMEL_SERVICE_CLASS (parent_class)->query_auth_types (service, connect, ex);
-	
-	if (connect) {
-		passwd = camel_service_connect (service, ex);
-		/*ignore the exception here; the server may just not support passwd */
-		
-		/* should we check apop too? */
-		apop = store->apop_timestamp != NULL;
-		if (passwd)
-			camel_service_disconnect (service, TRUE, ex);
-		camel_exception_clear (ex);
-		
+
+	if (passwd)
+		types = g_list_append (types, &camel_pop3_password_authtype);
+	if (apop)
+		types = g_list_append (types, &camel_pop3_apop_authtype);
 #ifdef HAVE_KRB4
-		saved_port = service->url->port;
-		service->url->port = KPOP_PORT;
-		kpop = camel_service_connect (service, ex);
-		service->url->port = saved_port;
-		/*ignore the exception here; the server may just not support kpop */
-		
-		if (kpop)
-			camel_service_disconnect (service, TRUE, ex);
-		camel_exception_clear (ex);
-#endif
-		
-		if (passwd)
-			types = g_list_append (types, &password_authtype);
-		if (apop)
-			types = g_list_append (types, &apop_authtype);
-#ifdef HAVE_KRB4
-		if (kpop)
-			types = g_list_append (types, &kpop_authtype);
-#endif
-		
-		if (!types) {
-			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
-					      _("Could not connect to POP server on "
-						"%s."), service->url->host);
-		}
-	} else {
-		types = g_list_append (types, &password_authtype);
-		types = g_list_append (types, &apop_authtype);
-#ifdef HAVE_KRB4
-		types = g_list_append (types, &kpop_authtype);
+	if (kpop)
+		types = g_list_append (types, &camel_pop3_kpop_authtype);
 #endif
+
+	if (!types) {
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
+				      _("Could not connect to POP server on "
+					"%s."), service->url->host);
 	}
-	
 	return types;
 }
 
@@ -447,40 +431,7 @@ pop3_connect (CamelService *service, CamelException *ex)
 {
 	char *errbuf = NULL;
 	gboolean tryagain, kpop = FALSE;
-	gboolean res;
 
-#ifdef HAVE_KRB4
-	gboolean set_port = FALSE;
-
-	kpop = (service->url->authmech &&
-		!strcmp (service->url->authmech, "+KPOP"));
-
-	if (kpop && service->url->port == 0) {
-		set_port = TRUE;
-		service->url->port = KPOP_PORT;
-	}
-#endif
-
-  	res = CAMEL_SERVICE_CLASS (parent_class)->connect (service, ex);
-
-#ifdef HAVE_KRB4
-	/* This is veeery nasty. When we set the port, we're changing the
-	 * hash value of our URL. service_cache_remove() gets called when
-	 * we're done checking the mail, but the hash table lookup fails
-	 * because the url port has changed. Then, a finalized instance of
-	 * the CamelService is stuck in the hash table, and the next time
-	 * we try to look up the service, with a URL of port 0, we look
-	 * up the freed service and a segfault results.
-	 */
-
-	if (kpop && set_port)
-		service->url->port = 0;
-#endif
-
-	if (res == FALSE)
-		return FALSE;
-
-	/*FIXME integrate these functions */
 	if (!connect_to_server (service, ex))
 		return FALSE;
 
@@ -503,6 +454,7 @@ pop3_connect (CamelService *service, CamelException *ex)
 
 		tryagain = pop3_try_authenticate (service, kpop, errbuf, ex);
 		g_free (errbuf);
+		errbuf = NULL;
 	} while (tryagain);
 
 	if (camel_exception_is_set (ex)) {
@@ -531,26 +483,26 @@ static CamelFolder *
 get_folder (CamelStore *store, const char *folder_name,
 	    guint32 flags, CamelException *ex)
 {
-	return camel_pop3_folder_new (store, ex);
-}
-
-static char *
-get_folder_name (CamelStore *store, const char *folder_name,
-		 CamelException *ex)
-{
-	if (!g_strcasecmp (folder_name, "inbox"))
-		return g_strdup ("inbox");
-	else {
+	if (g_strcasecmp (folder_name, "inbox") != 0) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_FOLDER_INVALID,
 				      _("No such folder `%s'."), folder_name);
 		return NULL;
 	}
+	return camel_pop3_folder_new (store, ex);
 }
 
-static char *
-get_root_folder_name (CamelStore *store, CamelException *ex)
+static void
+init_trash (CamelStore *store)
 {
-	return g_strdup ("inbox");
+	/* no-op */
+	;
+}
+
+static CamelFolder *
+get_trash (CamelStore *store, CamelException *ex)
+{
+	/* no-op */
+	return NULL;
 }
 
 
@@ -572,7 +524,8 @@ get_root_folder_name (CamelStore *store, CamelException *ex)
  * Return value: one of CAMEL_POP3_OK (command executed successfully),
  * CAMEL_POP3_ERR (command encounted an error), or CAMEL_POP3_FAIL
  * (a protocol-level error occurred, and Camel is uncertain of the
- * result of the command.)
+ * result of the command.) @ex will be set if the return value is
+ * CAMEL_POP3_FAIL, but *NOT* if it is CAMEL_POP3_ERR.
  **/
 int
 camel_pop3_command (CamelPop3Store *store, char **ret, CamelException *ex, char *fmt, ...)
@@ -632,6 +585,7 @@ pop3_get_response (CamelPop3Store *store, char **ret, CamelException *ex)
  * camel_pop3_command_get_additional_data: get "additional data" from
  * a POP3 command.
  * @store: the POP3 store
+ * @total: Total bytes expected (for progress reporting), use 0 for 'unknown'.
  *
  * This command gets the additional data returned by "multi-line" POP
  * commands, such as LIST, RETR, TOP, and UIDL. This command _must_
@@ -643,11 +597,12 @@ pop3_get_response (CamelPop3Store *store, char **ret, CamelException *ex)
  * Return value: the data, which the caller must free.
  **/
 char *
-camel_pop3_command_get_additional_data (CamelPop3Store *store, CamelException *ex)
+camel_pop3_command_get_additional_data (CamelPop3Store *store, int total, CamelException *ex)
 {
 	GPtrArray *data;
 	char *buf, *p;
 	int i, len = 0, status = CAMEL_POP3_OK;
+	int pc = 0;
 
 	data = g_ptr_array_new ();
 	while (1) {
@@ -661,6 +616,13 @@ camel_pop3_command_get_additional_data (CamelPop3Store *store, CamelException *e
 
 		g_ptr_array_add (data, buf);
 		len += strlen (buf) + 1;
+
+		if (total) {
+			pc = (len+1) * 100 / total;
+			camel_operation_progress(NULL, pc);
+		} else {
+			camel_operation_progress_count(NULL, len);
+		}
 	}
 	
 	if (buf)
