@@ -53,11 +53,11 @@ typedef struct {
 
 	CamelStream *istream, *ostream;
 
-	gboolean is_esmtp;
+	gboolean use_ssl, is_esmtp;
 
 	struct sockaddr_in localaddr;
 
-	GList *esmtp_supported_authtypes;
+	GHashTable *authtypes;
 	
 } CamelSmtpTransport;
 
