@@ -1007,7 +1007,9 @@ icon_new_from_pixbuf (EIconList *eil, GdkPixbuf *im,
 	gnome_canvas_item_set (GNOME_CANVAS_ITEM (icon->text),
 			       "use_broken_event_handling", FALSE,
 			       NULL);
-
+	
+	/* FIXME: should this use a selectable font?? */
+#warning "Ettore: should we allow this font to be selectable??"
 	gnome_icon_text_item_configure (icon->text,
 					0, 0, priv->icon_width, 
 					"-adobe-helvetica-medium-r-normal-*-*-120-*-*-p-*-iso8859-1",
