@@ -20,7 +20,6 @@
 
 #include <config.h>
 #include <gnome.h>
-#include <gnome-xml/xmlmemory.h>
 
 #include "filter-option.h"
 #include "filter-part.h"
@@ -193,7 +192,7 @@ xml_create (FilterElement *fe, xmlNodePtr node)
 				work = work->next;
 			}
 			d(printf ("creating new option:\n title %s\n value %s\n code %s\n",
-				  op->title, op->value, op->code));
+				  op->title, op->value, op->code ? op->code : "none"));
 			fo->options = g_list_append (fo->options, op);
 			if (fo->current == NULL)
 				fo->current = op;
@@ -228,6 +227,7 @@ xml_decode (FilterElement *fe, xmlNodePtr node)
 	char *value;
 	
 	d(printf ("Decoding option from xml\n"));
+	xmlFree (fe->name);
 	fe->name = xmlGetProp (node, "name");
 	value = xmlGetProp (node, "value");
 	if (value) {
@@ -296,7 +296,7 @@ build_code (FilterElement *fe, GString *out, struct _FilterPart *ff)
 	
 	d(printf ("building option code %p, current = %p\n", fo, fo->current));
 	
-	if (fo->current) {
+	if (fo->current && fo->current->code) {
 		filter_part_expand_code (ff, fo->current->code, out);
 	}
 }
@@ -328,7 +328,10 @@ clone (FilterElement *fe)
 		d(printf ("  option %s\n", op->title));
 		fn->title = g_strdup (op->title);
 		fn->value = g_strdup (op->value);
-		fn->code = g_strdup (op->code);
+		if (op->code)
+			fn->code = g_strdup (op->code);
+		else
+			fn->code = NULL;
 		new->options = g_list_append (new->options, fn);
 		l = g_list_next (l);
 		
