@@ -21,7 +21,6 @@
 #include <config.h>
 #include <gnome.h>
 
-#include <gnome-xml/xmlmemory.h>
 #include <gal/widgets/e-unicode.h>
 
 #include "filter-part.h"
@@ -414,7 +413,7 @@ filter_part_expand_code (FilterPart *ff, const char *source, GString *out)
 	FilterElement *fe;
 	
 	start = source;
-	while ( (newstart = strstr (start, "${"))
+	while (start && (newstart = strstr (start, "${"))
 		&& (end = strstr (newstart+2, "}")) ) {
 		len = end - newstart - 2;
 		if (len + 1 > namelen) {
