@@ -21,7 +21,6 @@
 
 #include <config.h>
 #include <gnome.h>
-#include <gnome-xml/xmlmemory.h>
 
 #include <gal/widgets/e-unicode.h>
 
@@ -30,6 +29,7 @@
 
 #define d(x)
 
+static int validate(FilterRule *);
 static xmlNodePtr xml_encode (FilterRule *);
 static int xml_decode (FilterRule *, xmlNodePtr, RuleContext *);
 static void build_code (FilterRule *, GString * out);
@@ -86,6 +86,7 @@ filter_rule_class_init (FilterRuleClass * class)
 	object_class->finalize = filter_rule_finalise;
 	
 	/* override methods */
+	class->validate = validate;
 	class->xml_encode = xml_encode;
 	class->xml_decode = xml_decode;
 	class->build_code = build_code;
@@ -138,9 +139,29 @@ filter_rule_new ()
 	return o;
 }
 
+FilterRule *
+filter_rule_clone(FilterRule *base, RuleContext *f)
+{
+	xmlNodePtr xml;
+	FilterRule *rule;
+
+	g_assert(IS_FILTER_RULE(base));
+	g_assert(IS_RULE_CONTEXT(f));
+
+	/* TODO: do this more directly/efficiently */
+	xml = filter_rule_xml_encode(base);
+	rule = gtk_type_new(((GtkObject *)base)->klass->type);
+	filter_rule_xml_decode(rule, xml, f);
+	xmlFreeNodeList(xml);
+
+	return rule;
+}
+
 void
 filter_rule_set_name (FilterRule *fr, const char *name)
 {
+	g_assert(IS_FILTER_RULE(fr));
+
 	g_free (fr->name);
 	fr->name = g_strdup (name);
 }
@@ -148,13 +169,41 @@ filter_rule_set_name (FilterRule *fr, const char *name)
 void
 filter_rule_set_source (FilterRule *fr, const char *source)
 {
+	g_assert(IS_FILTER_RULE(fr));
+
 	g_free (fr->source);
 	fr->source = g_strdup (source);
 }
 
+int
+filter_rule_validate (FilterRule *fr)
+{
+	g_assert(IS_FILTER_RULE(fr));
+
+	return ((FilterRuleClass *) ((GtkObject *) fr)->klass)->validate(fr);
+}
+
+static int
+validate (FilterRule *fr)
+{
+	int valid = TRUE;
+	GList *parts;
+
+	/* validate rule parts */
+	parts = fr->parts;
+	while (parts && valid) {
+		valid = filter_part_validate((FilterPart *)parts->data);
+		parts = parts->next;
+	}
+
+	return valid;
+}
+
 xmlNodePtr
 filter_rule_xml_encode (FilterRule *fr)
 {
+	g_assert(IS_FILTER_RULE(fr));
+
 	return ((FilterRuleClass *) ((GtkObject *) fr)->klass)->xml_encode(fr);
 }
 
@@ -173,7 +222,7 @@ xml_encode (FilterRule *fr)
 		xmlSetProp (node, "grouping", "any");
 		break;
 	}
-	
+
 	if (fr->source) {
 		xmlSetProp (node, "source", fr->source);
 	} else {
@@ -182,12 +231,13 @@ xml_encode (FilterRule *fr)
 	}
 	
 	if (fr->name) {
-		gchar *encstr;
+		char *encstr;
+
 		work = xmlNewNode (NULL, "title");
-		encstr = e_utf8_xml1_encode (fr->name);
-		xmlNodeSetContent (work, encstr);
-		g_free (encstr);
-		xmlAddChild (node, work);
+		encstr = e_utf8_xml1_encode(fr->name);
+		xmlNodeSetContent(work, encstr);
+		g_free(encstr);
+		xmlAddChild(node, work);
 	}
 	
 	set = xmlNewNode (NULL, "partset");
@@ -232,6 +282,10 @@ load_set (xmlNodePtr node, FilterRule *fr, RuleContext *f)
 int
 filter_rule_xml_decode (FilterRule *fr, xmlNodePtr node, RuleContext *f)
 {
+	g_assert(IS_FILTER_RULE(fr));
+	g_assert(IS_RULE_CONTEXT(f));
+	g_assert(node != NULL);
+
 	return ((FilterRuleClass *) ((GtkObject *) fr)->klass)->xml_decode(fr, node, f);
 }
 
@@ -254,9 +308,11 @@ xml_decode (FilterRule *fr, xmlNodePtr node, RuleContext *f)
 		fr->grouping = FILTER_GROUP_ALL;
 	xmlFree (grouping);
 	
+	g_free (fr->source);
 	source = xmlGetProp (node, "source");
 	if (source) {
-		fr->source = source;
+		fr->source = g_strdup (source);
+		xmlFree (source);
 	} else {
 		/* default filter type */
 		fr->source = g_strdup ("incoming");
@@ -284,12 +340,18 @@ xml_decode (FilterRule *fr, xmlNodePtr node, RuleContext *f)
 void
 filter_rule_add_part (FilterRule *fr, FilterPart *fp)
 {
+	g_assert(IS_FILTER_RULE(fr));
+	g_assert(IS_FILTER_PART(fp));
+
 	fr->parts = g_list_append (fr->parts, fp);
 }
 
 void
 filter_rule_remove_part (FilterRule *fr, FilterPart *fp)
 {
+	g_assert(IS_FILTER_RULE(fr));
+	g_assert(IS_FILTER_PART(fp));
+
 	fr->parts = g_list_remove (fr->parts, fp);
 }
 
@@ -297,6 +359,10 @@ void
 filter_rule_replace_part (FilterRule *fr, FilterPart *fp, FilterPart *new)
 {
 	GList *l;
+
+	g_assert(IS_FILTER_RULE(fr));
+	g_assert(IS_FILTER_PART(fp));
+	g_assert(IS_FILTER_PART(new));
 	
 	l = g_list_find (fr->parts, fp);
 	if (l) {
@@ -309,6 +375,9 @@ filter_rule_replace_part (FilterRule *fr, FilterPart *fp, FilterPart *new)
 void
 filter_rule_build_code (FilterRule *fr, GString *out)
 {
+	g_assert(IS_FILTER_RULE(fr));
+	g_assert(out != NULL);
+
 	return ((FilterRuleClass *) ((GtkObject *) fr)->klass)->build_code(fr, out);
 }
 
@@ -676,3 +745,10 @@ filter_rule_find_list (GList * l, const char *name, const char *source)
 	
 	return NULL;
 }
+
+#ifdef FOR_TRANSLATIONS_ONLY
+
+static char *list[] = {
+  N_("incoming"), N_("outgoing")
+};
+#endif
