@@ -3,7 +3,7 @@
   FILE: icalerror.c
   CREATOR: eric 16 May 1999
   
-  $Id: icalerror.c,v 1.8 2000/12/12 00:27:39 federico Exp $
+  $Id: icalerror.c,v 1.10 2001/02/05 19:43:57 jpr Exp $
   $Locker:  $
     
 
@@ -53,15 +53,79 @@ void icalerror_clear_errno() {
     icalerrno = ICAL_NO_ERROR;
 }
 
-void icalerror_set_errno(icalerrorenum e) {
-
 #ifdef ICAL_ERRORS_ARE_FATAL
-    fprintf(stderr,"libical: icalerrno_set_error: %s",icalerror_strerror(e));
-    icalerror_crash_here();
+int icalerror_errors_are_fatal = 1;
+#else
+int icalerror_errors_are_fatal = 0;
 #endif
 
-    icalerror_stop_here();
+struct icalerror_state { 
+    icalerrorenum error;
+    icalerrorstate state; 
+};
+
+struct icalerror_state error_state_map[] = 
+{ 
+    { ICAL_BADARG_ERROR,ICAL_ERROR_DEFAULT},
+    { ICAL_NEWFAILED_ERROR,ICAL_ERROR_DEFAULT},
+    { ICAL_MALFORMEDDATA_ERROR,ICAL_ERROR_DEFAULT}, 
+    { ICAL_PARSE_ERROR,ICAL_ERROR_DEFAULT},
+    { ICAL_INTERNAL_ERROR,ICAL_ERROR_DEFAULT}, 
+    { ICAL_FILE_ERROR,ICAL_ERROR_DEFAULT},
+    { ICAL_ALLOCATION_ERROR,ICAL_ERROR_DEFAULT},
+    { ICAL_USAGE_ERROR,ICAL_ERROR_DEFAULT},
+    { ICAL_MULTIPLEINCLUSION_ERROR,ICAL_ERROR_DEFAULT},
+    { ICAL_TIMEDOUT_ERROR,ICAL_ERROR_DEFAULT},
+    { ICAL_UNKNOWN_ERROR,ICAL_ERROR_DEFAULT},
+    { ICAL_NO_ERROR,ICAL_ERROR_DEFAULT}
+
+};
+
+void icalerror_set_error_state( icalerrorenum error, 
+				icalerrorstate state)
+{
+    int i;
+
+    for(i = ICAL_BADARG_ERROR; error_state_map[i].error!= ICAL_NO_ERROR;i++){
+	if(error_state_map[i].error == error){
+	    error_state_map[i].state = state; 	
+	}
+    }
+}
+
+icalerrorstate icalerror_get_error_state( icalerrorenum error)
+{
+    int i;
+
+    for(i = ICAL_BADARG_ERROR; error_state_map[i].error!= ICAL_NO_ERROR;i++){
+	if(error_state_map[i].error == error){
+	    return error_state_map[i].state; 	
+	}
+    }
+
+    return ICAL_ERROR_UNKNOWN;	
+}
+
+
+void icalerror_set_errno(icalerrorenum e) {
+
+    icalerrorstate es;
+
     icalerrno = e;
+    es =  icalerror_get_error_state(e);
+
+    icalerror_stop_here();
+
+    if( (es == ICAL_ERROR_FATAL) ||
+	(es == ICAL_ERROR_DEFAULT && icalerror_errors_are_fatal == 1)){
+	
+	fprintf(stderr,"libical: icalerrno_set_error: %s\n",icalerror_strerror(e));
+#ifdef NDEBUG
+	icalerror_crash_here();
+#else
+	assert(0);
+#endif 
+    }
 }
 
 
@@ -75,7 +139,7 @@ static struct icalerror_string_map string_map[] =
     {ICAL_BADARG_ERROR,"Bad argument to function"},
     {ICAL_NEWFAILED_ERROR,"Failed to create a new object via a *_new() routine"},
     {ICAL_MALFORMEDDATA_ERROR,"An input string was not correctly formed or a component has missing or extra properties"},
-    {ICAL_PARSE_ERROR,"Failed to parse a part of an iCal componet"},
+    {ICAL_PARSE_ERROR,"Failed to parse a part of an iCal component"},
     {ICAL_INTERNAL_ERROR,"Random internal error. This indicates an error in the library code, not an error in use"}, 
     {ICAL_FILE_ERROR,"An operation on a file failed. Check errno for more detail."},
     {ICAL_ALLOCATION_ERROR,"Failed to allocate memory"},
