@@ -3,7 +3,7 @@
   FILE: icalrecur.c
   CREATOR: eric 16 May 2000
   
-  $Id: icalrecur.c,v 1.4 2001/02/05 19:43:57 jpr Exp $
+  $Id: icalrecur.c,v 1.5 2001/02/06 19:36:59 clahey Exp $
   $Locker:  $
     
 
@@ -277,9 +277,12 @@ void icalrecur_add_bydayrules(struct icalrecur_parser *parser, const char* vals)
     icalrecurrencetype_weekday wd;
     short *array = parser->rt.by_day;
     char* end;
+    char* vals_copy;
 
-    end = (char*)vals+strlen(vals);
-    n = vals;
+    vals_copy = strdup(vals);
+
+    end = (char*)vals_copy+strlen(vals_copy);
+    n = vals_copy;
 
     while(n != 0){
 	
@@ -323,6 +326,8 @@ void icalrecur_add_bydayrules(struct icalrecur_parser *parser, const char* vals)
 
     }
 
+    free(vals_copy);
+
 }
 
 
