@@ -25,7 +25,7 @@
 #include <config.h>
 #endif
 
-#include <bonobo.h>
+#include <bonobo/bonobo-generic-factory.h>
 
 #include "camel.h"
 
@@ -36,6 +36,7 @@
 #include "evolution-shell-component.h"
 #include "folder-browser.h"
 #include "mail.h"		/* YUCK FIXME */
+#include "mail-config.h"
 #include "mail-tools.h"
 #include "mail-ops.h"
 #include "mail-local.h"
@@ -49,10 +50,14 @@
 #include "component-factory.h"
 
 #include "mail-summary.h"
+#include "mail-send-recv.h"
 
+char *default_drafts_folder_uri;
 CamelFolder *drafts_folder = NULL;
+char *default_sent_folder_uri;
+CamelFolder *sent_folder = NULL;
+char *default_outbox_folder_uri;
 CamelFolder *outbox_folder = NULL;
-CamelFolder *sent_folder = NULL;     /* this one should be configurable? */
 char *evolution_dir;
 
 #define COMPONENT_FACTORY_ID "OAFIID:GNOME_Evolution_Mail_ShellComponentFactory"
@@ -100,6 +105,8 @@ create_view (EvolutionShellComponent *shell_component,
 		camel_object_unref (CAMEL_OBJECT (store));
 
 		control = folder_browser_factory_new_control ("", corba_shell);
+	} else if (g_strcasecmp (folder_type, "vtrash") == 0) {
+		control = folder_browser_factory_new_control ("vtrash:file:/", corba_shell);
 	} else
 		return EVOLUTION_SHELL_COMPONENT_UNSUPPORTEDTYPE;
 
@@ -111,7 +118,7 @@ create_view (EvolutionShellComponent *shell_component,
 }
 
 static void
-do_create_folder(char *uri, CamelFolder *folder, void *data)
+do_create_folder (char *uri, CamelFolder *folder, void *data)
 {
 	GNOME_Evolution_ShellComponentListener listener = data;
 	CORBA_Environment ev;
@@ -139,31 +146,47 @@ create_folder (EvolutionShellComponent *shell_component,
 	CORBA_Environment ev;
 
 	CORBA_exception_init(&ev);
-	if (!strcmp(type, "mail")) {
+	if (!strcmp (type, "mail")) {
 		uri = g_strdup_printf ("mbox://%s", physical_uri);
-		mail_create_folder(uri, do_create_folder, CORBA_Object_duplicate(listener, &ev));
+		mail_create_folder (uri, do_create_folder, CORBA_Object_duplicate (listener, &ev));
+		GNOME_Evolution_ShellComponentListener_notifyResult (listener,
+								     GNOME_Evolution_ShellComponentListener_OK, &ev);
+								     
 	} else {
-		GNOME_Evolution_ShellComponentListener_notifyResult(listener, GNOME_Evolution_ShellComponentListener_UNSUPPORTED_TYPE, &ev);
+		GNOME_Evolution_ShellComponentListener_notifyResult (
+			listener, GNOME_Evolution_ShellComponentListener_UNSUPPORTED_TYPE, &ev);
 	}
 	CORBA_exception_free(&ev);
 }
 
 static struct {
-	char *name;
+	char *name, **uri;
 	CamelFolder **folder;
 } standard_folders[] = {
-	{ "Drafts", &drafts_folder },
-	{ "Outbox", &outbox_folder },
-	{ "Sent", &sent_folder },
+	{ "Drafts", &default_drafts_folder_uri, &drafts_folder },
+	{ "Outbox", &default_outbox_folder_uri, &outbox_folder },
+	{ "Sent", &default_sent_folder_uri, &sent_folder },
 };
 
-static void got_folder(char *uri, CamelFolder *folder, void *data)
+static void
+unref_standard_folders (void)
+{
+	int i;
+	
+	for (i = 0; i < sizeof (standard_folders) / sizeof (standard_folders[0]); i++) {
+		if (standard_folders[i].folder)
+			camel_object_unref (CAMEL_OBJECT (*standard_folders[i].folder));
+	}
+}
+
+static void
+got_folder (char *uri, CamelFolder *folder, void *data)
 {
 	CamelFolder **fp = data;
-
+	
 	if (folder) {
 		*fp = folder;
-		camel_object_ref((CamelObject *)folder);
+		camel_object_ref (CAMEL_OBJECT (folder));
 	}
 }
 
@@ -184,12 +207,13 @@ owner_set_cb (EvolutionShellComponent *shell_component,
 
 	evolution_dir = g_strdup (evolution_homedir);
 	mail_session_init ();
+	
 	mail_config_init ();
 	
 	openpgp_init (mail_config_get_pgp_path (), mail_config_get_pgp_type ());
 	
 	storages_hash = g_hash_table_new (NULL, NULL);
-
+	
 	vfolder_create_storage (shell_component);
 
 	corba_shell = bonobo_object_corba_objref (BONOBO_OBJECT (shell_client));
@@ -203,14 +227,16 @@ owner_set_cb (EvolutionShellComponent *shell_component,
 #endif
 
 	mail_local_storage_startup (shell_client, evolution_dir);
-
+	mail_importer_init (shell_client);
+	
 	for (i = 0; i < sizeof (standard_folders) / sizeof (standard_folders[0]); i++) {
-		char *uri = g_strdup_printf ("file://%s/local/%s", evolution_dir, standard_folders[i].name);
-		mail_msg_wait (mail_get_folder (uri, got_folder, standard_folders[i].folder));
-		g_free (uri);
+		*standard_folders[i].uri = g_strdup_printf ("file://%s/local/%s", evolution_dir, standard_folders[i].name);
+		mail_msg_wait (mail_get_folder (*standard_folders[i].uri, got_folder, standard_folders[i].folder));
 	}
-
+	
 	mail_session_enable_interaction (TRUE);
+	
+	mail_autoreceive_setup ();
 }
 
 static void
@@ -232,7 +258,6 @@ idle_quit (gpointer user_data)
 	g_hash_table_foreach (storages_hash, free_storage, NULL);
 	g_hash_table_destroy (storages_hash);
 
-	mail_operations_terminate ();
 	gtk_main_quit ();
 
 	return FALSE;
@@ -241,12 +266,23 @@ idle_quit (gpointer user_data)
 static void
 owner_unset_cb (EvolutionShellComponent *shell_component, gpointer user_data)
 {
+	unref_standard_folders ();
+	
 	g_idle_add_full (G_PRIORITY_LOW, idle_quit, NULL, NULL);
 }
 
+static void
+debug_cb (EvolutionShellComponent *shell_component, gpointer user_data)
+{
+	extern gboolean camel_verbose_debug;
+
+	camel_verbose_debug = 1;
+}
+
 static const EvolutionShellComponentFolderType folder_types[] = {
 	{ "mail", "evolution-inbox.png" },
 	{ "mailstorage", "evolution-inbox.png" },
+	{ "vtrash", "evolution-trash.png" },
 	{ NULL, NULL }
 };
 
@@ -260,13 +296,16 @@ component_fn (BonoboGenericFactory *factory, void *closure)
 							 create_folder,
 							 NULL, /* remove_folder_fn */
 							 NULL, /* copy_folder_fn */
-							 NULL, /* populate_folder_context_menu */
+							 NULL, /* populate_folder_context_menu_fn */
+							 NULL, /* get_dnd_selection_fn */
 							 NULL);
 
 	gtk_signal_connect (GTK_OBJECT (shell_component), "owner_set",
 			    GTK_SIGNAL_FUNC (owner_set_cb), NULL);
 	gtk_signal_connect (GTK_OBJECT (shell_component), "owner_unset",
 			    GTK_SIGNAL_FUNC (owner_unset_cb), NULL);
+	gtk_signal_connect (GTK_OBJECT (shell_component), "debug",
+			    GTK_SIGNAL_FUNC (debug_cb), NULL);
 	gtk_signal_connect (GTK_OBJECT (shell_component), "destroy",
 			    GTK_SIGNAL_FUNC (owner_unset_cb), NULL);
 
@@ -287,7 +326,6 @@ component_factory_init (void)
 							component_fn, NULL);
 	summary_factory = bonobo_generic_factory_new (SUMMARY_FACTORY_ID,
 						      summary_fn, NULL);
-	mail_importer_init ();
 
 	if (component_factory == NULL || summary_factory == NULL) {
 		e_notice (NULL, GNOME_MESSAGE_BOX_ERROR,
@@ -296,6 +334,90 @@ component_factory_init (void)
 	}
 }
 
+static void
+notify_bonobo_listener (const Bonobo_Listener listener,
+			EvolutionStorageResult result,
+			const char *physical_path)
+{
+	CORBA_any any;
+	GNOME_Evolution_Storage_FolderResult folder_result;
+	CORBA_Environment ev;
+
+	folder_result.result = result;
+	folder_result.path = CORBA_string_dup (physical_path ? physical_path : "");
+	any._type = TC_GNOME_Evolution_Storage_FolderResult;
+	any._value = &folder_result;
+
+	CORBA_exception_init (&ev);
+	Bonobo_Listener_event (listener, "evolution-shell:folder_created", 
+			       &any, &ev);
+	CORBA_exception_free (&ev);
+}
+
+static void
+storage_create_folder (EvolutionStorage *storage, 
+		       const Bonobo_Listener bonobo_listener,
+		       const char *path,
+		       const char *type, 
+		       const char *description,
+		       const char *parent_physical_uri, 
+		       gpointer user_data)
+{
+	CamelStore *store = user_data;
+	char *name;
+	CamelURL *url;
+	CamelException ex;
+	CamelFolderInfo *fi;
+
+	if (strcmp (type, "mail") != 0) {
+		notify_bonobo_listener (bonobo_listener, 
+					EVOLUTION_STORAGE_ERROR_UNSUPPORTED_TYPE,
+					NULL);
+		return; /* EVOLUTION_STORAGE_ERROR_UNSUPPORTED_TYPE;*/
+	}
+
+	name = strrchr (path, '/');
+	if (!name++) {
+		notify_bonobo_listener (bonobo_listener,
+					EVOLUTION_STORAGE_ERROR_INVALID_URI,
+					NULL);
+		return; /* EVOLUTION_STORAGE_ERROR_INVALID_URI; */
+	}
+
+	camel_exception_init (&ex);
+	if (*parent_physical_uri) {
+		url = camel_url_new (parent_physical_uri, NULL);
+		if (!url) {
+			notify_bonobo_listener (bonobo_listener,
+						EVOLUTION_STORAGE_ERROR_INVALID_URI,
+						NULL);
+			return; /* EVOLUTION_STORAGE_ERROR_INVALID_URI; */
+		}
+		fi = camel_store_create_folder (store, url->path + 1, name, &ex);
+		camel_url_free (url);
+	} else
+		fi = camel_store_create_folder (store, NULL, name, &ex);
+
+	if (camel_exception_is_set (&ex)) {
+		/* FIXME: do better than this */
+		camel_exception_clear (&ex);
+		notify_bonobo_listener (bonobo_listener,
+					EVOLUTION_STORAGE_ERROR_INVALID_URI,
+					NULL);
+		return; /* EVOLUTION_STORAGE_ERROR_INVALID_URI; */
+	}
+	
+	if (camel_store_supports_subscriptions (store))
+		camel_store_subscribe_folder (store, fi->full_name, NULL);
+	
+	folder_created (store, fi);
+	
+	camel_store_free_folder_info (store, fi);
+	
+	notify_bonobo_listener (bonobo_listener, EVOLUTION_STORAGE_OK, NULL);
+	return; /* EVOLUTION_STORAGE_OK; */
+}
+
 static void
 add_storage (const char *name, const char *uri, CamelService *store,
 	     GNOME_Evolution_Shell corba_shell, CamelException *ex)
@@ -304,6 +426,9 @@ add_storage (const char *name, const char *uri, CamelService *store,
 	EvolutionStorageResult res;
 	
 	storage = evolution_storage_new (name, uri, "mailstorage");
+	gtk_signal_connect (GTK_OBJECT (storage), "create_folder",
+			    GTK_SIGNAL_FUNC (storage_create_folder),
+			    store);
 	
 	res = evolution_storage_register_on_shell (storage, corba_shell);
 	
@@ -342,7 +467,8 @@ mail_load_storages (GNOME_Evolution_Shell shell, const GSList *sources, gboolean
 		const MailConfigService *service = NULL;
 		CamelService *store;
 		CamelProvider *prov;
-		
+		char *name;
+
 		if (is_account_data) {
 			account = iter->data;
 			service = account->source;
@@ -353,7 +479,26 @@ mail_load_storages (GNOME_Evolution_Shell shell, const GSList *sources, gboolean
 		if (service->url == NULL || service->url[0] == '\0')
 			continue;
 
-		store = camel_session_get_service (session, service->url, 
+		prov = camel_session_get_provider (session, service->url, &ex);
+		if (prov == NULL) {
+			/* FIXME: real error dialog */
+			g_warning ("couldn't get service %s: %s\n", service->url,
+				   camel_exception_get_description (&ex));
+			camel_exception_clear (&ex);
+			continue;
+		}
+
+		/* FIXME: this case is ambiguous for things like the
+		 * mbox provider, which can really be a spool
+		 * (/var/spool/mail/user) or a storage (~/mail/, eg).
+		 * That issue can't be resolved on the provider level
+		 * -- it's a per-URL problem.
+		 */
+		if (!(prov->flags & CAMEL_PROVIDER_IS_STORAGE) ||
+		    !(prov->flags & CAMEL_PROVIDER_IS_REMOTE))
+			continue;
+
+		store = camel_session_get_service (session, service->url,
 						   CAMEL_PROVIDER_STORE, &ex);
 		if (store == NULL) {
 			/* FIXME: real error dialog */
@@ -363,31 +508,19 @@ mail_load_storages (GNOME_Evolution_Shell shell, const GSList *sources, gboolean
 			continue;
 		}
 
-		prov = camel_service_get_provider (store);
+		if (is_account_data)
+			name = g_strdup (account->name);
+		else
+			name = camel_service_get_name (store, TRUE);
 
-		/* FIXME: this case is ambiguous for things like the
-		 * mbox provider, which can really be a spool
-		 * (/var/spool/mail/user) or a storage (~/mail/, eg).
-		 * That issue can't be resolved on the provider level
-		 * -- it's a per-URL problem.
-		 */
-		if (prov->flags & CAMEL_PROVIDER_IS_STORAGE && prov->flags & CAMEL_PROVIDER_IS_REMOTE) {
-			char *name;
-			
-			if (is_account_data) {
-				name = g_strdup (account->name);
-			} else {
-				name = camel_service_get_name (store, TRUE);
-			}
-			add_storage (name, service->url, store, shell, &ex);
-			g_free (name);
-			
-			if (camel_exception_is_set (&ex)) {
-				/* FIXME: real error dialog */
-				g_warning ("Cannot load storage: %s",
-					   camel_exception_get_description (&ex));
-				camel_exception_clear (&ex);
-			}
+		add_storage (name, service->url, store, shell, &ex);
+		g_free (name);
+
+		if (camel_exception_is_set (&ex)) {
+			/* FIXME: real error dialog */
+			g_warning ("Cannot load storage: %s",
+				   camel_exception_get_description (&ex));
+			camel_exception_clear (&ex);
 		}
 		
 		camel_object_unref (CAMEL_OBJECT (store));
