@@ -9,7 +9,7 @@
 #include "camel/camel-stream.h"
 #include <bonobo/bonobo-property-bag.h>
 #include <bonobo/bonobo-ui-component.h>
-#include <widgets/misc/e-search-bar.h>
+#include <widgets/misc/e-filter-bar.h>
 #include "filter/filter-rule.h"
 #include "filter/filter-context.h" /*eek*/
 #include "message-list.h"
@@ -54,7 +54,7 @@ struct  _FolderBrowser {
 	MailDisplay *mail_display;
 	GtkWidget   *vpaned;
 	
-	ESearchBar  *search;
+	EFilterBar  *search;
 	FilterRule  *search_full; /* if we have a full search active */
 
 	gboolean     preview_shown;
@@ -85,6 +85,7 @@ void       folder_browser_clear_search         (FolderBrowser         *fb);
 void vfolder_subject   (GtkWidget *w, FolderBrowser *fb);
 void vfolder_sender    (GtkWidget *w, FolderBrowser *fb);
 void vfolder_recipient (GtkWidget *w, FolderBrowser *fb);
+void vfolder_mlist     (GtkWidget *w, FolderBrowser *fb);
 
 void filter_subject    (GtkWidget *w, FolderBrowser *fb);
 void filter_sender     (GtkWidget *w, FolderBrowser *fb);
@@ -110,4 +111,9 @@ void folder_browser_toggle_view_source (BonoboUIComponent           *component,
 					const char                  *state,
 					gpointer                     user_data);
 
+void folder_browser_toggle_hide_deleted (BonoboUIComponent           *component,
+					 const char                  *path,
+					 Bonobo_UIComponent_EventType type,
+					 const char                  *state,
+					 gpointer                     user_data);
 #endif /* _FOLDER_BROWSER_H_ */
