@@ -33,16 +33,18 @@ extern "C" {
 #endif /* __cplusplus }*/
 
 gboolean mail_crypto_is_rfc2015_signed (CamelMimePart *part);
-
 gboolean mail_crypto_is_rfc2015_encrypted (CamelMimePart *part);
 
+gboolean mail_crypto_is_smime_v3_signed (CamelMimePart *mime_part);
+gboolean mail_crypto_is_pkcs7_mime (CamelMimePart *mime_part);
+
 void pgp_mime_part_sign (CamelMimePart **mime_part,
 			 const gchar *userid,
 			 PgpHashType hash,
 			 CamelException *ex);
 
-gboolean pgp_mime_part_verify (CamelMimePart *mime_part,
-			       CamelException *ex);
+PgpValidity *pgp_mime_part_verify (CamelMimePart *mime_part,
+				   CamelException *ex);
 
 void pgp_mime_part_encrypt (CamelMimePart **mime_part,
 			    const GPtrArray *recipients,
