@@ -36,13 +36,15 @@
 #include <gdk-pixbuf/gdk-pixbuf-loader.h>
 #include <gtkhtml/gtkhtml-embedded.h>
 #include <gtkhtml/htmlengine.h>	/* XXX */
+#include <gtkhtml/htmlobject.h>	/* XXX */
+#include <gtkhtml/htmlinterval.h> /* XXX */
+#include <gtkhtml/htmltext.h> /* XXX */
 
 #define PARENT_TYPE (gtk_vbox_get_type ())
 
 static GtkObjectClass *mail_display_parent_class;
 
 struct _PixbufLoader {
-	MailDisplay *md;
 	CamelDataWrapper *wrapper; /* The data */
 	CamelStream *mstream;
 	GdkPixbufLoader *loader; 
@@ -53,6 +55,7 @@ struct _PixbufLoader {
 	GtkWidget *pixmap;
 	guint32 destroy_id;
 };
+static GHashTable *thumbnail_cache = NULL;
 
 /*----------------------------------------------------------------------*
  *                        Callbacks
@@ -264,8 +267,27 @@ inline_cb (GtkWidget *widget, gpointer user_data)
 	mail_display_queue_redisplay (md);
 }
 
+static void
+button_press (GtkWidget *widget, CamelMimePart *part)
+{
+	MailDisplay *md;
+
+	md = gtk_object_get_data (GTK_OBJECT (widget), "MailDisplay");
+	if (md == NULL) {
+		g_warning ("No MailDisplay on button!");
+		return;
+	}
+
+	if (mail_part_is_inline (part))
+		camel_mime_part_set_disposition (part, "attachment");
+	else
+		camel_mime_part_set_disposition (part, "inline");
+
+	mail_display_queue_redisplay (md);
+}
+
 static gboolean
-pixmap_press (GtkWidget *ebox, GdkEventButton *event, EScrollFrame *user_data)
+pixmap_press (GtkWidget *widget, GdkEventButton *event, EScrollFrame *user_data)
 {
 	EPopupMenu menu[] = {
 		{ N_("Save to Disk..."), NULL,
@@ -280,14 +302,22 @@ pixmap_press (GtkWidget *ebox, GdkEventButton *event, EScrollFrame *user_data)
 	MailMimeHandler *handler;
 	int mask = 0;
 
+#ifdef USE_OLD_DISPLAY_STYLE
 	if (event->button != 3) {
 		gtk_propagate_event (GTK_WIDGET (user_data),
 				     (GdkEvent *)event);
 		return TRUE;
 	}
+#endif
 
-	part = gtk_object_get_data (GTK_OBJECT (ebox), "CamelMimePart");
-	handler = mail_lookup_handler (gtk_object_get_data (GTK_OBJECT (ebox),
+	if (event->button != 1 && event->button != 3) {
+		gtk_propagate_event (GTK_WIDGET (user_data),
+				     (GdkEvent *)event);
+		return TRUE;
+	}
+
+	part = gtk_object_get_data (GTK_OBJECT (widget), "CamelMimePart");
+	handler = mail_lookup_handler (gtk_object_get_data (GTK_OBJECT (widget),
 							    "mime_type"));
 
 	/* Save item */
@@ -332,7 +362,7 @@ pixmap_press (GtkWidget *ebox, GdkEventButton *event, EScrollFrame *user_data)
 		mask |= 2;
 	}
 
-	e_popup_menu_run (menu, (GdkEvent *)event, mask, 0, ebox);
+	e_popup_menu_run (menu, (GdkEvent *)event, mask, 0, widget);
 	g_free (menu[1].name);
 	g_free (menu[2].name);
 	return TRUE;
@@ -387,6 +417,18 @@ pixbuf_for_mime_type (const char *mime_type)
 	return pixbuf;
 }
 
+static gboolean
+pixbuf_uncache (gpointer key)
+{
+	GdkPixbuf *pixbuf;
+
+	pixbuf = g_hash_table_lookup (thumbnail_cache, key);
+	gdk_pixbuf_unref (pixbuf);
+	g_hash_table_remove (thumbnail_cache, key);
+	g_free (key);
+	return FALSE;
+}
+
 static gint
 pixbuf_gen_idle (struct _PixbufLoader *pbl)
 {
@@ -394,10 +436,11 @@ pixbuf_gen_idle (struct _PixbufLoader *pbl)
 	gboolean error = FALSE;
 	char tmp[4096];
 	int len, width, height, ratio;
+	gpointer orig_key;
 
 	/* Get the pixbuf from the cache */
-	mini = g_hash_table_lookup (pbl->md->thumbnail_cache, pbl->cid);
-	if (mini) {
+	if (g_hash_table_lookup_extended (thumbnail_cache, pbl->cid,
+					  &orig_key, (gpointer *)&mini)) {
 		width = gdk_pixbuf_get_width (mini);
 		height = gdk_pixbuf_get_height (mini);
 
@@ -405,6 +448,10 @@ pixbuf_gen_idle (struct _PixbufLoader *pbl)
 		        BONOBO_UI_TOOLBAR_ICON (pbl->pixmap), mini);
 		gtk_widget_set_usize (pbl->pixmap, width, height);
 		
+		/* Restart the cache-cleaning timer */
+		g_source_remove_by_user_data (orig_key);
+		g_timeout_add (5 * 60 * 1000, pixbuf_uncache, orig_key);
+
 		if (pbl->loader) {
 			gdk_pixbuf_loader_close (pbl->loader);
 			gtk_object_destroy (GTK_OBJECT (pbl->loader));
@@ -412,12 +459,14 @@ pixbuf_gen_idle (struct _PixbufLoader *pbl)
 		}
 		gtk_signal_disconnect (GTK_OBJECT (pbl->eb), pbl->destroy_id);
 		g_free (pbl->type);
+		g_free (pbl->cid);
 		g_free (pbl);
 
 		return FALSE;
 	}
 
 	/* Not in cache, so get a pixbuf from the wrapper */
+
 	if (!GTK_IS_WIDGET (pbl->pixmap)) {
 		/* Widget has died */
 		if (pbl->mstream)
@@ -429,6 +478,7 @@ pixbuf_gen_idle (struct _PixbufLoader *pbl)
 		}
 	
 		g_free (pbl->type);
+		g_free (pbl->cid);
 		g_free (pbl);
 		return FALSE;
 	}
@@ -474,11 +524,11 @@ pixbuf_gen_idle (struct _PixbufLoader *pbl)
 		gdk_pixbuf_unref (pixbuf);
 	bonobo_ui_toolbar_icon_set_pixbuf (
 		BONOBO_UI_TOOLBAR_ICON (pbl->pixmap), mini);
+  	gtk_widget_set_usize (pbl->pixmap, 24, 24);
 
 	/* Add the pixbuf to the cache */
-
-	g_hash_table_insert (pbl->md->thumbnail_cache, pbl->cid, mini);
-	gtk_widget_set_usize (pbl->pixmap, width, height);
+	g_hash_table_insert (thumbnail_cache, pbl->cid, mini);
+	g_timeout_add (5 * 60 * 1000, pixbuf_uncache, pbl->cid);
 
 	gtk_signal_disconnect (GTK_OBJECT (pbl->eb), pbl->destroy_id);
 	if (pbl->loader) {
@@ -508,6 +558,7 @@ embeddable_destroy_cb (GtkObject *embeddable,
 	}
 	
 	g_free (pbl->type);
+	g_free (pbl->cid);
 	g_free (pbl);
 };
 
@@ -582,6 +633,33 @@ get_embedded_for_component (const char *iid, MailDisplay *md)
 	return embedded;
 }
 
+static void
+handle_embedded_address_object (GtkHTMLEmbedded *eb)
+{
+	const gchar *name, *email;
+	GtkWidget *w;
+
+	w = bonobo_widget_new_control ("OAFIID:GNOME_Evolution_Addressbook_AddressWidget",
+				       CORBA_OBJECT_NIL);
+
+	name  = gtk_html_embedded_get_parameter (eb, "name");
+	email = gtk_html_embedded_get_parameter (eb, "email");
+
+	bonobo_widget_set_property (BONOBO_WIDGET (w),
+				    "name", name,
+				    "email", email,
+				    /* Hackish: this is the bg color defined for the HTML table
+				       in mail-format.c.  If you change it there, you'd better
+				       change it here as well. */
+				    "background_rgb", 0xeeeeee,
+				    NULL);
+
+	gtk_widget_show (w);
+	gtk_container_add (GTK_CONTAINER (eb), w);
+
+	gtk_html_embedded_set_descent (eb, 0);
+}
+
 static gboolean
 on_object_requested (GtkHTML *html, GtkHTMLEmbedded *eb, gpointer data)
 {
@@ -600,6 +678,12 @@ on_object_requested (GtkHTML *html, GtkHTMLEmbedded *eb, gpointer data)
 	char *cid;
 
 	cid = eb->classid;
+
+	if (!strcmp (cid, "address")) {
+		handle_embedded_address_object (eb);
+		return TRUE;
+	}
+
 	if (!strncmp (cid, "popup:", 6))
 		cid += 6;
 	if (strncmp (cid, "cid:", 4) != 0)
@@ -613,7 +697,12 @@ on_object_requested (GtkHTML *html, GtkHTMLEmbedded *eb, gpointer data)
 
 	if (cid != eb->classid) {
 		/* This is a part wrapper */
+#ifdef USE_OLD_DISPLAY_STYLE
 		GtkWidget *ebox;
+#else
+		GtkWidget *button, *mainbox, *hbox, *arrow, *popup;
+		MailMimeHandler *handler;
+#endif
 		struct _PixbufLoader *pbl;
 
 		pbl = g_new0 (struct _PixbufLoader, 1);
@@ -631,7 +720,6 @@ on_object_requested (GtkHTML *html, GtkHTMLEmbedded *eb, gpointer data)
 		pbl->cid = g_strdup (cid);
 		pbl->pixmap = bonobo_ui_toolbar_icon_new ();
 		pbl->eb = eb;
-		pbl->md = md;
 		pbl->destroy_id = gtk_signal_connect (GTK_OBJECT (eb),
 						      "destroy",
 						      embeddable_destroy_cb,
@@ -640,6 +728,7 @@ on_object_requested (GtkHTML *html, GtkHTMLEmbedded *eb, gpointer data)
 		g_idle_add_full (G_PRIORITY_LOW, (GSourceFunc)pixbuf_gen_idle, 
 				 pbl, NULL);
 
+#ifdef USE_OLD_DISPLAY_STYLE
 		ebox = gtk_event_box_new ();
 		gtk_widget_set_sensitive (GTK_WIDGET (ebox), TRUE);
 		gtk_widget_add_events (GTK_WIDGET (ebox),
@@ -657,6 +746,56 @@ on_object_requested (GtkHTML *html, GtkHTMLEmbedded *eb, gpointer data)
 		gtk_container_add (GTK_CONTAINER (ebox), pbl->pixmap);
 		gtk_widget_show_all (ebox);
 		gtk_container_add (GTK_CONTAINER (eb), ebox);
+#else
+		mainbox = gtk_hbox_new (FALSE, 0);
+
+		button = gtk_button_new ();
+		gtk_object_set_data (GTK_OBJECT (button), "MailDisplay", md);
+
+		gtk_signal_connect (GTK_OBJECT (button), "clicked",
+				    GTK_SIGNAL_FUNC (button_press), medium);
+
+		hbox = gtk_hbox_new (FALSE, 2);
+		gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
+
+		if (mail_part_is_inline (CAMEL_MIME_PART (medium))) {
+			arrow = gnome_stock_new_with_icon (GNOME_STOCK_PIXMAP_DOWN);
+		} else {
+			arrow = gnome_stock_new_with_icon (GNOME_STOCK_PIXMAP_FORWARD);
+		}
+		gtk_box_pack_start (GTK_BOX (hbox), arrow, TRUE, TRUE, 0);
+		gtk_box_pack_start (GTK_BOX (hbox), pbl->pixmap, TRUE, TRUE, 0);
+		gtk_container_add (GTK_CONTAINER (button), hbox);
+
+		popup = gtk_button_new ();
+		gtk_container_add (GTK_CONTAINER (popup),
+				   gtk_arrow_new (GTK_ARROW_DOWN,
+						  GTK_SHADOW_ETCHED_IN));
+
+		gtk_object_set_data (GTK_OBJECT (popup), "MailDisplay", md);
+		gtk_object_set_data (GTK_OBJECT (popup), "CamelMimePart",
+				     medium);
+		gtk_object_set_data_full (GTK_OBJECT (popup), "mime_type",
+					  g_strdup (eb->type),
+					  (GDestroyNotify)g_free);
+
+		gtk_signal_connect (GTK_OBJECT (popup), "button_press_event",
+				    GTK_SIGNAL_FUNC (pixmap_press), md->scroll);
+
+		gtk_box_pack_start (GTK_BOX (mainbox), button, TRUE, TRUE, 0);
+		gtk_box_pack_start (GTK_BOX (mainbox), popup, TRUE, TRUE, 0);
+		gtk_widget_show_all (mainbox);
+
+		handler = mail_lookup_handler (eb->type);
+		if (handler && handler->builtin) {
+			gtk_widget_set_sensitive (button, TRUE);
+		} else {
+			gtk_widget_set_sensitive (button, FALSE);
+		}
+
+		gtk_container_add (GTK_CONTAINER (eb), mainbox);
+#endif
+
 		return TRUE;
 	}
 
@@ -721,8 +860,10 @@ on_url_requested (GtkHTML *html, const char *url, GtkHTMLStream *handle,
 	g_return_if_fail (urls != NULL);
 
 	user_data = g_hash_table_lookup (urls, url);
-	if (user_data == NULL)
+	if (user_data == NULL) {
+		gtk_html_end (html, handle, GTK_HTML_STREAM_ERROR);
 		return;
+	}
 
 	if (strncmp (url, "cid:", 4) == 0) {
 		CamelMedium *medium = user_data;
@@ -746,6 +887,8 @@ on_url_requested (GtkHTML *html, const char *url, GtkHTMLStream *handle,
 		g_return_if_fail (ba != NULL);
 		gtk_html_write (html, handle, ba->data, ba->len);
 	}
+
+	gtk_html_end (html, handle, GTK_HTML_STREAM_OK);
 }
 
 void
@@ -773,10 +916,12 @@ mail_text_write (GtkHTML *html, GtkHTMLStream *stream,
 	buf = g_strdup_vprintf (format, ap);
 	va_end (ap);
 
-	htmltext = e_text_to_html (buf,
-				   E_TEXT_TO_HTML_CONVERT_URLS |
-				   E_TEXT_TO_HTML_CONVERT_NL |
-				   E_TEXT_TO_HTML_CONVERT_SPACES);
+	htmltext = e_text_to_html_full (buf,
+					E_TEXT_TO_HTML_CONVERT_URLS |
+					E_TEXT_TO_HTML_CONVERT_NL |
+					E_TEXT_TO_HTML_CONVERT_SPACES |
+					(mail_config_get_citation_highlight () ? E_TEXT_TO_HTML_MARK_CITATION : 0),
+					mail_config_get_citation_color ());
 	gtk_html_write (html, stream, "<tt>", 4);
 	gtk_html_write (html, stream, htmltext, strlen (htmltext));
 	gtk_html_write (html, stream, "</tt>", 5);
@@ -823,6 +968,7 @@ mail_display_redisplay (MailDisplay *md, gboolean unscroll)
 {
 	printf("redisplaying\n");
 
+	md->last_active = NULL;
 	md->stream = gtk_html_begin (GTK_HTML (md->html));
 	if (!unscroll) {
 		/* This is a hack until there's a clean way to do this. */
@@ -832,7 +978,6 @@ mail_display_redisplay (MailDisplay *md, gboolean unscroll)
 	mail_html_write (md->html, md->stream, "%s%s", HTML_HEADER, "<BODY>\n");
 
 	if (md->current_message) {
-		camel_object_ref (CAMEL_OBJECT (md->current_message));
 		if (mail_config_get_view_source ())
 			mail_format_raw_message (md->current_message, md);
 		else
@@ -888,16 +1033,6 @@ mail_display_init (GtkObject *object)
 
 	/* various other initializations */
 	mail_display->current_message = NULL;
-	mail_display->thumbnail_cache = g_hash_table_new (g_str_hash, g_str_equal);
-}
-
-static void
-thumbnail_cache_free (gpointer key,
-		      gpointer value,
-		      gpointer user_data)
-{
-	g_free (key);
-	gdk_pixbuf_unref (value);
 }
 
 static void
@@ -905,9 +1040,6 @@ mail_display_destroy (GtkObject *object)
 {
 	MailDisplay *mail_display = MAIL_DISPLAY (object);
 
-	g_hash_table_foreach (mail_display->thumbnail_cache, 
-			      thumbnail_cache_free, NULL);
-	g_hash_table_destroy (mail_display->thumbnail_cache);
 	g_datalist_clear (mail_display->data);
 	g_free (mail_display->data);
 
@@ -919,6 +1051,158 @@ mail_display_class_init (GtkObjectClass *object_class)
 {
 	object_class->destroy = mail_display_destroy;
 	mail_display_parent_class = gtk_type_class (PARENT_TYPE);
+
+	thumbnail_cache = g_hash_table_new (g_str_hash, g_str_equal);
+}
+
+static void
+link_open_in_browser (GtkWidget *w, MailDisplay *mail_display)
+{
+	g_print ("FIXME\n");
+}
+
+static void
+link_save_as (GtkWidget *w, MailDisplay *mail_display)
+{
+	g_print ("FIXME\n");
+}
+
+static void
+link_copy_location (GtkWidget *w, MailDisplay *mail_display)
+{
+	g_print ("FIXME\n");
+}
+
+#define SEPARATOR  { "", NULL, (NULL), NULL,  0 }
+#define TERMINATOR { NULL, NULL, (NULL), NULL,  0 }
+
+static EPopupMenu link_menu [] = {
+	{ N_("Open link in browser (FIXME)"),    NULL,
+	  GTK_SIGNAL_FUNC (link_open_in_browser),  NULL,  0 },
+	{ N_("Save as (FIXME)"),                 NULL,
+	  GTK_SIGNAL_FUNC (link_save_as), NULL,  0 },
+	{ N_("Copy location (FIXME)"),                 NULL,
+	  GTK_SIGNAL_FUNC (link_copy_location), NULL,  0 },
+	
+	TERMINATOR
+};
+
+static int
+html_button_press_event (GtkWidget *widget, GdkEventButton *event, MailDisplay *mail_display)
+{
+	g_return_val_if_fail (widget != NULL, FALSE);
+	g_return_val_if_fail (event != NULL, FALSE);
+
+	if (event->type == GDK_BUTTON_PRESS) {
+		if (event->button == 3) {
+			HTMLEngine *e;
+			HTMLPoint *point;
+			GtkWidget *popup_thing;
+			const gchar *email;
+			const gchar *name;
+			const gchar *link;
+
+			e     = GTK_HTML (widget)->engine;
+			point = html_engine_get_point_at (e, event->x + e->x_offset, event->y + e->y_offset, FALSE);
+			if (point) {
+				email = (const gchar *) html_object_get_data (point->object, "email");
+				if (email) {
+					name = (const gchar *) html_object_get_data (point->object, "name");
+
+					popup_thing = bonobo_widget_new_control ("OAFIID:GNOME_Evolution_Addressbook_AddressPopup",
+										 CORBA_OBJECT_NIL);
+
+					bonobo_widget_set_property (BONOBO_WIDGET (popup_thing),
+								    "name", name,
+								    "email", email,
+								    NULL);
+					gtk_widget_show (popup_thing);
+
+				} else if ((link = html_object_get_url (point->object))) {
+					e_popup_menu_run (link_menu, (GdkEvent *) event, 0, 0, mail_display);
+				}
+				html_point_destroy (point);
+			}
+			return TRUE;
+		}
+	}
+		
+	return FALSE;
+}
+
+static inline void
+set_underline (HTMLEngine *e, HTMLObject *o, gboolean underline)
+{
+	HTMLText *text = HTML_TEXT (o);
+
+	html_text_set_font_style (text, e, underline
+				  ? html_text_get_font_style (text) | GTK_HTML_FONT_STYLE_UNDERLINE
+				  : html_text_get_font_style (text) & ~GTK_HTML_FONT_STYLE_UNDERLINE);
+	html_engine_queue_draw (e, o);
+}
+
+static void
+update_active (GtkWidget *widget, gint x, gint y, MailDisplay *mail_display)
+{
+	HTMLEngine *e;
+	HTMLPoint *point;
+	const gchar *email;
+
+	e = GTK_HTML (widget)->engine;
+
+	point = html_engine_get_point_at (e, x + e->x_offset, y + e->y_offset, FALSE);
+	if (mail_display->last_active && (!point || mail_display->last_active != point->object)) {
+		set_underline (e, HTML_OBJECT (mail_display->last_active), FALSE);
+		mail_display->last_active = NULL;
+	}
+	if (point) {
+		email = (const gchar *) html_object_get_data (point->object, "email");
+		if (email && html_object_is_text (point->object)) {
+			set_underline (e, point->object, TRUE);
+			mail_display->last_active = point->object;
+		}
+		html_point_destroy (point);
+	}
+}
+
+static gint
+html_enter_notify_event (GtkWidget *widget, GdkEventCrossing *event, MailDisplay *mail_display)
+{
+	update_active (widget, event->x, event->y, mail_display);
+
+	return TRUE;
+}
+
+static gint
+html_motion_notify_event (GtkWidget *widget, GdkEventMotion *event, MailDisplay *mail_display)
+{
+	gint x, y;
+
+	g_return_val_if_fail (widget != NULL, 0);
+	g_return_val_if_fail (GTK_IS_HTML (widget), 0);
+	g_return_val_if_fail (event != NULL, 0);
+
+	if (event->is_hint)
+		gdk_window_get_pointer (GTK_LAYOUT (widget)->bin_window, &x, &y, NULL);
+	else {
+		x = event->x;
+		y = event->y;
+	}
+
+	update_active (widget, x, y, mail_display);
+
+	return TRUE;
+}
+
+static void
+html_iframe_created (GtkWidget *w, GtkHTML *iframe, MailDisplay *mail_display)
+{
+	gtk_signal_connect (GTK_OBJECT (iframe), "button_press_event",
+			    GTK_SIGNAL_FUNC (html_button_press_event), mail_display);
+	gtk_signal_connect (GTK_OBJECT (iframe), "motion_notify_event",
+			    GTK_SIGNAL_FUNC (html_motion_notify_event), mail_display);
+	gtk_signal_connect (GTK_OBJECT (iframe), "enter_notify_event",
+			    GTK_SIGNAL_FUNC (html_enter_notify_event), mail_display);
 }
 
 GtkWidget *
@@ -951,6 +1235,14 @@ mail_display_new (void)
 	gtk_signal_connect (GTK_OBJECT (html), "link_clicked",
 			    GTK_SIGNAL_FUNC (on_link_clicked),
 			    mail_display);
+	gtk_signal_connect (GTK_OBJECT (html), "button_press_event",
+			    GTK_SIGNAL_FUNC (html_button_press_event), mail_display);
+	gtk_signal_connect (GTK_OBJECT (html), "motion_notify_event",
+			    GTK_SIGNAL_FUNC (html_motion_notify_event), mail_display);
+	gtk_signal_connect (GTK_OBJECT (html), "enter_notify_event",
+			    GTK_SIGNAL_FUNC (html_enter_notify_event), mail_display);
+	gtk_signal_connect (GTK_OBJECT (html), "iframe_created",
+			    GTK_SIGNAL_FUNC (html_iframe_created), mail_display);
 
 	gtk_container_add (GTK_CONTAINER (scroll), html);
 	gtk_widget_show (GTK_WIDGET (html));
@@ -958,6 +1250,7 @@ mail_display_new (void)
 	mail_display->scroll = E_SCROLL_FRAME (scroll);
 	mail_display->html = GTK_HTML (html);
 	mail_display->stream = NULL;
+	mail_display->last_active = NULL;
 	mail_display->data = g_new0 (GData *, 1);
 	g_datalist_init (mail_display->data);
 
