@@ -26,11 +26,11 @@ struct _MailDisplay {
 	EScrollFrame *scroll;
 	GtkHTML *html;
 	GtkHTMLStream *stream;
+	gpointer last_active;
 	guint idle_id;
 	
 	CamelMimeMessage *current_message;
 	GData **data;
-	GHashTable *thumbnail_cache;
 };
 
 typedef struct {
