@@ -3,6 +3,7 @@
 #include <unistd.h>
 
 #include "e-util/e-msgport.h"
+#include "camel/camel-operation.h"
 #include <glib.h>
 #include <pthread.h>
 
@@ -16,13 +17,36 @@
 
 #include "folder-browser-factory.h"
 
+#include <libgnomeui/gnome-dialog.h>
+#include <gtk/gtkprogress.h>
+
+/*#define MALLOC_CHECK*/
 #define d(x) 
 
 static void set_view_data(const char *current_message, int busy);
+static void set_stop(int sensitive);
+static void mail_enable_stop(void);
+static void mail_disable_stop(void);
+static void mail_operation_status(struct _CamelOperation *op, const char *what, int pc, void *data);
 
 #define MAIL_MT_LOCK(x) pthread_mutex_lock(&x)
 #define MAIL_MT_UNLOCK(x) pthread_mutex_unlock(&x)
 
+/* background operation status stuff */
+struct _mail_msg_priv {
+	GtkProgressBar *bar;
+	GtkLabel *label;
+
+	/* for pending requests, before timeout_id is activated (then bar will be ! NULL) */
+	char *what;
+	int pc;
+	int timeout_id;
+};
+
+static GtkWindow *progress_dialogue;
+static int progress_row;
+
+/* mail_msg stuff */
 static unsigned int mail_msg_seq; /* sequence number of each message */
 static GHashTable *mail_msg_active; /* table of active messages, must hold mail_msg_lock to access */
 static pthread_mutex_t mail_msg_lock = PTHREAD_MUTEX_INITIALIZER;
@@ -40,19 +64,63 @@ void *mail_msg_new(mail_msg_op_t *ops, EMsgPort *reply_port, size_t size)
 	msg->ops = ops;
 	msg->seq = mail_msg_seq++;
 	msg->msg.reply_port = reply_port;
-	msg->cancel = camel_cancel_new();
+	msg->cancel = camel_operation_new(mail_operation_status, (void *)msg->seq);
 	camel_exception_init(&msg->ex);
+	msg->priv = g_malloc0(sizeof(*msg->priv));
 
 	g_hash_table_insert(mail_msg_active, (void *)msg->seq, msg);
 
+	d(printf("New message %p\n", msg));
+
 	MAIL_MT_UNLOCK(mail_msg_lock);
 
 	return msg;
 }
 
+/* either destroy the progress (in event_data), or the whole dialogue (in data) */
+static void destroy_widgets(CamelObject *o, void *event_data, void *data)
+{
+	if (data)
+		gtk_widget_destroy((GtkWidget *)data);
+	if (event_data)
+		gtk_widget_destroy((GtkWidget *)event_data);
+}
+
+#ifdef MALLOC_CHECK
+#include <mcheck.h>
+
+static void
+checkmem(void *p)
+{
+	if (p) {
+		int status = mprobe(p);
+
+		switch (status) {
+		case MCHECK_HEAD:
+			printf("Memory underrun at %p\n", p);
+			abort();
+		case MCHECK_TAIL:
+			printf("Memory overrun at %p\n", p);
+			abort();
+		case MCHECK_FREE:
+			printf("Double free %p\n", p);
+			abort();
+		}
+	}
+}
+#endif
+
 void mail_msg_free(void *msg)
 {
 	struct _mail_msg *m = msg;
+	void *bar = NULL, *label = NULL;
+
+#ifdef MALLOC_CHECK
+	checkmem(m);
+	checkmem(m->cancel);
+	checkmem(m->priv);
+#endif
+	d(printf("Free message %p\n", msg));
 
 	if (m->ops->destroy_msg)
 		m->ops->destroy_msg(m);
@@ -62,11 +130,31 @@ void mail_msg_free(void *msg)
 	g_hash_table_remove(mail_msg_active, (void *)m->seq);
 	pthread_cond_broadcast(&mail_msg_cond);
 
+	/* this closes the bar, and/or the whole progress dialogue, once we're out of things to do */
+	if (g_hash_table_size(mail_msg_active) == 0) {
+		if (progress_dialogue != NULL) {
+			bar = progress_dialogue;
+			progress_dialogue = NULL;
+			progress_row = 0;
+		}
+	} else if (m->priv->bar) {
+		bar = m->priv->bar;
+		label = m->priv->label;
+	}
+
+	if (m->priv->timeout_id > 0)
+		gtk_timeout_remove(m->priv->timeout_id);
+
 	MAIL_MT_UNLOCK(mail_msg_lock);
 
-	camel_cancel_unref(m->cancel);
+	camel_operation_unref(m->cancel);
 	camel_exception_clear(&m->ex);
+	g_free(m->priv->what);
+	g_free(m->priv);
 	g_free(m);
+
+	if (bar || label)
+		mail_proxy_event(destroy_widgets, NULL, bar, label);
 }
 
 void mail_msg_check_error(void *msg)
@@ -76,6 +164,12 @@ void mail_msg_check_error(void *msg)
 	char *text;
 	GnomeDialog *gd;
 
+#ifdef MALLOC_CHECK
+	checkmem(m);
+	checkmem(m->cancel);
+	checkmem(m->priv);
+#endif
+
 	if (!camel_exception_is_set(&m->ex)
 	    || m->ex.id == CAMEL_EXCEPTION_USER_CANCEL)
 		return;
@@ -101,7 +195,7 @@ void mail_msg_cancel(unsigned int msgid)
 	m = g_hash_table_lookup(mail_msg_active, (void *)msgid);
 
 	if (m)
-		camel_cancel_cancel(m->cancel);
+		camel_operation_cancel(m->cancel);
 
 	MAIL_MT_UNLOCK(mail_msg_lock);
 }
@@ -151,6 +245,13 @@ mail_msgport_replied(GIOChannel *source, GIOCondition cond, void *d)
 	mail_msg_t *m;
 
 	while (( m = (mail_msg_t *)e_msgport_get(port))) {
+
+#ifdef MALLOC_CHECK
+		checkmem(m);
+		checkmem(m->cancel);
+		checkmem(m->priv);
+#endif
+
 		if (m->ops->reply_msg)
 			m->ops->reply_msg(m);
 		mail_msg_check_error(m);
@@ -169,6 +270,11 @@ mail_msgport_received(GIOChannel *source, GIOCondition cond, void *d)
 	mail_msg_t *m;
 
 	while (( m = (mail_msg_t *)e_msgport_get(port))) {
+#ifdef MALLOC_CHECK
+		checkmem(m);
+		checkmem(m->cancel);
+		checkmem(m->priv);
+#endif
 		if (m->ops->describe_msg) {
 			char *text = m->ops->describe_msg(m, FALSE);
 			mail_status_start(text);
@@ -195,6 +301,12 @@ mail_msg_destroy(EThread *e, EMsg *msg, void *data)
 {
 	mail_msg_t *m = (mail_msg_t *)msg;
 
+#ifdef MALLOC_CHECK
+	checkmem(m);
+	checkmem(m->cancel);
+	checkmem(m->priv);
+#endif	
+
 	if (m->ops->describe_msg)
 		mail_status_end();
 	mail_msg_free(m);
@@ -205,6 +317,12 @@ mail_msg_received(EThread *e, EMsg *msg, void *data)
 {
 	mail_msg_t *m = (mail_msg_t *)msg;
 
+#ifdef MALLOC_CHECK
+	checkmem(m);
+	checkmem(m->cancel);
+	checkmem(m->priv);
+#endif
+
 	if (m->ops->describe_msg) {
 		char *text = m->ops->describe_msg(m, FALSE);
 		d(printf("message received at thread\n"));
@@ -212,8 +330,11 @@ mail_msg_received(EThread *e, EMsg *msg, void *data)
 		g_free(text);
 	}
 
-	if (m->ops->receive_msg)
+	if (m->ops->receive_msg) {
+		mail_enable_stop();
 		m->ops->receive_msg(m);
+		mail_disable_stop();
+	}
 }
 
 static void mail_msg_cleanup(void)
@@ -437,7 +558,7 @@ void mail_statusf(const char *fmt, ...)
 
 struct _pass_msg {
 	struct _mail_msg msg;
-	char *prompt;
+	const char *prompt;
 	int secret;
 	char *result;
 };
@@ -492,7 +613,7 @@ struct _mail_msg_op get_pass_op = {
 
 /* returns the password, or NULL if cancelled */
 char *
-mail_get_password(char *prompt, gboolean secret)
+mail_get_password(const char *prompt, gboolean secret)
 {
 	char *ret;
 	struct _pass_msg *m, *r;
@@ -531,6 +652,95 @@ mail_get_password(char *prompt, gboolean secret)
 
 /* ******************** */
 
+/* ********************************************************************** */
+
+struct _accept_msg {
+	struct _mail_msg msg;
+	const char *prompt;
+	gboolean result;
+};
+
+static void
+do_get_accept (struct _mail_msg *mm)
+{
+	struct _accept_msg *m = (struct _accept_msg *)mm;
+	GtkWidget *dialog;
+	GtkWidget *label;
+	
+	dialog = gnome_dialog_new (_("Do you accept?"),
+				   GNOME_STOCK_BUTTON_YES,
+				   GNOME_STOCK_BUTTON_NO,
+				   NULL);
+	gnome_dialog_set_default (GNOME_DIALOG (dialog), 1);
+	gtk_window_set_policy (GTK_WINDOW (dialog), TRUE, TRUE, TRUE);
+	
+	label = gtk_label_new (m->prompt);
+	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+	
+	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), label,
+			    TRUE, TRUE, 0);
+	gtk_widget_show (label);
+	
+	/* hrm, we can't run this async since the gui_port from which we're called
+	   will reply to our message for us */
+	m->result = gnome_dialog_run_and_close (GNOME_DIALOG (dialog)) == 0;
+}
+
+static void
+do_free_accept (struct _mail_msg *mm)
+{
+	/*struct _accept_msg *m = (struct _accept_msg *)mm;*/
+	
+	/* nothing to do here */
+}
+
+struct _mail_msg_op get_accept_op = {
+	NULL,
+	do_get_accept,
+	NULL,
+	do_free_accept,
+};
+
+/* prompt the user with a yes/no question and return the response */
+gboolean
+mail_get_accept (const char *prompt)
+{
+	struct _accept_msg *m, *r;
+	EMsgPort *accept_reply;
+	gboolean accept;
+	
+	accept_reply = e_msgport_new ();
+	
+	m = mail_msg_new (&get_accept_op, accept_reply, sizeof (*m));
+	
+	m->prompt = prompt;
+	
+	if (pthread_self () == mail_gui_thread) {
+		do_get_accept ((struct _mail_msg *)m);
+		r = m;
+	} else {
+		static pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
+		
+		/* we want this single-threaded, this is the easiest way to do it without blocking ? */
+		pthread_mutex_lock (&lock);
+		e_msgport_put (mail_gui_port, (EMsg *)m);
+		e_msgport_wait (accept_reply);
+		r = (struct _accept_msg *)e_msgport_get (accept_reply);
+		pthread_mutex_unlock (&lock);
+	}
+	
+	g_assert (r == m);
+	
+	accept = m->result;
+	
+	mail_msg_free (m);
+	e_msgport_destroy (accept_reply);
+	
+	return accept;
+}
+
+/* ******************** */
+
 struct _proxy_msg {
 	struct _mail_msg msg;
 	CamelObjectEventHookFunc func;
@@ -578,6 +788,202 @@ int mail_proxy_event(CamelObjectEventHookFunc func, CamelObject *o, void *event_
 	}
 }
 
+/* ********************************************************************** */
+/* locked via status_lock */
+static int busy_state;
+
+static void do_set_busy(struct _mail_msg *mm)
+{
+	set_stop(busy_state > 0);
+}
+
+struct _mail_msg_op set_busy_op = {
+	NULL,
+	do_set_busy,
+	NULL,
+	NULL,
+};
+
+static void mail_enable_stop(void)
+{
+	struct _mail_msg *m;
+
+	MAIL_MT_LOCK(status_lock);
+	busy_state++;
+	if (busy_state == 1) {
+		m = mail_msg_new(&set_busy_op, NULL, sizeof(*m));
+		e_msgport_put(mail_gui_port, (EMsg *)m);
+	}
+	MAIL_MT_UNLOCK(status_lock);
+}
+
+static void mail_disable_stop(void)
+{
+	struct _mail_msg *m;
+
+	MAIL_MT_LOCK(status_lock);
+	busy_state--;
+	if (busy_state == 0) {
+		m = mail_msg_new(&set_busy_op, NULL, sizeof(*m));
+		e_msgport_put(mail_gui_port, (EMsg *)m);
+	}
+	MAIL_MT_UNLOCK(status_lock);
+}
+
+/* ******************************************************************************** */
+
+struct _op_status_msg {
+	struct _mail_msg msg;
+
+	struct _CamelOperation *op;
+	char *what;
+	int pc;
+	void *data;
+};
+
+GtkTable *progress_table;
+
+static int op_status_timeout(void *d)
+{
+	int id = (int)d;
+	struct _mail_msg *msg;
+	struct _mail_msg_priv *data;
+	
+	MAIL_MT_LOCK(mail_msg_lock);
+
+	msg = g_hash_table_lookup(mail_msg_active, (void *)id);
+	if (msg == NULL) {
+		MAIL_MT_UNLOCK(mail_msg_lock);
+		return FALSE;
+	}
+
+	data = msg->priv;
+
+	if (progress_dialogue == NULL) {
+		progress_dialogue = (GtkWindow *)gtk_window_new(GTK_WINDOW_DIALOG);
+		gtk_window_set_title(progress_dialogue, _("Evolution progress"));
+		gtk_window_set_policy(progress_dialogue, 0, 0, 1);
+		gtk_window_set_position(progress_dialogue, GTK_WIN_POS_CENTER);
+		progress_table = (GtkTable *)gtk_table_new(1, 2, FALSE);
+		gtk_container_add((GtkContainer *)progress_dialogue, (GtkWidget *)progress_table);
+	}
+
+	data->bar = (GtkProgressBar *)gtk_progress_bar_new();
+	gtk_progress_set_show_text((GtkProgress *)data->bar, TRUE);
+
+	gtk_progress_set_percentage((GtkProgress *)data->bar, (gfloat)(data->pc/100.0));
+	gtk_progress_set_format_string((GtkProgress *)data->bar, data->what);
+
+	if (msg->ops->describe_msg) {
+		char *desc = msg->ops->describe_msg(msg, FALSE);
+		data->label = (GtkLabel *)gtk_label_new(desc);
+		g_free(desc);
+	} else {
+		data->label = (GtkLabel *)gtk_label_new(_("Working"));
+	}
+
+	gtk_table_attach(progress_table, (GtkWidget *)data->label, 0, 1, progress_row, progress_row+1, GTK_EXPAND|GTK_FILL, 0, 3, 1);
+	gtk_table_attach(progress_table, (GtkWidget *)data->bar, 1, 2, progress_row, progress_row+1, GTK_EXPAND|GTK_FILL, 0, 3, 1);
+	progress_row++;
+	
+	gtk_widget_show_all((GtkWidget *)progress_table);
+	gtk_widget_show((GtkWidget *)progress_dialogue);
+
+	data->timeout_id = -1;
+
+	MAIL_MT_UNLOCK(mail_msg_lock);
+
+	return FALSE;
+}
+
+static void do_op_status(struct _mail_msg *mm)
+{
+	struct _op_status_msg *m = (struct _op_status_msg *)mm;
+	struct _mail_msg *msg;
+	struct _mail_msg_priv *data;
+	char *out, *p, *o, c;
+
+	g_assert(mail_gui_thread == pthread_self());
+
+	MAIL_MT_LOCK(mail_msg_lock);
+
+	msg = g_hash_table_lookup(mail_msg_active, m->data);
+	if (msg == NULL) {
+		MAIL_MT_UNLOCK(mail_msg_lock);
+		return;
+	}
+
+	data = msg->priv;
+
+	out = alloca(strlen(m->what)*2+1);
+	o = out;
+	p = m->what;
+	while ((c = *p++)) {
+		if (c=='%')
+			*o++ = '%';
+		*o++ = c;
+	}
+	*o = 0;
+
+	if (data->timeout_id == 0) {
+		data->what = g_strdup(out);
+		data->pc = m->pc;
+		data->timeout_id = gtk_timeout_add(2000, op_status_timeout, m->data);
+		MAIL_MT_UNLOCK(mail_msg_lock);
+		return;
+	}
+
+	if (data->bar == NULL) {
+		g_free(data->what);
+		data->what = g_strdup(out);
+		data->pc = m->pc;
+		MAIL_MT_UNLOCK(mail_msg_lock);
+		return;
+	}
+
+	gtk_progress_set_percentage((GtkProgress *)data->bar, (gfloat)(m->pc/100.0));
+	gtk_progress_set_format_string((GtkProgress *)data->bar, out);
+
+	MAIL_MT_UNLOCK(mail_msg_lock);
+}
+
+static void do_op_status_free(struct _mail_msg *mm)
+{
+	struct _op_status_msg *m = (struct _op_status_msg *)mm;
+
+	g_free(m->what);
+}
+
+struct _mail_msg_op op_status_op = {
+	NULL,
+	do_op_status,
+	NULL,
+	do_op_status_free,
+};
+
+static void
+mail_operation_status(struct _CamelOperation *op, const char *what, int pc, void *data)
+{
+	struct _op_status_msg *m;
+
+	d(printf("got operation statys: %s %d%%\n", what, pc));
+
+	m = mail_msg_new(&op_status_op, NULL, sizeof(*m));
+	m->op = op;
+	m->what = g_strdup(what);
+	switch (pc) {
+	case CAMEL_OPERATION_START:
+		pc = 0;
+		break;
+	case CAMEL_OPERATION_END:
+		pc = 100;
+		break;
+	}
+	m->pc = pc;
+	m->data = data;
+	e_msgport_put(mail_gui_port, (EMsg *)m);
+}
+
 /* ******************** */
 
 /* FIXME FIXME FIXME This is a totally evil hack.  */
@@ -633,10 +1039,10 @@ set_view_data(const char *current_message, int busy)
 
 		if (shell_view_interface != CORBA_OBJECT_NIL) {
 			if ((current_message == NULL || current_message[0] == 0) && ! busy) {
-				printf("clearing msg\n");
+				d(printf("clearing msg\n"));
 				GNOME_Evolution_ShellView_unsetMessage (shell_view_interface, &ev);
 			} else {
-				printf("setting msg %s\n", current_message);
+				d(printf("setting msg %s\n", current_message ? current_message : "(null)"));
 				GNOME_Evolution_ShellView_setMessage (shell_view_interface,
 								      current_message?current_message:"",
 								      busy,
@@ -652,3 +1058,29 @@ set_view_data(const char *current_message, int busy)
 	}
 	gtk_object_unref(GTK_OBJECT(it));
 }
+
+static void
+set_stop(int sensitive)
+{
+	EList *controls;
+	EIterator *it;
+	static int last = FALSE;
+
+	if (last == sensitive)
+		return;
+
+	controls = folder_browser_factory_get_control_list ();
+	for (it = e_list_get_iterator (controls); e_iterator_is_valid (it); e_iterator_next (it)) {
+		BonoboControl *control;
+		BonoboUIComponent *uic;
+
+		control = BONOBO_CONTROL (e_iterator_get (it));
+		uic = bonobo_control_get_ui_component (control);
+		if (uic == CORBA_OBJECT_NIL || bonobo_ui_component_get_container(uic) == CORBA_OBJECT_NIL)
+			continue;
+
+		bonobo_ui_component_set_prop(uic, "/commands/MailStop", "sensitive", sensitive?"1":"0", NULL);
+	}
+	gtk_object_unref(GTK_OBJECT(it));
+	last = sensitive;
+}
