@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000 Helix Code Inc.
+ *  Copyright (C) 2000, 2001 Ximian, Inc.
  *
  *  Authors: Not Zed <notzed@lostzed.mmc.com.au>
  *
@@ -72,6 +72,7 @@ mail_search_dialogue_construct (MailSearchDialogue *o, FilterRule *rule)
 	GnomeDialog *dialogue = GNOME_DIALOG (o);
 	
 	gtk_window_set_policy (GTK_WINDOW (dialogue), FALSE, TRUE, FALSE);
+	gtk_window_set_default_size (GTK_WINDOW (dialogue), 500, 400);
 	
 	o->context = rule_context_new ();
 	rule_context_add_part_set (o->context, "partset", filter_part_get_type (),
@@ -101,7 +102,12 @@ mail_search_dialogue_init (MailSearchDialogue *o)
 {
 	GnomeDialog *dialogue = GNOME_DIALOG (o);
 	
-	gnome_dialog_append_buttons (dialogue, _("Ok"), _("Search"), _("Cancel"), 0);
+	gnome_dialog_append_buttons (dialogue,
+				     GNOME_STOCK_BUTTON_OK,
+				     _("_Search"),
+				     GNOME_STOCK_BUTTON_CANCEL,
+				     NULL);
+	gnome_dialog_set_default (dialogue, 0);
 }
 
 
