@@ -24,7 +24,20 @@
  */
 
 #include <config.h>
-#include <gnome.h>
+
+#include <glib.h>
+#include <gtk/gtksignal.h>
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-exec.h>
+#include <libgnome/gnome-i18n.h>
+#include <libgnome/gnome-url.h>
+#include <libgnome/gnome-util.h>
+#include <libgnomeui/gnome-about.h>
+#include <libgnomeui/gnome-dialog.h>
+#include <libgnomeui/gnome-dialog-util.h>
+
+#include <liboaf/liboaf.h>
+#include <bonobo/bonobo-moniker-util.h>
 
 #include "e-shell-folder-creation-dialog.h"
 #include "e-shell-folder-selection-dialog.h"
@@ -34,8 +47,7 @@
 #include "e-shell-view-menu.h"
 #include "importer/importer.h"
 
-#include <liboaf/liboaf.h>
-#include <bonobo.h>
+#include "e-util/e-gui-utils.h"
 
 
 const char *authors[] = {
@@ -60,9 +72,11 @@ const char *authors[] = {
 	"Ettore Perazzoli",
 	"JP Rosevear",
 	"Jeffrey Stedfast",
+        "Jakub Steiner",
 	"Russell Steinthal",
 	"Peter Teichman",
 	"Chris Toshok",
+	"Jon Trowbridge",
 	"Peter Williams",
 	"Dan Winship",
 	"Michael Zucchi",
@@ -448,7 +462,7 @@ BonoboUIVerb new_verbs [] = {
 };
 
 BonoboUIVerb file_verbs [] = {
-	BONOBO_UI_VERB ("FileImporter", show_import_wizard),
+	BONOBO_UI_VERB ("FileImporter", (BonoboUIVerbFn) show_import_wizard),
 	BONOBO_UI_VERB ("FileGoToFolder", command_goto_folder),
 	BONOBO_UI_VERB ("FileCreateFolder", command_create_folder),
 	BONOBO_UI_VERB ("FileClose", command_close),
@@ -467,6 +481,14 @@ BonoboUIVerb help_verbs [] = {
 	BONOBO_UI_VERB_END
 };
 
+static EPixmap pixmaps [] = {
+	E_PIXMAP ("/menu/File/New/Folder",	"folder.xpm"),
+	E_PIXMAP ("/menu/File/Folder/Folder",	"folder.xpm"),
+	E_PIXMAP ("/menu/File/FileImporter",	"import.xpm"),
+	E_PIXMAP ("/menu/File/WorkOffLine",	"work_offline.xpm"),
+	E_PIXMAP_END
+};
+
 static void
 menu_do_misc (BonoboUIComponent *component,
 	      EShellView        *shell_view)
@@ -513,6 +535,8 @@ e_shell_view_menu_setup (EShellView *shell_view)
 
 	menu_do_misc (uic, shell_view);
 
+	e_pixmaps_update (uic, pixmaps);
+
 	gtk_signal_connect (GTK_OBJECT (shell_view), "shortcut_bar_mode_changed",
 			    GTK_SIGNAL_FUNC (shortcut_bar_mode_changed_cb),
 			    SHORTCUT_BAR_TOGGLE_PATH);
