@@ -24,10 +24,6 @@
 #ifndef _E_SHELL_VIEW_H_
 #define _E_SHELL_VIEW_H_
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <bonobo/bonobo-win.h>
 
 #include "e-shell.h"
