@@ -41,7 +41,7 @@ struct _EvolutionLocalStoragePrivate {
 
 
 enum {
-	SET_DISPLAY_NAME,
+	UPDATE_FOLDER,
 	LAST_SIGNAL
 };
 
@@ -50,13 +50,14 @@ static guint signals[LAST_SIGNAL] = { 0 };
 
 /* CORBA interface implementation.  */
 
-static POA_Evolution_LocalStorage__vepv LocalStorage_vepv;
+static POA_GNOME_Evolution_LocalStorage__vepv LocalStorage_vepv;
 
 static void
-impl_Evolution_LocalStorage_set_display_name (PortableServer_Servant servant,
-					      const CORBA_char *path,
-					      const CORBA_char *display_name,
-					      CORBA_Environment *ev)
+impl_GNOME_Evolution_LocalStorage_updateFolder (PortableServer_Servant servant,
+					   const CORBA_char *path,
+					   const CORBA_char *display_name,
+					   CORBA_boolean highlighted,
+					   CORBA_Environment *ev)
 {
 	BonoboObject *bonobo_object;
 	EvolutionLocalStorage *local_storage;
@@ -64,21 +65,21 @@ impl_Evolution_LocalStorage_set_display_name (PortableServer_Servant servant,
 	bonobo_object = bonobo_object_from_servant (servant);
 	local_storage = EVOLUTION_LOCAL_STORAGE (bonobo_object);
 
-	gtk_signal_emit (GTK_OBJECT (local_storage), signals[SET_DISPLAY_NAME], path, display_name);
+	gtk_signal_emit (GTK_OBJECT (local_storage), signals[UPDATE_FOLDER], path, display_name, highlighted);
 }
 
-static POA_Evolution_LocalStorage *
+static POA_GNOME_Evolution_LocalStorage *
 create_servant (void)
 {
-	POA_Evolution_LocalStorage *servant;
+	POA_GNOME_Evolution_LocalStorage *servant;
 	CORBA_Environment ev;
 
-	servant = (POA_Evolution_LocalStorage *) g_new0 (BonoboObjectServant, 1);
+	servant = (POA_GNOME_Evolution_LocalStorage *) g_new0 (BonoboObjectServant, 1);
 	servant->vepv = &LocalStorage_vepv;
 
 	CORBA_exception_init (&ev);
 
-	POA_Evolution_LocalStorage__init ((PortableServer_Servant) servant, &ev);
+	POA_GNOME_Evolution_LocalStorage__init ((PortableServer_Servant) servant, &ev);
 	if (ev._major != CORBA_NO_EXCEPTION) {
 		g_free (servant);
 		CORBA_exception_free (&ev);
@@ -111,7 +112,7 @@ impl_destroy (GtkObject *object)
 static void
 corba_class_init (void)
 {
-	POA_Evolution_LocalStorage__vepv *vepv;
+	POA_GNOME_Evolution_LocalStorage__vepv *vepv;
 	PortableServer_ServantBase__epv *base_epv;
 
 	base_epv = g_new0 (PortableServer_ServantBase__epv, 1);
@@ -120,9 +121,10 @@ corba_class_init (void)
 	base_epv->default_POA = NULL;
 
 	vepv = &LocalStorage_vepv;
-	vepv->Bonobo_Unknown_epv         = bonobo_object_get_epv ();
-	vepv->Evolution_Storage_epv      = evolution_storage_get_epv ();
-	vepv->Evolution_LocalStorage_epv = evolution_local_storage_get_epv ();
+	vepv->_base_epv                        = base_epv;
+	vepv->Bonobo_Unknown_epv               = bonobo_object_get_epv ();
+	vepv->GNOME_Evolution_Storage_epv      = evolution_storage_get_epv ();
+	vepv->GNOME_Evolution_LocalStorage_epv = evolution_local_storage_get_epv ();
 }
 
 static void
@@ -135,15 +137,16 @@ class_init (EvolutionLocalStorageClass *klass)
 
 	parent_class = gtk_type_class (PARENT_TYPE);
 
-	signals[SET_DISPLAY_NAME] = gtk_signal_new ("set_display_name",
-						    GTK_RUN_FIRST,
-						    object_class->type,
-						    GTK_SIGNAL_OFFSET (EvolutionLocalStorageClass,
-								       set_display_name),
-						    gtk_marshal_NONE__POINTER_POINTER,
-						    GTK_TYPE_NONE, 2,
-						    GTK_TYPE_STRING,
-						    GTK_TYPE_STRING);
+	signals[UPDATE_FOLDER] = gtk_signal_new ("update_folder",
+						 GTK_RUN_FIRST,
+						 object_class->type,
+						 GTK_SIGNAL_OFFSET (EvolutionLocalStorageClass,
+								    update_folder),
+						 e_marshal_NONE__POINTER_POINTER_INT,
+						 GTK_TYPE_NONE, 3,
+						 GTK_TYPE_STRING,
+						 GTK_TYPE_STRING,
+						 GTK_TYPE_BOOL);
 
 	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
 
@@ -161,20 +164,20 @@ init (EvolutionLocalStorage *local_storage)
 }
 
 
-POA_Evolution_LocalStorage__epv *
+POA_GNOME_Evolution_LocalStorage__epv *
 evolution_local_storage_get_epv (void)
 {
-	POA_Evolution_LocalStorage__epv *epv;
+	POA_GNOME_Evolution_LocalStorage__epv *epv;
 
-	epv = g_new0 (POA_Evolution_LocalStorage__epv, 1);
-	epv->set_display_name = impl_Evolution_LocalStorage_set_display_name;
+	epv = g_new0 (POA_GNOME_Evolution_LocalStorage__epv, 1);
+	epv->updateFolder = impl_GNOME_Evolution_LocalStorage_updateFolder;
 
 	return epv;
 }
 
 void
 evolution_local_storage_construct (EvolutionLocalStorage *local_storage,
-				   Evolution_LocalStorage corba_object,
+				   GNOME_Evolution_LocalStorage corba_object,
 				   const char *name)
 {
 	g_return_if_fail (local_storage != NULL);
@@ -183,15 +186,15 @@ evolution_local_storage_construct (EvolutionLocalStorage *local_storage,
 	g_return_if_fail (name != NULL);
 	g_return_if_fail (name[0] != '\0');
 
-	evolution_storage_construct (EVOLUTION_STORAGE (local_storage), corba_object, name);
+	evolution_storage_construct (EVOLUTION_STORAGE (local_storage), corba_object, name, NULL, NULL);
 }
 
 EvolutionLocalStorage *
 evolution_local_storage_new (const char *name)
 {
 	EvolutionLocalStorage *new;
-	POA_Evolution_LocalStorage *servant;
-	Evolution_LocalStorage corba_object;
+	POA_GNOME_Evolution_LocalStorage *servant;
+	GNOME_Evolution_LocalStorage corba_object;
 
 	g_return_val_if_fail (name != NULL, NULL);
 	g_return_val_if_fail (name[0] != '\0', NULL);
