@@ -38,6 +38,8 @@
 #include "evolution-shell-component-client.h"
 
 
+char *evolution_debug_log;
+
 #define PARENT_TYPE BONOBO_OBJECT_CLIENT_TYPE
 static BonoboObjectClass *parent_class = NULL;
 
@@ -47,6 +49,9 @@ struct _EvolutionShellComponentClientPrivate {
 
 	GNOME_Evolution_ShellComponentListener listener_interface;
 	PortableServer_Servant listener_servant;
+
+	GNOME_Evolution_ShellComponentDnd_SourceFolder dnd_source_folder_interface;
+	GNOME_Evolution_ShellComponentDnd_DestinationFolder dnd_destination_folder_interface;
 };
 
 
@@ -124,6 +129,16 @@ dispatch_callback (EvolutionShellComponentClient *shell_component_client,
 	(* callback) (shell_component_client, result, callback_data);
 }
 
+static EvolutionShellComponentResult
+shell_component_result_from_corba_exception (const CORBA_Environment *ev)
+{
+	if (ev->_major == CORBA_NO_EXCEPTION)
+		return EVOLUTION_SHELL_COMPONENT_OK;
+	if (ev->_major == CORBA_SYSTEM_EXCEPTION)
+		return EVOLUTION_SHELL_COMPONENT_CORBAERROR;
+	return EVOLUTION_SHELL_COMPONENT_CORBAERROR; /* FIXME? */
+}
+
 
 /* CORBA listener interface implementation.  */
 
@@ -259,6 +274,7 @@ impl_destroy (GtkObject *object)
 {
 	EvolutionShellComponentClient *shell_component_client;
 	EvolutionShellComponentClientPrivate *priv;
+	CORBA_Environment ev;
 
 	shell_component_client = EVOLUTION_SHELL_COMPONENT_CLIENT (object);
 	priv = shell_component_client->priv;
@@ -266,6 +282,20 @@ impl_destroy (GtkObject *object)
 	if (priv->callback != NULL)
 		dispatch_callback (shell_component_client, EVOLUTION_SHELL_COMPONENT_INTERRUPTED);
 
+	CORBA_exception_init (&ev);
+
+	if (priv->dnd_source_folder_interface != CORBA_OBJECT_NIL) {
+		Bonobo_Unknown_unref (priv->dnd_source_folder_interface, &ev);
+		CORBA_Object_release (priv->dnd_source_folder_interface, &ev);
+	}
+
+	if (priv->dnd_destination_folder_interface != CORBA_OBJECT_NIL) {
+		Bonobo_Unknown_unref (priv->dnd_destination_folder_interface, &ev);
+		CORBA_Object_release (priv->dnd_destination_folder_interface, &ev);
+	}
+
+	CORBA_exception_free (&ev);
+
 	g_free (priv);
 
 	(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
@@ -289,10 +319,15 @@ init (EvolutionShellComponentClient *shell_component_client)
 	EvolutionShellComponentClientPrivate *priv;
 
 	priv = g_new (EvolutionShellComponentClientPrivate, 1);
-	priv->listener_interface = CORBA_OBJECT_NIL;
-	priv->listener_servant   = NULL;
-	priv->callback           = NULL;
-	priv->callback_data      = NULL;
+
+	priv->listener_interface               = CORBA_OBJECT_NIL;
+	priv->listener_servant                 = NULL;
+
+	priv->callback                         = NULL;
+	priv->callback_data                    = NULL;
+
+	priv->dnd_source_folder_interface      = CORBA_OBJECT_NIL;
+	priv->dnd_destination_folder_interface = CORBA_OBJECT_NIL;
 
 	shell_component_client->priv = priv;
 }
@@ -353,6 +388,69 @@ evolution_shell_component_client_new_for_objref (const GNOME_Evolution_ShellComp
 	return new;
 }
 
+
+/* Querying DnD interfaces.  */
+
+GNOME_Evolution_ShellComponentDnd_SourceFolder
+evolution_shell_component_client_get_dnd_source_interface (EvolutionShellComponentClient *shell_component_client)
+{
+	EvolutionShellComponentClientPrivate *priv;
+	GNOME_Evolution_ShellComponentDnd_SourceFolder interface;
+	CORBA_Environment ev;
+
+	g_return_val_if_fail (shell_component_client != NULL, CORBA_OBJECT_NIL);
+	g_return_val_if_fail (EVOLUTION_IS_SHELL_COMPONENT_CLIENT (shell_component_client), CORBA_OBJECT_NIL);
+
+	priv = shell_component_client->priv;
+
+	if (priv->dnd_source_folder_interface != CORBA_OBJECT_NIL)
+		return priv->dnd_source_folder_interface;
+
+	CORBA_exception_init (&ev);
+
+	interface = Bonobo_Unknown_queryInterface (bonobo_object_corba_objref (BONOBO_OBJECT (shell_component_client)),
+						   "IDL:GNOME/Evolution/ShellComponentDnd/SourceFolder:1.0",
+						   &ev);
+
+	if (ev._major != CORBA_NO_EXCEPTION)
+		interface = CORBA_OBJECT_NIL;
+
+	CORBA_exception_free (&ev);
+
+	priv->dnd_source_folder_interface = interface;
+	return interface;
+}
+
+GNOME_Evolution_ShellComponentDnd_DestinationFolder
+evolution_shell_component_client_get_dnd_destination_interface (EvolutionShellComponentClient *shell_component_client)
+{
+	EvolutionShellComponentClientPrivate *priv;
+	GNOME_Evolution_ShellComponentDnd_DestinationFolder interface;
+	CORBA_Environment ev;
+
+	g_return_val_if_fail (shell_component_client != NULL, CORBA_OBJECT_NIL);
+	g_return_val_if_fail (EVOLUTION_IS_SHELL_COMPONENT_CLIENT (shell_component_client), CORBA_OBJECT_NIL);
+
+	priv = shell_component_client->priv;
+
+	if (priv->dnd_destination_folder_interface != CORBA_OBJECT_NIL)
+		return priv->dnd_destination_folder_interface;
+
+	CORBA_exception_init (&ev);
+
+	interface = Bonobo_Unknown_queryInterface (bonobo_object_corba_objref (BONOBO_OBJECT (shell_component_client)),
+						   "IDL:GNOME/Evolution/ShellComponentDnd/DestinationFolder:1.0",
+						   &ev);
+
+	if (ev._major != CORBA_NO_EXCEPTION)
+		interface = CORBA_OBJECT_NIL;
+
+	CORBA_exception_free (&ev);
+
+	priv->dnd_destination_folder_interface = interface;
+	return interface;
+}
+
 
 /* Synchronous operations.  */
 
@@ -375,6 +473,9 @@ evolution_shell_component_client_set_owner (EvolutionShellComponentClient *shell
 
 	result = corba_exception_to_result (&ev);
 
+	if (result == EVOLUTION_SHELL_COMPONENT_OK && evolution_debug_log)
+		GNOME_Evolution_ShellComponent_debug (bonobo_object_corba_objref (BONOBO_OBJECT (shell_component_client)), evolution_debug_log, &ev);
+
 	CORBA_exception_free (&ev);
 
 	return result;
@@ -488,6 +589,14 @@ evolution_shell_component_client_async_create_folder (EvolutionShellComponentCli
 							  physical_uri, type,
 							  &ev);
 
+	if (ev._major != CORBA_NO_EXCEPTION && priv->callback != NULL) {
+		(* callback) (shell_component_client,
+			      shell_component_result_from_corba_exception (&ev),
+			      data);
+		priv->callback = NULL;
+		priv->callback_data = NULL;
+	}
+
 	CORBA_exception_free (&ev);
 }
 
@@ -500,6 +609,58 @@ evolution_shell_component_client_async_remove_folder (EvolutionShellComponentCli
 	/* FIXME to do. */
 }
 
+void
+evolution_shell_component_client_async_xfer_folder (EvolutionShellComponentClient *shell_component_client,
+						    const char *source_physical_uri,
+						    const char *destination_physical_uri,
+						    gboolean remove_source,
+						    EvolutionShellComponentClientCallback callback,
+						    void *data)
+{
+	EvolutionShellComponentClientPrivate *priv;
+	GNOME_Evolution_ShellComponent corba_shell_component;
+	CORBA_Environment ev;
+	
+	g_return_if_fail (shell_component_client != NULL);
+	g_return_if_fail (EVOLUTION_IS_SHELL_COMPONENT_CLIENT (shell_component_client));
+	g_return_if_fail (source_physical_uri != NULL);
+	g_return_if_fail (destination_physical_uri != NULL);
+	g_return_if_fail (data != NULL);
+
+	priv = shell_component_client->priv;
+
+	if (priv->callback != NULL) {
+		(* callback) (shell_component_client, EVOLUTION_SHELL_COMPONENT_BUSY, data);
+		return;
+	}
+
+	create_listener_interface (shell_component_client);
+
+	CORBA_exception_init (&ev);
+
+	corba_shell_component = bonobo_object_corba_objref (BONOBO_OBJECT (shell_component_client));
+
+	priv->callback      = callback;
+	priv->callback_data = data;
+
+	GNOME_Evolution_ShellComponent_xferFolderAsync (corba_shell_component,
+							priv->listener_interface,
+							source_physical_uri,
+							destination_physical_uri,
+							remove_source,
+							&ev);
+
+	if (ev._major != CORBA_NO_EXCEPTION && priv->callback != NULL) {
+		(* callback) (shell_component_client,
+			      shell_component_result_from_corba_exception (&ev),
+			      data);
+		priv->callback = NULL;
+		priv->callback_data = NULL;
+	}
+
+	CORBA_exception_free (&ev);
+}
+
 void
 evolution_shell_component_client_populate_folder_context_menu (EvolutionShellComponentClient *shell_component_client,
 							       BonoboUIComponent *uih,
