@@ -24,7 +24,7 @@
 #ifndef EVOLUTION_SHELL_COMPONENT_H
 #define EVOLUTION_SHELL_COMPONENT_H
 
-#include <bonobo/bonobo-object.h>
+#include <bonobo/bonobo-xobject.h>
 #include <bonobo/bonobo-control.h>
 
 #include "Evolution.h"
@@ -83,32 +83,45 @@ typedef void (* EvolutionShellComponentRemoveFolderFn) (EvolutionShellComponent
 						        const char *physical_uri,
 						        const GNOME_Evolution_ShellComponentListener listener,
 						        void *closure);
-typedef void (* EvolutionShellComponentCopyFolderFn)   (EvolutionShellComponent *shell_component,
+typedef void (* EvolutionShellComponentXferFolderFn)   (EvolutionShellComponent *shell_component,
 							const char *source_physical_uri,
 							const char *destination_physical_uri,
 							gboolean remove_source,
 						        const GNOME_Evolution_ShellComponentListener listener,
 						        void *closure);
-typedef void (* EvolutionShellComponentPopulateFolderContextMenu) (EvolutionShellComponent *shell_component,
+typedef void (* EvolutionShellComponentPopulateFolderContextMenuFn) (EvolutionShellComponent *shell_component,
 								   BonoboUIComponent *uic,
 								   const char *physical_uri,
 								   const char *type,
 								   void *closure);
+typedef char * (* EvolutionShellComponentGetDndSelectionFn) (EvolutionShellComponent *shell_component,
+							     const char *physical_uri,
+							     int type,
+							     int *format_return,
+							     const char **selection_return,
+							     int *selection_length_return,
+							     void *closure);
 
 struct _EvolutionShellComponentFolderType {
 	char *name;
 	char *icon_name;
+
+	/* The following are NULL-terminated arrays.  */
+	char **accepted_dnd_types;
+	char **exported_dnd_types;
 };
 typedef struct _EvolutionShellComponentFolderType EvolutionShellComponentFolderType;
 
 struct _EvolutionShellComponent {
-	BonoboObject parent;
+	BonoboXObject parent;
 
 	EvolutionShellComponentPrivate *priv;
 };
 
 struct _EvolutionShellComponentClass {
-	BonoboObjectClass parent_class;
+	BonoboXObjectClass parent_class;
+
+	POA_GNOME_Evolution_ShellComponent__epv epv;
 
 	/* Signals.  */
 
@@ -116,27 +129,29 @@ struct _EvolutionShellComponentClass {
 			      EvolutionShellClient *shell_client,
 			      const char *evolution_homedir);
 	void (* owner_unset) (EvolutionShellComponent *shell_component);
+	void (* debug)       (EvolutionShellComponent *shell_component);
 };
 
 
 GtkType                  evolution_shell_component_get_type   (void);
-void                     evolution_shell_component_construct  (EvolutionShellComponent                          *shell_component,
-							       const EvolutionShellComponentFolderType           folder_types[],
-							       GNOME_Evolution_ShellComponent                          corba_object,
-							       EvolutionShellComponentCreateViewFn               create_view_fn,
-							       EvolutionShellComponentCreateFolderFn             create_folder_fn,
-							       EvolutionShellComponentRemoveFolderFn             remove_folder_fn,
-							       EvolutionShellComponentCopyFolderFn               copy_folder_fn,
-							       EvolutionShellComponentPopulateFolderContextMenu  populate_folder_context_menu_fn,
-							       void                                             *closure);
-EvolutionShellComponent *evolution_shell_component_new        (const EvolutionShellComponentFolderType           folder_types[],
-							       EvolutionShellComponentCreateViewFn               create_view_fn,
-							       EvolutionShellComponentCreateFolderFn             create_folder_fn,
-							       EvolutionShellComponentRemoveFolderFn             remove_folder_fn,
-							       EvolutionShellComponentCopyFolderFn               copy_folder_fn,
-							       EvolutionShellComponentPopulateFolderContextMenu  populate_folder_context_menu_fn,
-							       void                                             *closure);
-EvolutionShellClient    *evolution_shell_component_get_owner  (EvolutionShellComponent                          *shell_component);
+void                     evolution_shell_component_construct  (EvolutionShellComponent                            *shell_component,
+							       const EvolutionShellComponentFolderType             folder_types[],
+							       EvolutionShellComponentCreateViewFn                 create_view_fn,
+							       EvolutionShellComponentCreateFolderFn               create_folder_fn,
+							       EvolutionShellComponentRemoveFolderFn               remove_folder_fn,
+							       EvolutionShellComponentXferFolderFn                 xfer_folder_fn,
+							       EvolutionShellComponentPopulateFolderContextMenuFn  populate_folder_context_menu_fn,
+							       EvolutionShellComponentGetDndSelectionFn            get_dnd_selection_fn,
+							       void                                               *closure);
+EvolutionShellComponent *evolution_shell_component_new        (const EvolutionShellComponentFolderType             folder_types[],
+							       EvolutionShellComponentCreateViewFn                 create_view_fn,
+							       EvolutionShellComponentCreateFolderFn               create_folder_fn,
+							       EvolutionShellComponentRemoveFolderFn               remove_folder_fn,
+							       EvolutionShellComponentXferFolderFn                 xfer_folder_fn,
+							       EvolutionShellComponentPopulateFolderContextMenuFn  populate_folder_context_menu_fn,
+							       EvolutionShellComponentGetDndSelectionFn            get_dnd_selection_fn,
+							       void                                               *closure);
+EvolutionShellClient    *evolution_shell_component_get_owner  (EvolutionShellComponent                            *shell_component);
 
 #ifdef cplusplus
 }
