@@ -31,6 +31,9 @@
 
 #include <config.h>
 #include <time.h>
+#include <glib.h>
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-i18n.h>
 #include "e-meeting-time-sel-item.h"
 #include "e-meeting-time-sel.h"
 
