@@ -240,7 +240,8 @@ enum {
 	ARG_MAXIMUM_DAYS_SELECTED,
 	ARG_DAYS_TO_START_WEEK_SELECTION,
 	ARG_MOVE_SELECTION_WHEN_MOVING,
-	ARG_ROUND_SELECTION_WHEN_MOVING
+	ARG_ROUND_SELECTION_WHEN_MOVING,
+	ARG_DISPLAY_POPUP
 };
 
 enum {
@@ -330,6 +331,9 @@ e_calendar_item_class_init (ECalendarItemClass *class)
 	gtk_object_add_arg_type ("ECalendarItem::round_selection_when_moving",
 				 GTK_TYPE_BOOL, GTK_ARG_READWRITE,
 				 ARG_ROUND_SELECTION_WHEN_MOVING);
+	gtk_object_add_arg_type ("ECalendarItem::display_popup",
+				 GTK_TYPE_BOOL, GTK_ARG_READWRITE,
+				 ARG_DISPLAY_POPUP);
 
 	e_calendar_item_signals[DATE_RANGE_CHANGED] =
 		gtk_signal_new ("date_range_changed",
@@ -393,11 +397,12 @@ e_calendar_item_init (ECalendarItem *calitem)
 	calitem->show_week_numbers = FALSE;
 	calitem->week_start_day = 0;
 	calitem->expand = TRUE;
-	calitem->max_days_selected = 42;
-	calitem->days_to_start_week_selection = 9;
+	calitem->max_days_selected = 1;
+	calitem->days_to_start_week_selection = -1;
 	calitem->move_selection_when_moving = TRUE;
 	calitem->round_selection_when_moving = FALSE;
-
+	calitem->display_popup = TRUE;
+	
 	calitem->x1 = 0.0;
 	calitem->y1 = 0.0;
 	calitem->x2 = 0.0;
@@ -503,10 +508,10 @@ e_calendar_item_get_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 		GTK_VALUE_BOOL (*arg) = calitem->show_week_numbers;
 		break;
 	case ARG_MAXIMUM_DAYS_SELECTED:
-		GTK_VALUE_INT (*arg) = calitem->max_days_selected;
+		GTK_VALUE_INT (*arg) = e_calendar_item_get_max_days_sel (calitem);
 		break;
 	case ARG_DAYS_TO_START_WEEK_SELECTION:
-		GTK_VALUE_INT (*arg) = calitem->days_to_start_week_selection;
+		GTK_VALUE_INT (*arg) = e_calendar_item_get_days_start_week_sel (calitem);
 		break;
 	case ARG_MOVE_SELECTION_WHEN_MOVING:
 		GTK_VALUE_BOOL (*arg) = calitem->move_selection_when_moving;
@@ -514,6 +519,11 @@ e_calendar_item_get_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 	case ARG_ROUND_SELECTION_WHEN_MOVING:
 		GTK_VALUE_BOOL (*arg) = calitem->round_selection_when_moving;
 		break;
+	case ARG_DISPLAY_POPUP:
+		GTK_VALUE_BOOL (*arg) = e_calendar_item_get_display_popup (calitem);
+		break;
+	default:
+		g_warning ("Invalid arg");
 	}
 }
 
@@ -639,12 +649,11 @@ e_calendar_item_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 		break;
 	case ARG_MAXIMUM_DAYS_SELECTED:
 		ivalue = GTK_VALUE_INT (*arg);
-		ivalue = MAX (1, ivalue);
-		calitem->max_days_selected = ivalue;
+		e_calendar_item_set_max_days_sel (calitem, ivalue);
 		break;
 	case ARG_DAYS_TO_START_WEEK_SELECTION:
 		ivalue = GTK_VALUE_INT (*arg);
-		calitem->days_to_start_week_selection = ivalue;
+		e_calendar_item_set_days_start_week_sel (calitem, ivalue);
 		break;
 	case ARG_MOVE_SELECTION_WHEN_MOVING:
 		bvalue = GTK_VALUE_BOOL (*arg);
@@ -654,6 +663,10 @@ e_calendar_item_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 		bvalue = GTK_VALUE_BOOL (*arg);
 		calitem->round_selection_when_moving = bvalue;
 		break;
+	case ARG_DISPLAY_POPUP:
+		bvalue = GTK_VALUE_BOOL (*arg);
+		e_calendar_item_set_display_popup (calitem, bvalue);
+		break;
 	default:
 		g_warning ("Invalid arg");
 	}
@@ -779,11 +792,18 @@ e_calendar_item_update		(GnomeCanvasItem *item,
 	width -= xthickness * 2;
 	height -= ythickness * 2;
 
-	rows = height / calitem->min_month_height;
+	if (calitem->min_month_height == 0)
+		rows = 1;
+	else
+		rows = height / calitem->min_month_height;
 	rows = MAX (rows, calitem->min_rows);
 	if (calitem->max_rows > 0)
 		rows = MIN (rows, calitem->max_rows);
-	cols = width / calitem->min_month_width;
+
+	if (calitem->min_month_width == 0)
+		cols = 1;
+	else
+		cols = width / calitem->min_month_width;
 	cols = MAX (cols, calitem->min_cols);
 	if (calitem->max_cols > 0)
 		cols = MIN (cols, calitem->max_cols);
@@ -1491,17 +1511,21 @@ e_calendar_item_recalc_sizes		(ECalendarItem *calitem)
 	canvas_item = GNOME_CANVAS_ITEM (calitem);
 	style = GTK_WIDGET (canvas_item->canvas)->style;
 
+	if (!style)
+		return;
+
 	font = calitem->font;
 	if (!font)
 		font = style->font;
 	wkfont = calitem->week_number_font;
 	if (!wkfont)
 		wkfont = font;
-	char_height = font->ascent + font->descent;
 
 	g_return_if_fail (font != NULL);
 	g_return_if_fail (wkfont != NULL);
 
+	char_height = font->ascent + font->descent;
+
 	/* If both fonts are the same, just return. */
 	if (font != calitem->old_font
 	    || wkfont != calitem->old_week_number_font) {
@@ -1620,7 +1644,8 @@ e_calendar_item_button_press	(ECalendarItem	*calitem,
 						      &all_week))
 		return FALSE;
 
-	if (event->button.button == 3 && day == -1) {
+	if (event->button.button == 3 && day == -1 
+	    && e_calendar_item_get_display_popup (calitem)) {
 		e_calendar_item_show_popup_menu (calitem,
 						 (GdkEventButton*) event,
 						 month_offset);
@@ -1630,6 +1655,9 @@ e_calendar_item_button_press	(ECalendarItem	*calitem,
 	if (event->button.button != 1 || day == -1)
 		return FALSE;
 
+	if (calitem->max_days_selected < 1)
+		return TRUE;
+
 	if (gnome_canvas_item_grab (GNOME_CANVAS_ITEM (calitem),
 				    GDK_POINTER_MOTION_MASK
 				    | GDK_BUTTON_RELEASE_MASK,
@@ -2147,6 +2175,54 @@ e_calendar_item_set_first_month(ECalendarItem	*calitem,
 	gnome_canvas_item_request_update (GNOME_CANVAS_ITEM (calitem));
 }
 
+/* Get the maximum number of days selectable */
+gint
+e_calendar_item_get_max_days_sel       (ECalendarItem	*calitem)
+{
+	return calitem->max_days_selected;
+}
+
+
+/* Set the maximum number of days selectable */
+void	 
+e_calendar_item_set_max_days_sel       (ECalendarItem	*calitem,
+					gint             days)
+{
+	calitem->max_days_selected = MAX (0, days);
+	gnome_canvas_item_request_update (GNOME_CANVAS_ITEM (calitem));
+}
+
+
+/* Get the maximum number of days before whole weeks are selected */
+gint     
+e_calendar_item_get_days_start_week_sel(ECalendarItem	*calitem)
+{
+	return calitem->days_to_start_week_selection;
+}
+
+
+/* Set the maximum number of days before whole weeks are selected */
+void	 
+e_calendar_item_set_days_start_week_sel(ECalendarItem	*calitem,
+					gint            days)
+{
+	calitem->days_to_start_week_selection = days;
+}
+
+gboolean 
+e_calendar_item_get_display_popup      (ECalendarItem	*calitem)
+{
+	return calitem->display_popup;
+}
+
+
+void	 
+e_calendar_item_set_display_popup      (ECalendarItem	*calitem,
+					gboolean        display)
+{
+	calitem->display_popup = display;
+}
+
 
 /* This will make sure that the given year & month are valid, i.e. if month
    is < 0 or > 11 the year and month will be updated accordingly. */
@@ -2196,8 +2272,11 @@ e_calendar_item_add_days_to_selection	(ECalendarItem	*calitem,
 }
 
 
-/* Returns the range of dates actually shown. Months are 0 to 11. */
-void
+/* Gets the range of dates actually shown. Months are 0 to 11.
+   This also includes the last days of the previous month and the first days
+   of the following month, which are normally shown in gray.
+   It returns FALSE if no dates are currently shown. */
+gboolean
 e_calendar_item_get_date_range	(ECalendarItem	*calitem,
 				 gint		*start_year,
 				 gint		*start_month,
@@ -2208,6 +2287,9 @@ e_calendar_item_get_date_range	(ECalendarItem	*calitem,
 {
 	gint first_day_offset, days_in_month, days_in_prev_month;
 
+	if (calitem->rows == 0 || calitem->cols == 0)
+		return FALSE;
+
 	/* Calculate the first day shown. This will be one of the greyed-out
 	   days before the first full month begins. */
 	e_calendar_item_get_month_info (calitem, 0, 0, &first_day_offset,
@@ -2231,6 +2313,8 @@ e_calendar_item_get_date_range	(ECalendarItem	*calitem,
 	*end_month %= 12;
 	*end_day = E_CALENDAR_ROWS_PER_MONTH * E_CALENDAR_COLS_PER_MONTH
 		- first_day_offset - days_in_month;
+
+	return TRUE;
 }
 
 
